Package: anansi
Type: Package
Title: Annotation-Based Analysis of Specific Interactions
Version: 1.3.0
Authors@R: c(
   person("Thomaz", "Bastiaanssen", email = "thomazbastiaanssen@gmail.com", 
       role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6891-734X")), 
   person("Thomas", "Quinn", email = "contacttomquinn@gmail.com", 
       role = c("aut"), comment = c(ORCID = "0000-0003-0286-6329")),
   person("Giulio", "Benedetti", email = "giulio.benedetti@utu.fi", 
       role = c("aut"), comment = c(ORCID = "0000-0002-8732-7692")), 
   person("Tuomas", "Borman", email = "tuomas.v.borman@utu.fi", 
       role = c("aut"), comment = c(ORCID = "0000-0002-8563-8884")), 
   person("Leo", "Lahti", email = "leo.lahti@utu.fi",
       role = c("aut"), comment = c(ORCID = "0000-0001-5537-637X"))    
   )
Description: Studies including both microbiome and metabolomics data are 
    becoming more common. Often, it would be helpful to integrate both datasets 
    in order to see if they corroborate each others patterns. All vs all 
    association is imprecise and likely to yield spurious associations. This
    package takes a knowledge-based approach to constrain association search 
    space, only considering metabolite-function pairs that have been recorded in
    a pathway database. This package also provides a framework to assess 
    differential association. 
Depends: R (>= 4.5.0)
Imports: S7, stats, methods, igraph, Matrix, forcats, S4Vectors,
        SummarizedExperiment, MultiAssayExperiment,
        SingleCellExperiment, TreeSummarizedExperiment, rlang, ggplot2,
        ggforce, patchwork, ggraph, tidygraph
Suggests: BiocStyle, dplyr, tidyr, graph, mia, KEGGREST, testthat (>=
        3.0.0), knitr, rmarkdown
License: GPL-3
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.3.3
Config/testthat/edition: 3
biocViews: Microbiome, Metabolomics, Regression, Pathways, KEGG
URL: https://github.com/thomazbastiaanssen/anansi,
        https://thomazbastiaanssen.github.io/anansi
BugReports: https://github.com/thomazbastiaanssen/anansi/issues
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/anansi
git_branch: devel
git_last_commit: 536c3f2
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:13:49 UTC; biocbuild
Author: Thomaz Bastiaanssen [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6891-734X>),
  Thomas Quinn [aut] (ORCID: <https://orcid.org/0000-0003-0286-6329>),
  Giulio Benedetti [aut] (ORCID: <https://orcid.org/0000-0002-8732-7692>),
  Tuomas Borman [aut] (ORCID: <https://orcid.org/0000-0002-8563-8884>),
  Leo Lahti [aut] (ORCID: <https://orcid.org/0000-0001-5537-637X>)
Maintainer: Thomaz Bastiaanssen <thomazbastiaanssen@gmail.com>

# Generated by roxygen2: do not edit by hand

S3method("[<-","anansi::MultiFactor")
S3method("[[<-","anansi::MultiFactor")
S3method(as.data.frame,"anansi::AnansiWeb")
S3method(as.list,"anansi::AnansiWeb")
S3method(as.list,"anansi::MultiFactor")
S3method(levels,"anansi::MultiFactor")
S3method(pairs,"anansi::AnansiWeb")
export("metadata<-")
export(AnansiTale)
export(AnansiWeb)
export(MultiFactor)
export(anansi)
export(asMAE)
export(asMultiFactor)
export(asTSE)
export(dictionary)
export(getEdgeList)
export(getFeaturePairs)
export(getGraph)
export(kegg_link)
export(krebsDemoWeb)
export(linkBiobakeryMap)
export(metadata)
export(pairwiseApply)
export(plotAnansi)
export(randomMultiFactor)
export(randomWeb)
export(tableX)
export(tableY)
export(unfactor)
export(weaveKEGG)
export(weaveWeb)
if (getRversion() < "4.3.0") importFrom("S7", "@")
import(S7)
import(methods)
importClassesFrom(Matrix,Matrix)
importClassesFrom(MultiAssayExperiment,MultiAssayExperiment)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importClassesFrom(TreeSummarizedExperiment,TreeSummarizedExperiment)
importFrom(Matrix,Matrix)
importFrom(Matrix,colSums)
importFrom(Matrix,drop0)
importFrom(Matrix,rowSums)
importFrom(Matrix,rsparsematrix)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,tcrossprod)
importFrom(Matrix,which)
importFrom(MultiAssayExperiment,ExperimentList)
importFrom(MultiAssayExperiment,MultiAssayExperiment)
importFrom(MultiAssayExperiment,experiments)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,isEmpty)
importFrom(S4Vectors,unfactor)
importFrom(S7,new_class)
importFrom(S7,props)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,altExp)
importFrom(SingleCellExperiment,altExpNames)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(TreeSummarizedExperiment,TreeSummarizedExperiment)
importFrom(forcats,lvls_expand)
importFrom(ggforce,facet_col)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_alpha_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggraph,geom_edge_link)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,geom_node_text)
importFrom(ggraph,ggraph)
importFrom(ggraph,scale_edge_colour_gradient2)
importFrom(ggraph,theme_graph)
importFrom(graphics,pairs)
importFrom(igraph,as_graphnel)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,shortest_paths)
importFrom(methods,getClass)
importFrom(methods,is)
importFrom(methods,show)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(rlang,.data)
importFrom(rlang,is_missing)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix.default)
importFrom(stats,p.adjust)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,terms.formula)
importFrom(stats,update.formula)
importFrom(tidygraph,tbl_graph)
importFrom(utils,data)
importMethodsFrom(Matrix,"%&%")
importMethodsFrom(S4Vectors,"metadata<-")
importMethodsFrom(S4Vectors,droplevels)
importMethodsFrom(S4Vectors,metadata)
importMethodsFrom(S4Vectors,unfactor)
importMethodsFrom(methods,show)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# Knowledge-based multi-modal integration using anansi <img src="man/figures/anansi_hex.png" align="right" width="120" alt="The anansi hex sticker" />

<!-- badges: start -->

[![GitHub
issues](https://img.shields.io/github/issues/thomazbastiaanssen/anansi)](https://github.com/thomazbastiaanssen/anansi/issues)
[![GitHub
pulls](https://img.shields.io/github/issues-pr/thomazbastiaanssen/anansi)](https://github.com/thomazbastiaanssen/anansi/pulls)
[![R
BiocCheck](https://github.com/thomazbastiaanssen/anansi/actions/workflows/test.yml/badge.svg)](https://github.com/thomazbastiaanssen/anansi/actions/workflows/test.yml)
[![Codecov test
coverage](https://codecov.io/gh/thomazbastiaanssen/anansi/graph/badge.svg)](https://app.codecov.io/gh/thomazbastiaanssen/anansi)
<!-- badges: end -->

## Introduction

The `anansi` package computes and compares the association between the
features of two ’omics data sets that are known to interact based on a
database such as KEGG. Studies including both functional microbiome and
metabolomics data are becoming more common. Often, it would be helpful
to integrate both data sets in order to see if they corroborate each
others patterns. However, all-vs-all association analyses are imprecise
and likely to yield spurious associations. This package takes a
knowledge-based approach to constrain association search space, only
considering metabolite-function interactions that have been recorded in
a pathway database. In addition, it provides a framework to assess
differential associations.

While `anansi` is geared towards metabolite-function interactions in the
context of host-microbe interactions, it is perfectly capable of
handling any other pair of data sets where some features interact
canonically.

## Installation instructions

Get the latest stable `R` release from
[CRAN](http://cran.r-project.org/). Then install `anansi` from
[Bioconductor](http://bioconductor.org/) using the following code:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("anansi")
```

And the development version from
[GitHub](https://github.com/thomazbastiaanssen/anansi) with `remotes`:

``` r
install.packages("remotes")
remotes::install_github("thomazbastiaanssen/anansi")
```

## Getting started using anansi

[See the vignettes on the package
site.](https://thomazbastiaanssen.github.io/anansi/articles/anansi.html)
Additionally, a [preprint](https://arxiv.org/abs/2305.10832) is
available.

## Citation

Below is the citation output from using `citation('anansi')` in R.

``` r
print(citation("anansi"), bibtex = TRUE)
#> To cite anansi in publications use:
#> 
#>   Bastiaanssen TFS, Quinn TP, Cryan JF (2023) Knowledge-based
#>   Integration of Multi-Omic Datasets with Anansi: Annotation-based
#>   Analysis of Specific Interactions arXiv. doi:
#>   10.48550/arXiv.2305.10832
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {Knowledge-based Integration of Multi-Omic Datasets with Anansi: Annotation-based Analysis of Specific Interactions},
#>     author = {Thomaz F S Bastiaanssen and Thomas P Quinn and John F Cryan},
#>     journal = {arXiv},
#>     year = {2023},
#>     doi = {10.48550/arXiv.2305.10832},
#>   }
```

## Code of Conduct

Please note that the `anansi` project is released with a [Contributor
Code of Conduct](http://bioconductor.org/about/code-of-conduct/). By
contributing to this project, you agree to abide by its terms.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    echo = TRUE,
    tidy.opts = list(width.cutoff = 80),
    tidy = TRUE
)
# long links
adj_link <- paste0(
    "https://thomazbastiaanssen.github.io/anansi/",
    "articles/adjacency_matrices.html"
)
dif_link <- paste0(
    "https://thomazbastiaanssen.github.io/anansi/",
    "articles/differential_associations.html"
)

library(anansi)

## ----install, eval = FALSE----------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#     install.packages("BiocManager")
# }
# 
# BiocManager::install("anansi")

## ----remotes-install, eval = FALSE--------------------------------------------
# install.packages("remotes")
# remotes::install_github("thomazbastiaanssen/anansi")

## ----load-libraries, eval=TRUE------------------------------------------------
# load anansi
library(anansi)

# load ggplot2 and ggforce to plot results
library(ggplot2)
library(ggforce)

# load example data + metadata from FMT Aging study
data(FMT_data)

## ----'ko-table-format', echo = FALSE------------------------------------------
cat("anansi expects samples to be rows and features to be columns:\n")
t(FMT_KOs[seq(4L),seq(10L)])

## ----'t-tables',  eval=TRUE---------------------------------------------------
# Transpose demo tables to input format
tableY <- t(FMT_metab)
tableX <- t(FMT_KOs)

## ----metadata-format, echo = FALSE--------------------------------------------
cat("anansi expects samples to be rows and features to be columns:\n")
`rownames<-`(FMT_metadata, NULL)[seq(10L),]

## ----weaveweb, eval=TRUE------------------------------------------------------
# Create AnansiWeb object from tables
web <- weaveWeb(
    x = "ko", y = "cpd", 
    link = kegg_link(),
    tableY = tableY,
    tableX = tableX,
    metadata = FMT_metadata
)

## ----bioc1--------------------------------------------------------------------
# Convert web to mae
mae <- asMAE(web)

## ----bioc2--------------------------------------------------------------------
web.mae <- weaveWeb(
    mae,
    tableY = "cpd", tableX = "ko",
    link = kegg_link()
)

out.mae <- anansi(
    web = web.mae,
    formula = ~Legend,
    adjust.method = "BH",
    verbose = TRUE
)

identical(web, web.mae)

## ----run-anansi, eval=TRUE----------------------------------------------------
# Run anansi on AnansiWeb object
anansi_out <- anansi(
    web = web,
    formula = ~Legend,
    adjust.method = "BH",
    verbose = TRUE
)

## ----show-output, eval=TRUE---------------------------------------------------
# General structure of anansi() output
head(anansi_out, c(5, 5))

# Let's take a look at the column names
colnames(anansi_out)

## ----bioc3--------------------------------------------------------------------
# Filter associations by q value
anansi_out <- anansi_out[anansi_out$full_q.values < 0.2, ]

# Visualize disjointed associations
plotAnansi(
    anansi_out,
    association.type = "disjointed",
    model.var = "Legend",
    signif.threshold = 0.05,
    fill_by = "group"
)

## ----r-web--------------------------------------------------------------------
rweb <- randomWeb(
    n_samples = 10,
    n_reps = 1,
    n_features_x = 8,
    n_features_y = 12,
    sparseness = 0.5
)

# The random object comes with some dummy metadata:
metadata(rweb)

# also see `?randomAnansi`

## ----pairwiseApply1-----------------------------------------------------------
# For each feature pair, was the value for x higher than the value for y?
pairwise_gt <- pairwiseApply(
    X = web,
    FUN = function(x, y) x > y, 
    USE.NAMES = TRUE
)

pairwise_gt[1:5, 1:5]

## ----pairwiseApply2-----------------------------------------------------------
# Run cor() on each pair of features
pairwise_cor <- pairwiseApply(
    X = web,
    FUN = function(x, y) cor(x, y), 
    USE.NAMES = FALSE
)

pairwise_cor

## ----plot-extra1--------------------------------------------------------------
# Use tidyr to wrangle the correlation r-values to a single column
library(tidyr)

anansiLong <- anansi_out |>
    pivot_longer(starts_with("All") | contains("FMT")) |>
    separate_wider_delim(
        name,
        delim = "_", names = c("cor_group", "param")
    ) |>
    pivot_wider(names_from = param, values_from = value)

# Now it's ready to be plugged into ggplot2, though let's clean up a bit more.
# Only consider interactions where the entire model fits well enough.
anansiLong <- anansiLong[anansiLong$full_q.values < 0.2, ]

ggplot(anansiLong) +

    # Define aesthetics
    aes(
        x = r.values,
        y = feature_X,
        fill = cor_group,
        alpha = disjointed_Legend_p.values < 0.05
    ) +

    # Make a vertical dashed red line at x = 0
    geom_vline(xintercept = 0, linetype = "dashed", colour = "red") +

    # Points show  raw correlation coefficients
    geom_point(shape = 21, size = 3) +

    # facet per compound
    ggforce::facet_col(~feature_Y, space = "free", scales = "free_y") +

    # fix the scales, labels, theme and other layout
    scale_y_discrete(limits = rev, position = "right") +
    scale_alpha_manual(
        values = c("TRUE" = 1, "FALSE" = 1 / 3),
        "Disjointed association\np < 0.05"
    ) +
    scale_fill_manual(
        values = c(
            "Young yFMT" = "#2166ac",
            "Aged oFMT" = "#b2182b",
            "Aged yFMT" = "#ef8a62",
            "All" = "gray"
        ),
        breaks = c("Young yFMT", "Aged oFMT", "Aged yFMT", "All"),
        name = "Treatment"
    ) +
    theme_bw() +
    ylab("") +
    xlab("Pearson's \u03c1")

## ----plot-extra2--------------------------------------------------------------
library(patchwork)

# Get a list containing data.frames for each feature pair.
feature_pairs <- getFeaturePairs(web)

# For each, prepare a ggplot object with those features on x & y axes.
plot_list <- lapply(
    feature_pairs,
    FUN = function(pair_i) {
        ggplot(pair_i) +
            aes(
                y = .data[[colnames(pair_i)[1L]]],
                x = .data[[colnames(pair_i)[2L]]]
            ) +
            # We'll use facet_grid for labels instead of labs().
            facet_grid(colnames(pair_i)[1L] ~ colnames(pair_i)[2L])
    }
)

# Feed the first six of our ggplots to patchwork, using wrap_plots()
wrap_plots(plot_list[seq_len(6L)], guides = "collect") &
    # Continue as with an ordinary ggplot, but use & instead of +
    geom_point(
        shape = 21, show.legend = FALSE,
        aes(fill = FMT_metadata$Legend)
    ) &
    geom_smooth(
        method = "lm", se = FALSE,
        aes(colour = FMT_metadata$Legend),
        show.legend = FALSE
    ) &

    # Appearance
    scale_fill_manual(
        aesthetics = c("colour", "fill"),
        values = c(
            "Young yFMT" = "#2166ac",
            "Aged oFMT" = "#b2182b",
            "Aged yFMT" = "#ef8a62",
            "All" = "gray"
        )
    ) &
    theme_bw() &
    labs(x = NULL, y = NULL)

## ----session-info-------------------------------------------------------------
sessionInfo()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    echo = TRUE,
    tidy.opts = list(width.cutoff = 80),
    tidy = TRUE
)
library(anansi)
library(ggplot2)

## ----plot-krebs---------------------------------------------------------------
library(patchwork)
library(ggraph)
library(igraph)
library(tidygraph)
library(dplyr)

metabs <- c(
    "isocitrate", "cis-aconitate", "citrate", "oxaloacetate",
    "L-malate", "fumarate", "succinate", "succinyl-CoA",
    "ketoglutarate"
)
d <- data.frame(
    from = c(
        "isocitrate", "ketoglutarate", "succinyl-CoA",
        "succinate", "fumarate", "L-malate", "oxaloacetate", "citrate",
        "cis-aconitate"
    ),
    to = c(
        "ketoglutarate", "succinyl-CoA", "succinate",
        "fumarate", "L-malate", "oxaloacetate", "citrate", "cis-aconitate",
        "isocitrate"
    ),
    vertices = c(
        "isocitrate\ndehydrogenase", "ketoglutarate\ndehydrogenase",
        "succinyl-CoA\nsynthetase", "succinate\ndehydrogenase",
        "\nfumarase", "malate\ndehydrogenase", "citrate\nsynthase",
        "aconitase\n", "aconitase\n"
    )
)

krebs_layout <- data.frame(
    x = (cos(seq(2 * pi, 0, length = 10) + pi / 2) * 1.25)[c(3:9, 1:2)],
    y = (sin(seq(2 * pi, 0, length = 10) + pi / 2) * 1.25)[c(3:9, 1:2)],
    circular = FALSE,
    name = c(
        "isocitrate", "ketoglutarate", "succinyl-CoA",
        "succinate", "fumarate", "L-malate",
        "oxaloacetate", "citrate", "cis-aconitate"
    ),
    .ggraph.orig_index = 1:9,
    .ggraph.index = 1:9
)

graph_from_data_frame(d, directed = TRUE) |>
    as_tbl_graph() |>
    mutate(Legend = if_else(name %in% metabs, "Metabolite", "Enzyme")) |>
    ggraph(layout = krebs_layout) +

    geom_edge_link(
        arrow = arrow(length = unit(4, "mm"), type = "closed"),
        edge_width = 2.5, color = "black",
        end_cap = circle(11.5, "mm"),
        start_cap = circle(11.25, "mm")
    ) +

    geom_edge_link(
        aes(label = vertices),
        arrow = arrow(length = unit(4, "mm"), type = "closed"),
        edge_width = 2, color = "dodgerblue",
        end_cap = circle(11.5, "mm"),
        start_cap = circle(11.5, "mm"),
        angle_calc = "along",
        label_dodge = unit(c(rep(13, 5), rep(-13, 4)), "mm"),
        label_size = 3.15
    ) +

    geom_node_label(aes(label = name),
        size = 3.5, fill = "orange",
        show.legend = FALSE, label.padding = unit(0.4, "lines"),
        nudge_x = c(0, 0.1, 0.05, -0.05, -0.1, 0, -0.1, 0, 0.1)
    ) +

    coord_equal(
        xlim = c(-1.25, 1.25), ylim = c(-1, 1),
        ratio = 1, clip = "off"
    ) +
    theme_graph()

## ----all-v-all----------------------------------------------------------------
colmat <- data.frame(
    name = c(
        "aconitase", "isocitrate\ndehydrogenase",
        "ketoglutarate\ndehydrogenase", "succinyl-CoA\nsynthetase",
        "succinate\ndehydrogenase", "fumarase", "malate\ndehydrogenase",
        "citrate\nsynthase", "citrate", "cis-aconitate", "isocitrate",
        "ketoglutarate", "succinyl-CoA", "succinate", "fumarate", "L-malate",
        "oxaloacetate"
    ),
    y = c(seq(9, 1, length.out = 8), 9:1),
    x = rep(c(0.7, 0.3), times = c(8, 9)),
    fill = rep(c("Enzyme", "Metabolite"), times = c(8, 9))
)

p_0 <- ggplot() +
    annotate("segment",
        x = c(0.28, 0.72), y = 0.875,
        xend = c(0.28, 0.72), yend = 9.125, color = "gray"
    ) +
    geom_text(
        data = colmat, aes(y = y, x = x, label = name), colour = "black",
        size = 3.25, hjust = c(rep(0, 8), rep(1, 9)), lineheight = 7 / 10,
        nudge_x = c(rep(0.03, 8), rep(-0.03, 9))
    ) +
    scale_fill_manual(
        values = c(Metabolite = "orange", Enzyme = "dodgerblue")
    ) +
    coord_cartesian(xlim = c(1 / 20, 19 / 20), clip = "off") +
    annotate("text",
        label = c("Metabolites", "Enzymes"),
        x = c(2 / 8, 6 / 8), y = 9.6, size = 4
    ) +
    ylab(NULL) +
    xlab(NULL) +
    theme_void()

p_a <- p_0 +
    geom_point(
        data = colmat, aes(y = y, x = x, fill = fill),
        color = "black", shape = 21, size = 4, show.legend = FALSE
    )



p_b <- p_0 + geom_segment(
    data = expand.grid(seq(9, 1, length.out = 8), 9:1),
    aes(y = Var2, yend = Var1), x = 3 / 10, xend = 7 / 10,
    color = "#393D47", inherit.aes = FALSE
) +
    geom_point(
        data = colmat, aes(y = y, x = x, fill = fill),
        color = "black",
        shape = 21, size = 4, show.legend = FALSE
    )
p_c <- p_0 + geom_segment(
    data = data.frame(
        Var2 = seq(1, 9, length.out = 8)[
            c(8, 1, 8, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3, 3, 2, 2, 1)
        ],
        Var1 =
            c(9, 9, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3, 3, 2, 2, 1, 1)
    ),
    aes(y = Var1, yend = Var2), x = 3 / 10, xend = 7 / 10,
    color = "#393D47", inherit.aes = FALSE
) +
    geom_point(
        data = colmat, aes(y = y, x = x, fill = fill),
        color = "black", shape = 21, size = 4, show.legend = FALSE
    )

(p_b | p_c) + plot_annotation(tag_levels = "A")

## ----krebs-biadj--------------------------------------------------------------
#|

krebs_edge_df <- local({
    data("krebs", package = "anansi")
    return(krebs)
})

krebs_edge_df |>
    mutate(Enzyme = `levels<-`(
        Enzyme,
        gsub(x = levels(Enzyme), " ", replacement = "\n")
    )) |>
    ggplot() +
    geom_text(label = "X", x = I(0.5), y = I(0.5)) +
    facet_grid(Enzyme ~ Metabolite, switch = "y", space = "free_x") +
    theme_bw() +
    theme(
        strip.text.y.left = element_text(
            angle = 0, size = 10, lineheight = 7 / 10, hjust = 1
        ),
        strip.text.x.top = element_text(angle = 90, size = 10, hjust = 0),
        strip.background.x = element_rect(fill = "orange", color = "black"),
        strip.background.y = element_rect(fill = "dodgerblue", color = "black"),
        panel.spacing = unit(0.1, "lines")
    ) +
    coord_equal(ratio = 1) +
    labs(x = NULL, y = NULL)

## ----weaveWeb-intro-----------------------------------------------------------
# Two interfaces
form.web <- weaveWeb(ec ~ ko, link = kegg_link())

trad.web <- weaveWeb(y = "ec", x = "ko", link = kegg_link())

# Same output
identical(form.web, trad.web)

# Get the biadjacency matrix using the $ operator
head(dictionary(form.web), c(10, 10))

## ----peakec2ko----------------------------------------------------------------
# Extract the data.frame from the list that kegg_link() returns
ec2ko <- kegg_link()[["ec2ko"]]

head(ec2ko)

## ----names--------------------------------------------------------------------
# Note that the formula controls the order
names(weaveWeb(ko ~ ec, link = kegg_link()))

# names() is short for the above
names(weaveWeb(ec ~ ko, link = kegg_link()))

## ----weaveKEGG----------------------------------------------------------------
# Only print the first few entries
lapply(kegg_link(), head)

## ----web_formula--------------------------------------------------------------
# Formula notation
head(
    dictionary( weaveWeb(cpd ~ ko, link = kegg_link()) ), 
    c(10, 10)
)

# Character notation
head(
    dictionary( weaveWeb(y = "ko", x = "cpd", link = kegg_link()) ), 
    c(10, 10)
)

## ----AnansiWeb----------------------------------------------------------------
# Prep some dummy input tables
tX <- matrix(rnorm(30),
    nrow = 6,
    dimnames = list(NULL, x = paste("x", seq_len(5), sep = "_"))
)
tY <- matrix(rnorm(42),
    nrow = 6,
    dimnames = list(NULL, y = paste("y", seq_len(7), sep = "_"))
)

# Prep biadjacency matrix
# base::matrix is fine too, but will get coerced to Matrix::Matrix.
d <- matrix(
    data = sample(x = c(TRUE, FALSE), size = 35, replace = TRUE),
    nrow = NCOL(tY),
    ncol = NCOL(tX),
    dimnames = list(colnames(tY), colnames(tX))
)

# make the AnansiWeb
w <- AnansiWeb(
    tableY = tY,
    tableX = tX,
    dictionary = d
)

# Confirm the warning:
names(w)

## ----igraph-1, message=FALSE--------------------------------------------------
library(igraph)
# Prepared data.frame for the occasion
head(krebs_edge_df)

# Convert data.frame to graph
g <- graph_from_data_frame(krebs_edge_df, directed = FALSE)

## ----igraph-2, message=FALSE--------------------------------------------------
V(g)$type <- V(g)$name %in% krebs_edge_df$Metabolite

# Now that we've defined type, we can convert to biadjacency matrix:
bi_mat1 <- as_biadjacency_matrix(g, sparse = TRUE)


head(bi_mat1, n = c(4, 5))

## ----Matrix, message=FALSE----------------------------------------------------
library(Matrix)

# For this approach, it's useful to prepare the input as two factors:
Enzyme <- factor(krebs_edge_df$Enzyme)
Metabolite <- factor(krebs_edge_df$Metabolite)

# We can get integers out of factors, corresponding to their level indices
bi_mat2 <- sparseMatrix(
    i = as.integer(Enzyme),
    j = as.integer(Metabolite),
    dimnames = list(
        Enzyme = levels(Enzyme),
        Metabolite = levels(Metabolite)
    )
)

head(bi_mat2, n = c(4, 5))

## ----session-info-------------------------------------------------------------
sessionInfo()

## ----style, echo = FALSE, results = 'asis'------------------------------------
BiocStyle::markdown()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    echo = TRUE,
    tidy.opts = list(width.cutoff = 80),
    tidy = TRUE
)
# long links
adj_link <- paste0(
    "https://thomazbastiaanssen.github.io/anansi/",
    "articles/adjacency_matrices.html"
)
library(S7)

## ----formula-web-full---------------------------------------------------------
# Setup AnansiWeb with some dummy data
library(anansi)
web <- krebsDemoWeb()

## ----y-x-formula,echo=FALSE---------------------------------------------------
y ~ x

## ----formulate, echo = FALSE--------------------------------------------------
f <- y ~ x * (group_ab + score_a)
f

## ----reformulate, echo=FALSE--------------------------------------------------
update.formula(f, . ~ .)

## ----formula-anansi-full------------------------------------------------------
out <- anansi(web,
    formula = ~ group_ab + score_a,
    groups = "group_ab"
)

## ----plot_full----------------------------------------------------------------
#|
library(patchwork)
library(ggplot2)

full_df <- data.frame(
    citrate = tableX(web)[, "citrate"],
    `cis-aconitate` = tableX(web)[, "cis-aconitate"],
    aconitase = tableY(web)[, "aconitase"]
)

ggplot(full_df) +
    aes(x = aconitase, y = citrate) +
    facet_wrap(~"citrate") +
    ggplot(full_df) +
    aes(x = aconitase, y = cis.aconitate) +
    labs(y = "cis-aconitate") +
    facet_wrap(~"cis-aconitate") +
    plot_layout(axes = "collect", axis_titles = "collect") +
    plot_annotation(
        subtitle =
        ) &
    geom_point(shape = 21, fill = "#ffffbf") &

    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5)) &
    theme_bw()

## ----disj_cat-----------------------------------------------------------------
#|
diff_df <- data.frame(
    group_ab = paste0("Treatment ", metadata(web)$group_ab),
    score_a = metadata(web)$score_a,
    score_a_cat = cut(
        metadata(web)$score_a,
        labels = c("0-20%", "20-40%", "40-60%", "60-80%", "80-100%"),
        quantile(metadata(web)$score_a, probs = seq(0, 1, 1 / 5)),
        include.lowest = TRUE
    ),
    repeated = metadata(web)$repeated,
    sample_id = metadata(web)$sample_id,
    isocitrate = tableX(web)[, "isocitrate"],
    ketoglutarate = tableX(web)[, "ketoglutarate"],
    isocitrate.dehydrogenase = tableY(web)[, "isocitrate dehydrogenase"],
    ketoglutarate.dehydrogenase = tableY(web)[, "ketoglutarate dehydrogenase"],
    succinyl.CoA = tableX(web)[, "succinyl-CoA"],
    succinate = tableX(web)[, "succinate"],
    succinyl.CoA.synthetase = tableY(web)[, "succinyl-CoA synthetase"],
    succinate.dehydrogenase = tableY(web)[, "succinate dehydrogenase"],
    fumarate = tableX(web)[, "fumarate"],
    fumarase = tableY(web)[, "fumarase"]
)

ggplot(diff_df) +
    aes(x = isocitrate.dehydrogenase, y = isocitrate, fill = group_ab) +
    facet_wrap(~group_ab, ) +
    labs(x = "isocitrate dehydrogenase") +
    geom_point(shape = 21, show.legend = FALSE) +
    scale_fill_manual(values = c(
        "Treatment a" = "#d73027",
        "Treatment b" = "#2166ac"
    )) +
    theme_bw() +
    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5))

## ----disj_cont----------------------------------------------------------------
#|
ggplot(diff_df) +
    aes(
        x = ketoglutarate.dehydrogenase,
        y = ketoglutarate,
        fill = pnorm(score_a)
    ) +
    facet_grid(~score_a_cat) +
    geom_point(shape = 21, show.legend = FALSE) +
    scale_fill_gradientn(
        name = "Example health index",
        colors = c("#a50026", "#f46d43", "#ffffbf", "#74add1", "#313695")
    ) +
    labs(x = "ketoglutatate dehydrogenase") +
    theme_bw() +
    guides(fill = guide_colourbar(
        position = "bottom",
        legend.direction = "horizontal"
    )) +
    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5))

## ----disj-lm------------------------------------------------------------------

score_lm <- pairwiseApply(web, FUN = function(x, y) {
    anova(
        lm(y ~ x * (group_ab + score_a), data = metadata(web))
    )$`F value`[5L]
})

group_lm <- pairwiseApply(web, FUN = function(x, y) {
    anova(
        lm(y ~ x * (score_a + group_ab), data = metadata(web))
    )$`F value`[5L]
})

all.equal(score_lm, out$disjointed_score_a_f.values)
all.equal(group_lm, out$disjointed_group_ab_f.values)

## ----emerg_cat----------------------------------------------------------------
#|
ggplot(diff_df) +
    aes(x = succinyl.CoA.synthetase, y = succinyl.CoA, fill = group_ab) +
    facet_wrap(~group_ab) +
    geom_point(shape = 21, show.legend = FALSE) +
    scale_fill_manual(values = c(
        "Treatment a" = "#d73027",
        "Treatment b" = "#2166ac"
    )) +
    labs(x = "succinyl-CoA synthetase", y = "succinyl-CoA") +
    theme_bw() +
    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5))

## ----emerg_cont---------------------------------------------------------------
#|
ggplot(diff_df) +
    aes(x = succinate.dehydrogenase, y = succinate, fill = pnorm(score_a)) +
    facet_grid(~score_a_cat) +
    geom_point(shape = 21, show.legend = FALSE) +
    scale_fill_gradientn(
        name = "Example health index",
        colors = c("#a50026", "#f46d43", "#ffffbf", "#74add1", "#313695")
    ) +
    labs(x = "succinate dehydrogenase") +
    theme_bw() +
    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5))

## ----intro-error--------------------------------------------------------------
outErr <- anansi(web,
    formula = ~ group_ab + score_a + Error(sample_id),
    groups = "group_ab"
)

## ----aov-error----------------------------------------------------------------
summary(
    aov(fumarate ~ fumarase * repeated + Error(sample_id), data = diff_df)
)

## ----aov-regular--------------------------------------------------------------
summary(
    aov(fumarate ~ sample_id + fumarase * repeated, data = diff_df)
)

# See ?stats::aov for more.

## ----session-info-------------------------------------------------------------
sessionInfo()

#' Methods for pairwise operations on AnansiWeb objects
#' @description
#' Methods to run pairwise analysis on multi-modal data.
#'
#' @name AnansiWeb-pairwise
#' @rdname AnansiWeb-pairwise
#' @param X,x input, AnansiWeb object
#' @param ... additional arguments
#' @param FUN a function with at least two arguments. The variables `x` and `y`,
#'     in order, refer to the corresponding values of feature pairs in `tableX`
#'     and `tableY`.
#' @param MoreArgs,SIMPLIFY,USE.NAMES see ?base::mapply
#' @param which `integer matrix`, indicating pair positions in `x@tableY` and
#'     `x@tableX`, respectively. If `NULL` (default):
#'     `Matrix::which(x@dictionary, TRUE)`.
#' @param with.metadata `Logical scalar` whether to append metadata to output
#' @aliases pairs.AnansiWeb pairs.anansi::AnansiWeb pairwiseApply.AnansiWeb
#' @aliases pairwiseApply.anansi::AnansiWeb
#' @returns
#' pairs: A two-column array index, corresponding to i,j coordinates in matrix
#' notation. \cr
#' getFeaturePairs: A list of data.frames with the paired data \cr
#'
#'

#' @examples
#' web <- randomWeb()
#' # Extract data.frames in pairs (only show first)
#' getFeaturePairs(web)[1L]
#'
#' pairs(x = web)
#'
#' getFeaturePairs(x = web, which = NULL, with.metadata = FALSE)
#'
#' pairwiseApply(
#'     X = web,
#'     FUN = function(x, y) cor(x, y),
#'     MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES = TRUE
#' )
#'
NULL

#' @export
#' @importFrom graphics pairs
S7::method(pairs, AnansiWeb) <- function(x, ...) `pairs.anansi::AnansiWeb`(x)

#' @export
`pairs.anansi::AnansiWeb` <- function(x, ...) {
    Matrix::which(
        x@dictionary,
        arr.ind = TRUE, useNames = FALSE
    )
}

#' @importFrom Matrix which
#'
S7::method(getFeaturePairs, AnansiWeb) <- function(
        x, which = NULL, with.metadata = FALSE
) {
    if (is.null(which)) {
        which <- Matrix::which(x@dictionary, arr.ind = TRUE, useNames = FALSE)
    }
    tX <- x@tableX
    tY <- x@tableY
    xnames <- colnames(tX)
    ynames <- colnames(tY)
    if (!with.metadata) {
        return(
            lapply(seq_len(NROW(which)), FUN = function(z) {
                cbind(
                    tY[, which[z, 1L], drop = FALSE],
                    tX[, which[z, 2L], drop = FALSE]
                )
            })
        )
    } else {
        metadata <- x@metadata
        return(
            lapply(seq_len(NROW(which)), FUN = function(z) {
                cbind(
                    tY[, which[z, 1L], drop = FALSE],
                    tX[, which[z, 2L], drop = FALSE],
                    metadata
                )
            })
        )
    }
}


#' @export
S7::method(pairwiseApply, AnansiWeb) <- function(
        X, FUN, MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES = TRUE
) {
    tY <- as.data.frame.matrix(X@tableY, make.names = FALSE)
    tX <- as.data.frame.matrix(X@tableX, make.names = FALSE)
    wh <- Matrix::which(X@dictionary, arr.ind = TRUE, useNames = FALSE)

    out <- base::.mapply(
        FUN,
        dots = list(
            x = tX[wh[, 2L]],
            y = tY[wh[, 1L]]
        ),
        MoreArgs
    )
    if (USE.NAMES) {
        names(out) <- paste0(colnames(tX)[wh[, 2L]], colnames(tY)[wh[, 1L]])
    }

    if (SIMPLIFY) {
        out <- simplify2array(out)
    }
    return(out)
}

#' @title Coercion functions for anansi
#' @name anansi-coercion
#' @rdname anansi-coercion
#' @returns An object of the desired class.
#' @param x input object
#' @param use.names `Logical scalar`, whether output list should contain
#'     character (Default) or integer data frame. If `FALSE`, returns
#'     `unfactor(x)`.
#' @param row.names,optional Not used. See ?base::data.frame
#' @param ... additional arguments (currently not used).
#' @seealso [unfactor()]
#' @examples
#' # AnansiWeb
#' x <- randomWeb(
#'     n_samples = 5,
#'     n_features_x = 4,
#'     n_features_y = 6
#' )
#'
#' as.list(x)
#' as.data.frame(x)
#'
#' # AnansiWeb to MultiAssayExperiment
#' asMAE(x)
#' asTSE(x)
#'
#' # MultiFactor
#' x <- randomMultiFactor(n_types = 3, n_features = 3)
#' as.list(x, use.names = TRUE)
#'
NULL

#' @export
#' @importFrom S7 props
#' @rdname anansi-coercion
`as.list.anansi::AnansiWeb` <- function(x, ...) {
    out <- S7::props(x)
    names(out)[c(1L, 2L)] <- names(x)
    out
}

#' @export
S7::method(as.list, MultiFactor) <-
    function(x, ..., use.names = TRUE) {
        `as.list.anansi::MultiFactor`(
            x, ..., use.names
        )
    }

#' @export
#' @importFrom S4Vectors unfactor
#' @rdname anansi-coercion
`as.list.anansi::MultiFactor` <- function(x, ..., use.names = TRUE) {
    ifelse(
        use.names,
        yes = return(S4Vectors::unfactor(x)),
        no = return(`names<-`(x@index, rownames(x)))
    )
}

#' @export
S7::method(as.list, AnansiWeb) <- function(x, ...) {
    `as.list.anansi::AnansiWeb`(x)
}

#' @export
S7::method(as.data.frame, AnansiWeb) <- function(x, row.names, optional, ...) {
    `as.data.frame.anansi::AnansiWeb`(x)
}

#' @export
#' @rdname anansi-coercion
`as.data.frame.anansi::AnansiWeb` <- function(x, row.names, optional, ...) {
    cbind(
        x@tableY,
        x@tableX,
        x@metadata
    )
}

#' @name asMAE
#' @rdname anansi-coercion
#' @aliases as.MAE as.MultiAssayExperiment asMultiAssayExperiment
#' @importClassesFrom MultiAssayExperiment MultiAssayExperiment
#' @importFrom MultiAssayExperiment MultiAssayExperiment ExperimentList
#' @importFrom SummarizedExperiment SummarizedExperiment
#' @export
#'
asMAE <- function(x) {
    tY <- t(x@tableY)
    tX <- t(x@tableX)
    to_exp <- ExperimentList(
        y = SummarizedExperiment::SummarizedExperiment(tY),
        x = SummarizedExperiment::SummarizedExperiment(tX)
    )
    names(to_exp) <- names(x)

    to_md <- list(dictionary = x@dictionary)
    to_cd <- x@metadata

    if (prod(dim.data.frame(to_cd)) == 0L) {
        MultiAssayExperiment::MultiAssayExperiment(
            experiments = to_exp,
            metadata = to_md
        )
    } else {
        MultiAssayExperiment::MultiAssayExperiment(
            experiments = to_exp,
            metadata = to_md,
            colData = to_cd
        )
    }
}

#' @name asTSE
#' @rdname anansi-coercion
#' @aliases as.TSE as.TreeSummarizedExperiment asTreeSummarizedExperiment
#' @importClassesFrom TreeSummarizedExperiment TreeSummarizedExperiment
#' @importFrom TreeSummarizedExperiment TreeSummarizedExperiment
#' @export
#'
asTSE <- function(x) {
    tY <- t(x@tableY)
    tX <- t(x@tableX)
    to_exp <- list(
        y = tY
    )
    names(to_exp) <- names(x)[1]
    to_alt <- list(
        x = TreeSummarizedExperiment::TreeSummarizedExperiment(tX)
    )
    names(to_alt) <- names(x)[2]

    to_md <- list(dictionary = x@dictionary)
    to_cd <- x@metadata

    if (prod(dim.data.frame(to_cd)) == 0L) {
        TreeSummarizedExperiment::TreeSummarizedExperiment(
            assays = to_exp,
            altExps = to_alt,
            metadata = to_md
        )
    } else {
        TreeSummarizedExperiment::TreeSummarizedExperiment(
            assays = to_exp,
            altExps = to_alt,
            metadata = to_md,
            colData = to_cd
        )
    }
}

test_that("weaveWeb mae and tse methods work", {
    # Combine experiments into MultiAssayExperiment object
    web <- randomWeb(n_samples = 15, n_reps = 1)
    web@metadata$cat_XYZ <- rep(c("X", "Y", "Z"), 5)
    mae <- asMAE(web)

    web2 <- weaveWeb(x = mae, tableY = "y", tableX = "x")
    # Once more, my friends
    mae2 <- asMAE(web)

    expect_identical(web, web2)
    expect_identical(mae, mae2)

    expect_error(
        weaveWeb(mae, tableY = "wrong_name"),
        "'tableY' must be numeric or character value specifying experiment in experiment(x)",
        fixed = TRUE
    )

    expect_error(
        weaveWeb(
            mae,
            tableY = "y",
            tableX = "x",
            web = 0
        ),
        "unused argument (web = 0)",
        fixed = TRUE
    )
    ### Check identity with original anansi output ###

    table1 <- anansi(
        web = web,
        formula = ~cat_XYZ,
        verbose = FALSE
    )
    list1 <- anansi(
        web = web,
        formula = ~cat_XYZ,
        verbose = FALSE,
        return.format = "list"
    )
    raw1 <- anansi(
        web = web,
        formula = ~cat_XYZ,
        verbose = FALSE,
        return.format = "raw"
    )

    table.mae <- weaveWeb(
        mae,
        tableY = "y",
        tableX = "x"
    ) |> anansi(
        formula = ~cat_XYZ,
        verbose = FALSE
    )
    list.mae <- weaveWeb(
        mae,
        tableY = "y",
        tableX = "x"
    ) |> anansi(
        formula = ~cat_XYZ,
        return.format = "list",
        verbose = FALSE
    )
    raw.mae <- weaveWeb(
        mae,
        tableY = "y",
        tableX = "x"
    ) |> anansi(
        formula = ~cat_XYZ,
        return.format = "raw",
        verbose = FALSE
    )

    expect_identical(table1, table.mae)
    expect_identical(list1, list.mae)
    expect_identical(raw1, raw.mae)

    tse <- asTSE(web)

    table.tse <- weaveWeb(
        tse,
        tableY = "y",
        tableX = "x"
    ) |> anansi(
        formula = ~cat_XYZ,
        verbose = FALSE
    )
    list.tse <- weaveWeb(
        tse,
        tableY = "y",
        tableX = "x"
    ) |> anansi(
        formula = ~cat_XYZ,
        return.format = "list",
        verbose = FALSE
    )
    raw.tse <- weaveWeb(
        tse,
        tableY = "y",
        tableX = "x"
    ) |> anansi(
        formula = ~cat_XYZ,
        return.format = "raw",
        verbose = FALSE
    )


    expect_identical(table1, table.tse)
    expect_identical(list1, list.tse)
    expect_identical(raw1, raw.tse)
})

#' Generate a random AnansiWeb or MultiFactor
#' @name randomAnansi
#' @description
#' Randomly generate a valid `AnansiWeb` or `MultiFactor` object.
#' @param n_samples,n_reps `Numeric scalar` Number of samples and repeated
#'     measures of those samples to be generated. Ignored if `tableY` and
#'     `tableX` are provided. (defaults: 10 samples without repeats)
#' @param n_features_y,n_features_x `Numeric scalar` Number of features to be
#'     generated. Ignored if `tableY` and `tableX` are provided.
#' @param tableY,tableX A table containing features of interest. Rows should be
#'     samples and columns should be features. Y and X refer to the position of
#'     the features in a formula: Y ~ X.
#' @param dictionary A binary adjacency matrix of class `Matrix`, or
#' coercible to `Matrix`.
#' @returns a randomly generated object of the specified class.
#' @examples
#' # Make a random AnansiWeb object
#' randomWeb()
#' krebsDemoWeb()
#' randomMultiFactor()
#' @seealso [AnansiWeb()], [MultiFactor()]
#'
NULL

#' @rdname randomAnansi
#' @name randomWeb
#' @export
#'
randomWeb <- function(
    n_samples = 10,
    n_reps = 1L,
    n_features_x = 8,
    n_features_y = 12,
    sparseness = 0.5,
    tableY = NULL,
    tableX = NULL,
    dictionary = NULL
) {
    stopifnot(
        "'sparseness' must be a proportion [0-1]. " = sparseness <= 1 &&
            sparseness > 0
    )
    stopifnot(
        "At least one of 'tableY,tableX', 'dictionary' should be NULL." = any(c(
            is.null(tableY),
            is.null(tableX),
            is.null(dictionary)
        ))
    )
    stopifnot(
        "Both 'tableY,tableX' should be provided or NULL. " = is.null(
            tableY
        ) ==
            is.null(tableX)
    )

    density <- 1 - sparseness
    # All missing: return full random Web
    if (all(c(is.null(tableY), is.null(tableX), is.null(dictionary)))) {
        return(
            randomWebFull(
                n_samples,
                n_reps,
                n_features_x,
                n_features_y,
                density
            )
        )
    }
    # Dictionary missing: make random fitting dictionary, return filled Web
    if (is.null(dictionary)) {
        return(
            randomWebDic(tableY, tableX, density)
        )
    }
    # Tables missing: make random fitting tables, return filled Web
    return(randomWebTab(n_samples, n_reps, dictionary))
}

#' @rdname randomAnansi
#' @param n_types `Numeric scalar`, number of types of features to generate
#' @param n_features `Numeric scalar`, number of features per type
#' @param sparseness `Numeric scalar`, proportion: How rare are connections
#' @export
#'
randomMultiFactor <- function(n_types = 6, n_features = 100, sparseness = 0.5) {
    stopifnot(
        "'sparseness' must be a proportion [0-1]. " = sparseness <= 1 &&
            sparseness > 0
    )
    n_types <- max(min(n_types, 26), 2)
    ids <- letters[seq_len(n_types)]
    out_names <- paste0(ids[-n_types], "2", ids[-1L])
    id_list <- lapply(ids, function(x) {
        paste(
            x,
            formatC(seq_len(n_features), digits = 2, flag = "0"),
            sep = "_"
        )
    })

    out <- lapply(seq_len(n_types - 1), FUN = function(x) {
        randomLinkDF(
            l = id_list[-n_types][[x]],
            r = id_list[-1L][[x]],
            l_id = ids[-n_types][x],
            r_id = ids[-1L][x],
            p = (1 - sparseness)
        )
    })
    names(out) <- out_names
    asMultiFactor(out)
}

#' @rdname randomAnansi
#' @aliases krebsDemoWeb
#' @importFrom Matrix sparseMatrix
#' @importFrom stats pnorm
#' @export
#'
krebsDemoWeb <- function(n_samples = 100, n_reps = 4L) {
    # Load krebs edgelist
    kd <- local({
        data("krebs", package = "anansi", envir = environment())
        # explicitly assign from environment
        krebs <- get("krebs")
        # Define dictionary
        return(
            Matrix::sparseMatrix(
                i = as.integer(krebs$Enzyme),
                j = as.integer(krebs$Metabolite),
                dimnames = lapply(krebs, levels)
            )
        )
    })
    # Generate web with metadata
    w <- randomWeb(n_samples, n_reps, dictionary = kd)

    # Spike demo associations
    int_ab <- w@metadata$group_ab == "a"
    int_pr <- pnorm(w@metadata$score_a)

    # Positive association aconitase ~ citrate
    w@tableY[, 1L] <- scale(w@tableY[, 1L] * 0.25 + w@tableX[, 1L] * 0.75)
    # Negative association aconitase ~ cis-aconitate
    w@tableX[, 2L] <- scale(w@tableX[, 2L] * 0.25 + w@tableY[, 1L] * -0.75)
    # Disjointed association isocitrate dehydrogenase ~ isocitrate
    w@tableY[, 2L] <- scale(w@tableY[, 2L] * 0.25 + w@tableX[, 3L] * 0.75)
    w@tableY[int_ab, 2L] <- w@tableY[int_ab, 2L] * -1L
    # Disjointed association ketoglutarate dehydrogenase ~ ketoglutarate
    w@tableY[, 3L] <- scale(
        w@tableY[, 3L] * 0.25 + w@tableX[, 4L] * 0.75 * w@metadata$score_a
    )
    # Emergent association succinyl-CoA synthetase ~ succinyl-CoA
    w@tableY[, 4L] <- scale(
        w@tableY[, 4L] *
            (0.25 + 0.50 * !int_ab) +
            w@tableX[, 5L] * (0.25 + 0.50 * int_ab)
    )
    # Emergent association succinate dehydrogenase ~ succinate
    w@tableY[, 5L] <- scale(
        w@tableY[, 5L] * int_pr + w@tableX[, 6L] * (1 - int_pr)
    )

    return(w)
}

#' Generate a random AnansiWeb, without any prior components
#' @description
#' called by randomWeb, not for user.
#' @importFrom Matrix rsparsematrix
#' @importFrom stats rnorm
#' @rdname randomAnansi
#' @noRd
#'
randomWebFull <- function(n_samp, n_reps, n_x, n_y, density) {
    rn <- paste0(
        "sample_",
        rep(seq_len(n_samp), each = n_reps),
        "_",
        seq_len(n_reps)
    )
    tableY <- matrix(
        data = rnorm(n_y * n_samp * n_reps),
        nrow = n_samp * n_reps,
        ncol = n_y,
        dimnames = list(
            sample_id = rn,
            y = paste0("y_", seq_len(n_y))
        )
    )
    tableX <- matrix(
        data = rnorm(n_x * n_samp * n_reps),
        nrow = n_samp * n_reps,
        ncol = n_x,
        dimnames = list(
            sample_id = rn,
            x = paste0("x_", seq_len(n_x))
        )
    )
    randomWebDic(tableY, tableX, density)
}

#' Generate a random AnansiWeb, only missing tables
#' @description
#' called by randomWeb, not for user.
#' @rdname randomAnansi
#' @noRd
#'
randomWebTab <- function(n_samp, n_reps, dictionary, metadata) {
    d <- dim(dictionary)
    rn <- paste0(
        "sample_",
        rep(seq_len(n_samp), each = n_reps),
        "_",
        seq_len(n_reps)
    )
    tableY <- matrix(
        data = rnorm(d[1] * n_samp * n_reps),
        nrow = n_samp * n_reps,
        ncol = d[1],
        dimnames = c(
            list(
                sample_id = rn
            ),
            dimnames(dictionary)[1]
        )
    )
    tableX <- matrix(
        data = rnorm(d[2] * n_samp * n_reps),
        nrow = n_samp * n_reps,
        ncol = d[2],
        dimnames = c(
            list(
                sample_id = rn
            ),
            dimnames(dictionary)[2]
        )
    )
    names(dimnames(tableX))[2] <- names(dimnames(dictionary))[2]
    metadata <- randomWebMetadata(tableY, n_samp, n_reps)
    # return AnansiWeb
    AnansiWeb(
        tableY = tableY,
        tableX = tableX,
        dictionary = dictionary,
        metadata = metadata
    )
}

#' Generate a random AnansiWeb, only missing dictionary.
#' @description
#' called by randomWeb, not for user.
#' @importFrom Matrix rsparsematrix
#' @rdname randomAnansi
#' @noRd
#'
randomWebDic <- function(tableY, tableX, density) {
    dictionary <- rsparsematrix(
        nrow = NCOL(tableY),
        ncol = NCOL(tableX),
        density = density,
        rand.x = NULL,
        dimnames = list(
            y = colnames(tableY),
            x = colnames(tableX)
        )
    )
    names(dimnames(dictionary)) <- c(
        names(dimnames(tableY))[2L],
        names(dimnames(tableX))[2L]
    )
    metadata <- randomWebMetadata(tableY)
    # return AnansiWeb
    AnansiWeb(
        tableY = tableY,
        tableX = tableX,
        dictionary = dictionary,
        metadata = metadata
    )
}

#' Generate random metadata for AnansiWeb
#' @description
#' called by randomWeb, not for user.
#' @param table a web table
#' @rdname randomAnansi
#' @noRd
#'
randomWebMetadata <- function(table, n_samples = NULL, n_reps = NULL) {
    if (is.null(n_samples)) {
        n_samples <- NROW(table)
        n_reps <- 1L
    }
    m <- data.frame(
        sample_id = paste0("sample_", rep(seq_len(n_samples), each = n_reps)),
        repeated = paste0("rep_", seq_len(n_reps)),
        group_ab = rep(
            sample(c("a", "b"), n_samples, replace = TRUE),
            each = n_reps
        ),
        subtype = rep(
            sample(c("x", "y", "z"), n_samples, replace = TRUE),
            each = n_reps
        ),
        score_a = rnorm(n_samples),
        score_b = rnorm(n_samples),
        score_c = rnorm(n_samples),
        row.names = row.names(table)
    )
    return(m)
}

#' Make a single df for a random MultiFactor
#' @rdname randomAnansi
#' @description called by `randomMultiFactor`, shouldn't be called by user.
#' @param l,r character vector of left, right features
#' @param l_id,r_id character scalar of left, right feature names
#' @param p proportion of connections to keep
#' @noRd
#'
randomLinkDF <- function(l, r, l_id, r_id, p) {
    len <- length(l) * length(r)
    ind <- sort(sample(seq_len(len), size = ceiling(p * len)))
    out <- expand.grid(l, r, KEEP.OUT.ATTRS = FALSE)[ind, ]
    names(out) <- c(l_id, r_id)
    out
}

x <- MultiFactor(kegg_link(), drop.unmatched = FALSE)

test_that("Delayed shedding levels works; MultiFactor returns MultiFactor", {
    expect_identical(
        MultiFactor(x),
        MultiFactor(x)
    )
})

test_that("Dropping levels works", {
    x <- randomMultiFactor(n_features = 10)

    expect_error(
        droplevels(
            x,
            select = list(a = "a_001"),
            exclude = list(d = c("d_001"))
        ),
        regexp = "Only one of 'exclude' and 'select' may be provided"
    )
    y <- droplevels(x, select = list(a = "a_010"))
    z <- droplevels(x, exclude = list(a = paste0("a_00", seq(1, 9))))

    expect_identical(y, z)

    expect_identical(dim(y), c(5L, 6L))
    expect_identical(names(names(y)), rownames(y))
    expect_identical(y, y[])
})

x <- randomMultiFactor(n_features = 10, n_types = 3)

test_that("MultiFactor indexing works", {
    expect_identical(
        x,
        asMultiFactor(x[["b"]], levels = levels(x))
    )

    expect_identical(
        x,
        MultiFactor(c(x[["a"]], x[["c"]]), levels = levels(x))
    )
})

test_that("MultiFactor get/set works", {
    # two-way equivalence
    expect_identical(x[["a"]], x[["a"]] <- x[, c("a", "b")][1])
    expect_identical(x[, c("a", "b")][1], x[, c("a", "b")][1] <- x[["a"]])

    expect_identical(x@map, x@map <- x@map)
})


test_that("show works", {
    expect_null(show(x))
})

x <- randomWeb()

test_that("AnansiWeb coersion to data.frame and list works", {
    a <- as.data.frame(x)
    b <- do.call(cbind, unname(as.list(x)[c("y", "x", "metadata")]))
    expect_equal(a, b)
})

x <- randomWeb()

test_that("getFeaturePairs and mapply work", {
    expect_equal(
        lapply(
            pairwiseApply(
                x,
                FUN = function(x, y) cbind(y, x),
                USE.NAMES = FALSE,
                SIMPLIFY = FALSE
            ),
            unname
        ),
        lapply(getFeaturePairs(x), unname)
    )
})

test_that("show works", {
    expect_null(show(x))
})

#' @import S7
#' @import methods
#' @rawNamespace if (getRversion() < "4.3.0") importFrom("S7", "@")
NULL

rlang::on_load({
    S7::S4_register(anansi:::AnansiWeb)
    S7::S4_register(anansi:::MultiFactor)
    S7::methods_register()
})

.onLoad <- function(...) {
    rlang::run_on_load()
}

#' anansi: Annotation-based Analysis of Specific Interactions.
#'
#' @description
#' The anansi package package provides tools to prepare and facilitate
#' integrative association analysis between the features of two data sets that
#' are known to interact.
#'
#' ## 1. Input for `anansi()` with [AnansiWeb()] and [MultiFactor()]
#' \itemize{
#'     \item [randomAnansi], [kegg_link()]: Generate example input
#'     \item [AnansiWeb], [MultiFactor]: Handle and manipulate
#'     input
#'     }
#'
#' ## 2. Output and cross-compatibility
#' \itemize{
#'     \item [`getGraph()`]: Compatibility with [igraph::igraph]
#'     \item [`plotAnansi()`]: Plot output in the style of [miaViz::miaViz]
#'     }
#'
#' ## 3. Vignettes
#' \itemize{
#'     \item
#'     \href{https://thomazbastiaanssen.github.io/anansi/articles/anansi.html}{1. Getting started with anansi}
#'     \item
#'     \href{https://thomazbastiaanssen.github.io/anansi/articles/adjacency_matrices.html}{2. Adjacency matrices}
#'     \item
#'     \href{https://thomazbastiaanssen.github.io/anansi/articles/association_testing.html}{3. Association testing}
#'     }
#'
#' @aliases anansi-package
#' @name anansi
#' @keywords internal
#' @docType package
"_PACKAGE"

#' Calculate an association network
#' @description
#' ## Description: The `anansi()` function
#' The main workspider function in the anansi package is called
#' `anansi`. It manages the individual functionalities of anansi, including
#' correlation analysis, correlation by group and differential associations.
#' @param web An `AnansiWeb` object, containing two tables with 'omics data
#' and a dictionary that links them. See `weaveWebFromTables()` for how to
#' weave a web.
#' @param metadata A vector or data.frame of categorical or continuous value
#' necessary for differential correlations. Typically a state or treatment.
#' If no argument provided, anansi will let you know and still to regular
#' correlations according to your dictionary.
#' @param groups A vector of the column names of categorical values in the
#' metadata object to control which groups should be assessed for simple
#' correlations.
#' If no argument provided, anansi will let you know and still to regular
#' correlations according to your dictionary.
#' @param formula A formula object. Used to assess differential associations.
#' @param adjust.method Method to adjust p-values for multiple comparisons.
#' `adjust.method = "BH"` is the default value. See `p.adjust()` in
#' the base R `stats` package.
#' @param verbose `Logical scalar`. Whether to print diagnostic information
#'     (Default: `TRUE`).
#' while running. Useful for debugging errors on large datasets.
#' @param return.format `Character scalar`. Should be one of `"table"`,
#'     `"list"`, or `"raw"`. Should the output of [anansi()]
#' respectively be a wide `data.frame` of results, a list containing the results
#' and input, or a list of raw output (used for testing purposes).
#' convenient use. (Default: `"table"`)
#' @param ... additional arguments (currently not used).
#' @return A list of lists containing correlation coefficients, p-values and
#'     q-values for all operations.
#' @importFrom stats model.frame
#' @export
#' @examples
#' # Load example data:
#'
#' data(FMT_data)
#'
#' # Make sure that columns are features and rows are samples.
#'
#' t1 <- t(FMT_metab)
#' t2 <- t(FMT_KOs)
#'
#' # Run anansi pipeline.
#'
#' web <- weaveWeb(
#'     cpd ~ ko,
#'     tableY = t1,
#'     tableX = t2,
#'     link = kegg_link()
#' )
#'
#' anansi_out <- anansi(
#'     web = web,
#'     formula = ~Legend,
#'     groups = "Legend",
#'     metadata = FMT_metadata,
#'     adjust.method = "BH",
#'     verbose = TRUE
#' )
#'
#'
#' library(tidyr)
#'
#' # Use tidyr to wrangle the correlation r-values to a single column
#' anansiLong <- anansi_out |>
#'     pivot_longer(starts_with("All") | contains("FMT")) |>
#'     separate_wider_delim(
#'         name,
#'         delim = "_", names = c("cor_group", "param")
#'     ) |>
#'     pivot_wider(names_from = param, values_from = value)
#'
#' # Only consider interactions where the entire model fits well enough.
#' library(ggplot2)
#' anansiLong <- anansiLong[anansiLong$full_p.values < 0.05, ]
#'
#'
#'
#' ggplot(
#'     data = anansiLong,
#'     aes(
#'         x = r.values,
#'         y = feature_X,
#'         fill = cor_group,
#'         alpha = disjointed_Legend_p.values < 0.05
#'     )
#' ) +
#'
#'     # Make a vertical dashed red line at x = 0
#'     geom_vline(xintercept = 0, linetype = "dashed", colour = "red") +
#'
#'     # Points show  raw correlation coefficients
#'     geom_point(shape = 21, size = 3) +
#'
#'     # facet per compound
#'     ggforce::facet_col(~feature_Y, space = "free", scales = "free_y") +
#'     scale_fill_manual(values = c(
#'         "Young yFMT" = "#2166ac",
#'         "Aged oFMT" = "#b2182b",
#'         "Aged yFMT" = "#ef8a62",
#'         "All" = "gray"
#'     )) +
#'     theme_bw()
#'
#' # Using miaViz style function:
#'
#' p <- plotAnansi(anansi_out,
#'     association.type = "disjointed",
#'     model.var = "Legend",
#'     fill_by = "group",
#'     signif.threshold = 0.05,
#'     x_lab = "Pearson's rho"
#' )
#' p <- p +
#'     scale_fill_manual(values = c(
#'         "Young yFMT" = "#2166ac",
#'         "Aged oFMT" = "#b2182b",
#'         "Aged yFMT" = "#ef8a62",
#'         "All" = "gray"
#'     )) +
#'     theme_bw()
#'
#' p
#'
anansi <- function(
        web,
        formula,
        groups = NULL,
        metadata = NULL,
        adjust.method = "BH",
        verbose = TRUE,
        return.format = "table",
        ...) {
    return.format <-
        match.arg(return.format, choices = c("table", "list", "raw"))
    # generate anansiYarn input object
    input <- prepInput(
        web      = web,
        formula  = formula,
        groups   = groups,
        metadata = metadata,
        verbose  = verbose
    )
    int.terms <- input$int.terms
    groups <- input$groups
    group.id <- input$group.id
    errorterm <- input$error.term
    sat_model <- input$lm.formula
    metadata <- input$metadata

    out.list <- vector(
        "list",
        length = 1 + length(group.id) + (2 * length(int.terms))
    )
    out.list[seq_along(group.id)] <- call_groupwise(
        web,
        groups,
        metadata,
        verbose
    )
    # Sort out metadata formatting for differential association testing
    meta.frame <- model.frame(formula = sat_model, cbind(x = 1, metadata))

    out.list[
        length(group.id) +
            seq_len(1L + (2L * length(int.terms)))
    ] <- anansiDiffCor(
        web,
        sat_model,
        errorterm,
        int.terms,
        meta.frame,
        verbose
    )
    if (return.format != "raw") {
        results <- result.df(out.list, Matrix::as.matrix(web@dictionary))
        results <- anansi.p.adjust(results, adjust.method)
        attr(results, "group_terms") <-
            named_group_list(group.id, groups, metadata)
        attr(results, "model_terms") <- named_term_list(int.terms, metadata)
    }
    switch(return.format,
        "table" = return(results),
        "list" = return(list(results, input = input)),
        "raw" = return(out.list)
    )
}


#' Assess formula, trim metadata and prepare output for anansi workflow
#' @description Initialize `anansiInput` component of output. Should not be
#' called by user.
#' @noRd
#'
prepInput <- function(web, formula, groups, metadata, verbose) {
    # If no metadata argument try web slot.
    if (is.null(metadata)) metadata <- web@metadata

    stopifnot(
        "No metadata argument provided or found in AnansiWeb" =
            prod(dim(metadata)) > 0
    )
    raw_terms <- terms.formula(formula, "Error", data = metadata)
    indErr <- attr(raw_terms, "specials")$Error

    groups <- check_groups(groups, raw_terms, indErr, metadata, verbose)

    all_terms <- if (is.null(indErr)) {
        labels(raw_terms)
    } else {
        labels(raw_terms)[-indErr]
    }
    sat_model <- make_saturated_model(formula, raw_terms, indErr, verbose)
    error.term <- if (is.null(indErr)) {
        NULL
    } else {
        deparse1(
            attr(raw_terms, "variables")[[1L + indErr]][[2L]],
            backtick = TRUE
        )
    }
    input <- list(
        web = web,
        lm.formula = sat_model,
        error.term = error.term,
        int.terms = all_terms,
        groups = groups[[1]],
        group.id = groups[[2]],
        metadata = `row.names<-.data.frame`(metadata, NULL)
    )

    return(input)
}

#' Check group argument.
#' @noRd
#' @importFrom stats as.formula update.formula
#'
check_groups <- function(groups, raw_terms, indErr, metadata, verbose) {
    # If user input, check it
    if (!is.null(groups)) {
        missing_groups <- !groups %in% colnames(metadata)
        stopifnot(
            "Grouping variable(s) not recognised. Please check input." = !any(
                missing_groups
            )
        )
        group.id <- check_missing_combos(metadata[groups])
        return(list(groups, group.id))
    }

    # If no input, look for categorical variables
    ind_o1 <- attr(raw_terms, "order") == 1
    if (!is.null(indErr)) {
        ind_o1[indErr] <- FALSE
    }
    if (!any(ind_o1)) {
        if (verbose) {
            message("No grouping variable found for groupwise correlations.")
        }
        return(list(NULL, 1))
    }
    groups <- labels(raw_terms)[ind_o1]

    sub_meta <- metadata[, groups, drop = FALSE]
    sub_meta <- sub_meta[, vapply(sub_meta, is.categorical, NA), drop = FALSE]

    if (NCOL(sub_meta) == 0L) {
        if (verbose) {
            message("No grouping variable found for groupwise correlations.")
        }
        return(list(NULL, 1L))
    }
    group.id <- check_missing_combos(sub_meta)
    return(list(groups, group.id))
}

#' Prepare saturated model, deal with `Error` terms.
#' @noRd
#' @importFrom stats as.formula update.formula
#'
make_saturated_model <- function(formula, raw_terms, indErr, verbose) {
    # Simple case; No random intercept; make regular saturated model
    if (is.null(indErr)) {
        sat_model <- update.formula(old = formula, ~ x * 1 * (.))
        if (verbose) {
            message(
                paste0(
                    "Fitting least-squares for following model:\n",
                    paste0(as.character(sat_model), " ", collapse = "")
                )
            )
        }
        return(sat_model)
    }

    # Case with repeated measures:
    stopifnot(
        "Only one Error() term allowed; more detected." = length(indErr) < 2
    )
    errorterm <- attr(raw_terms, "variables")[[1L + indErr]]
    sat_model <- update.formula(
        old = formula,
        new = as.formula(
            paste(
                "~",
                deparse1(errorterm[[2L]], backtick = TRUE),
                "+ x * 1 * (. -",
                deparse1(errorterm, backtick = TRUE),
                ")"
            ),
            env = environment(formula)
        )
    )
    if (verbose) {
        message(paste0(
            "Fitting least-squares for following model:\n",
            paste0(
                as.character(update.formula(
                    old = formula,
                    new = as.formula(
                        paste(
                            "~ x * 1 * (. -",
                            deparse1(errorterm, backtick = TRUE),
                            ")"
                        ),
                        env = environment(formula)
                    )
                )),
                " ",
                collapse = ""
            ),
            "\nwith '",
            deparse1(errorterm[[2L]], backtick = TRUE),
            "' as random intercept."
        ))
    }
    return(sat_model)
}

#' Is character, factor or ordered factor
#' @description wrapper around
#' `is.character(x) || is.factor(x) || is.ordered(x)`
#' @param x an object to be evaluated as being categorical
#' @returns a boolean.
#' @noRd
#'
is.categorical <- function(x) is.character(x) || is.factor(x) || is.ordered(x)

#' Return levels or 'numeric'
#' @noRd
#'
lvs_or_num <- function(x) {
    if (is.categorical(x)) {
        return(unique(as.character(x)))
    } # else
    return("numeric")
}

#' Provide name and levels for categories for group terms
#' @description convenience function to generate group terms output
#' @param g character vector of all 'All' followed by all unique terms
#' @param t character vector of term(s)
#' @param m data.frame of provided metadata
#' @noRd
#' @returns a named list of character vectors, where names are terms and
#' containing vectors are unique levels or 'numeric' if not categorical. First
#' Entry is named 'All' and contains all levels.
#'
named_group_list <- function(g, t, m) c(list(All = g), lapply(m[t], lvs_or_num))

#' Provide name and levels for categories
#' @description convenience function to generate output
#' @param t character vector of term(s)
#' @param m data.frame of provided metadata
#' @noRd
#' @returns a named list of character vectors, where names are terms and
#' containing vectors are unique levels or 'numeric' if not categorical.
#'
named_term_list <- function(t, m) {
    order_one <- t %in% colnames(m)
    f_order <- t[order_one]
    h_order <- t[!order_one]

    f_list <- lapply(m[, f_order, drop = FALSE], lvs_or_num)
    h_list <- NULL
    if (length(h_order) != 0) {
        m_num <- !unlist(lapply(m, is.categorical))
        m[m_num] <- "numeric"
        h_terms <- strsplit(h_order, split = ":", fixed = TRUE)
        h_list <- lapply(
            h_terms,
            function(x) unique(do.call(paste, c(m[x], sep = "_")))
        )
        names(h_list) <- h_order
    }

    return(c(f_list, h_list))
}

#' Check whether any combination of categories in group argument is missing
#'
#' @noRd
#'
check_missing_combos <- function(metadata) {
    group.factor <- interaction(metadata, sep = "_")
    empir.factor <- factor(group.factor)
    if ("All" %in% levels(group.factor)) {
        stop("'All' cannot be present in columns of 'group' argument.")
    }
    if (nlevels(group.factor) > nlevels(empir.factor)) {
        missing_lv <- paste(
            setdiff(
                levels(group.factor),
                levels(empir.factor)
            ),
            collapse = ", "
        )
        warning(
            "Missing combinations of categorical variables: ",
            missing_lv,
            "\n",
            "NAs introduced; Estimated effects may be unbalanced.",
            call. = FALSE
        )
    }

    group.id <- c("All", levels(empir.factor))
    return(group.id)
}

test_that("plotAnansi", {
    library(ggplot2)
    data(FMT_data)

    t1 <- t(FMT_metab)
    t2 <- t(FMT_KOs)
    web <- weaveWeb(
        cpd ~ ko,
        tableY = t1,
        tableX = t2,
        metadata = FMT_metadata,
        link = kegg_link()
    )
    out <- anansi(
        web = web,
        formula = ~Legend,
        groups = "Legend",
        adjust.method = "BH",
    )
    # Check arguments
    expect_no_error(plotAnansi(out))
    expect_error(plotAnansi(out, association.type = "wrong"))
    expect_error(plotAnansi(out, model.var = "wrong"))
    expect_warning(
        plotAnansi(out, association.type = "full", model.var = "Legend"),
        "'model.var' is ignored when 'association type' is set to full"
    )
    expect_warning(
        plotAnansi(out, signif.threshold = 0.05),
        "'signif.threshold' is ignored when 'association type' is not defined"
    )
    expect_error(
        plotAnansi(out, association.type = "emergent"),
        "'model.var' must specify a variable of the anansi model when 'association type' is set to emergent",
        fixed = TRUE
    )
    expect_error(
        plotAnansi(
            out,
            association.type = "disjointed",
            model.var = "Legend",
            shape_by = "wrong"
        ),
        "'shape_by' must be a character string specifying the name of a 'groups' term used in the original anansi call",
        fixed = TRUE
    )
    expect_no_error(plotAnansi(
        out,
        association.type = "disjointed",
        model.var = "Legend",
        shape_by = "group"
    ))
    # Check output plot
    p <- plotAnansi(
        out,
        association.type = "emergent",
        model.var = "Legend",
        fill_by = "group"
    )
    expect_length(p$guides$guides, 2)
    expect_equal(dim(p$data), c(260, 8))
    expect_false(any(is.na(p$data[["fill"]])))
})

#' Weave an AnansiWeb
#' @name weaveWeb-methods
#' @rdname weaveWeb-methods
#' @aliases weaveWeb.character weaveWeb.formula weaveWeb.MultiAssayExperiment
#' @aliases weaveWeb.TreeSumarizedExperiment weaveWeb.SingleCellExperiment
#' @seealso \itemize{
#'     \item [AnansiWeb]: For general constructor and methods.
#'     \item [kegg_link()]: For examples of input for link argument.
#' }
#'
#' @returns an `AnansiWeb` object, with sparse binary biadjacency matrix
#' with features from `y` as rows and features from `x` as columns in
#' `dictionary` slot.
#' @description
#' Generate a biadjacency matrix, linking the features between two tables.
#' Return an `AnansiWeb` object which contains all three.
#'
#' `weaveWeb()` is for general use and has flexible default settings.
#'
#' `weaveKEGG()` is a wrapper that sets `link` to `kegg_link()`.
#' All variants are special cases of `weaveWeb()`.
#' @examples
#' # Setup demo tables, see first vignette.
#' data(FMT_data)
#'
#' t1 <- t(FMT_metab)
#' t2 <- t(FMT_KOs)
#'
#' # Input objects and syntax:
#' ## define `x` and `y` as characters
#' web <- weaveWeb(
#'     x = "ko", y = "cpd", link = kegg_link(),
#'     tableX = t2, tableY = t1,
#'     metadata = NULL, verbose = TRUE
#' )
#'
#' ## define `x` and `y` with a formula
#' web2 <- weaveWeb(
#'     x = cpd ~ ko, link = kegg_link(),
#'     tableX = t2, tableY = t1,
#'     metadata = NULL, verbose = TRUE
#' )
#'
#' identical(web, web2)
#'
#' # Method for MultiAssayExperiment S4 object
#' mae <- asMAE(web)
#' weaveWeb(
#'     x = mae, link = kegg_link(),
#'     tableY = "cpd", tableX = "ko",
#'     force_new = FALSE
#' )
#'
#' # Method for TreeSummarizedExperiment S4 object
#' tse <- asTSE(web)
#' weaveWeb(
#'     x = tse, link = kegg_link(),
#'     tableY = "cpd", tableX = "ko",
#'     force_new = FALSE
#' )
#'
#' @param x,y `Character scalar`, names of feature types that should be
#'     linked. Should be found in the column names of `link`.
#' @param link One of the following:
#'     \itemize{
#'         \item `Character scalar` with value `"none"`.
#'         \item `data.frame` with two columns
#'         \item `list` with two such `data.frame`s.
#'         }
#' @param tableY,tableX A table containing features of interest. Rows should be
#'     samples and columns should be features. Y and X refer to the position of
#'     the features in a formula: Y ~ X. \cr
#'     < For Bioconductor S4 objects > \cr
#'     `Character scalar` or `numeric scalar`. Selects experiment
#'     corresponding to `tableY` and `tableX` from `experiments(x)` of
#'     `MultiAssayExperiment` object by name or index, name is recommended.
#'     (Default slots: `Y = 1L`, `X = 2L`).
#' @param metadata Optional `data.frame` of sample metadata, to be included with
#'     output. Can be accessed from `AnansiWeb` generated by `weaveWeb()` with
#'     `output@metadata`.
#' @details
#' If the `link` argument is `"none"`, all features will be considered
#' linked. If one or more `data.frame`s, colnames should be as specified in
#' `x` and `y`.
#' @param verbose `Logical scalar`. Whether to print diagnostic information
#'     (Default: `TRUE`).#' @param force_new `boolean` If x already has a
#'     dictionary `Matrix` in metadata, ignore it and generate a new object
#'     anyway? (Default: FALSE).
#' @param typeY,typeX
#' `Character scalar` or `numeric scalar`. Selects assay from experiments to
#' `tableY` and `tableX` from `experiments(x)`. (Default: `1L` - the first assay
#'  in that experiment).
#' @param experiment1,experiment2 synonymous args to `tableY,tableX` for
#'     compatibility with `mia` argument style.
#' @param assay.type1,assay.type2 synonymous args to `typeY,typeX` for
#'     compatibility with `mia` argument style.
#'
NULL

#' @rdname weaveWeb-generic
#' @name weaveKEGG
#' @export
#'
weaveKEGG <- function(x, ...) weaveWeb(x, link = kegg_link(), ...)

#' @export
S7::method(weaveWeb, S7::class_character) <- function(x, y,
    link = NULL,
    tableX = NULL,
    tableY = NULL,
    metadata = NULL,
    verbose = TRUE) {
    weaveWeb.character(x, y, link, tableX, tableY, metadata, verbose)
}

weaveWeb.character <- function(x, y,
    link = NULL,
    tableX = NULL,
    tableY = NULL,
    metadata = NULL,
    verbose = TRUE) {
    terms <- c(y, x)
    stopifnot(
        "both 'x' and 'y' terms must be provided as character" =
            is(terms, "character") && length(terms) == 2L
    )
    if (is.null(link)) {
        stop(
            "'link' argument not provided. To explicitly disable ",
            "knowledge-based selection, use link = 'none' instead. "
        )
    }
    if (identical(link, "none")) {
        return(web_missing_link(tableX, tableY, terms))
    }

    # Ensure link is a MultiFactor
    link <- MultiFactor(link, drop.unmatched = TRUE)
    # Determine required ids in order, only keep relevant elements of link.
    all_terms <- termSeq(x, y, link)
    link <- subsetByPath(link, all_terms)

    # Trim link levels and tables based on feature overlap
    if (!is.null(tableX)) {
        keep <- sort(intersect(colnames(tableX), levels(link)[[x]]))
        if (verbose && length(keep) < NCOL(tableX)) {
            message("Dropped features in tableX: ", length(keep), " remain. ")
        }
        tableX <- tableX[, keep]
        link <- trimByInput(link, tableX, x)
    }
    if (!is.null(tableY)) {
        keep <- sort(intersect(colnames(tableY), levels(link)[[y]]))
        if (verbose && length(keep) < NCOL(tableY)) {
            message("Dropped features in tableY: ", length(keep), " remain. ")
        }
        tableY <- tableY[, keep]
        link <- trimByInput(link, tableY, y)
    }
    # Construct dictionary
    d <- dictionaryMatrix(link, all_terms)
    dimnames(d) <- list(y = colnames(tableY), x = colnames(tableX))
    names(dimnames(d)) <- c(y, x)

    # Dummy tables if missing
    if (is.null(tableX) && is.null(tableY)) {
        dimnames(d) <- levels(link)[terms]
        tableY <- matrix(ncol = NROW(d), dimnames = list(NULL, rownames(d)))
        tableX <- matrix(ncol = NCOL(d), dimnames = list(NULL, colnames(d)))
    }
    # Create web
    AnansiWeb(
        tableY = as.matrix(tableY)[, rownames(d), drop = FALSE],
        tableX = as.matrix(tableX)[, colnames(d), drop = FALSE],
        dictionary = d,
        metadata = metadata
    )
}

#' @export
S7::method(weaveWeb, S7::class_formula) <- function(x,
    link = NULL,
    tableX = NULL,
    tableY = NULL,
    metadata = NULL,
    verbose = TRUE) {
    weaveWeb.formula(x, link, tableX, tableY, metadata, verbose)
}

weaveWeb.formula <- function(x,
    link = NULL,
    tableX = NULL,
    tableY = NULL,
    metadata = NULL,
    verbose = TRUE) {
    if (missing(x) || (length(x) != 3L)) {
        stop("'formula' missing or incorrect")
    }

    terms <- all.vars(x)
    if (is.null(link) || identical(link, "none")) {
        return(
            weaveWeb(
                x = terms[2],
                y = terms[1],
                link,
                tableX,
                tableY,
                metadata,
                verbose
            )
        )
    }

    link <- MultiFactor(link, drop.unmatched = TRUE)

    if (sum(terms %in% colnames(link)) != 2L) {
        stop("Variables from 'formula' not found in 'link'.")
    }

    weaveWeb(
        x = terms[2],
        y = terms[1],
        link,
        tableX,
        tableY,
        metadata,
        verbose
    )
}

#' @importFrom SummarizedExperiment colData assay assayNames
#' @importFrom MultiAssayExperiment MultiAssayExperiment experiments
#' @importFrom methods getClass
#' @export
#'
S7::method(
    weaveWeb,
    methods::getClass("MultiAssayExperiment", where = "MultiAssayExperiment")
) <- function(x,
    link = NULL,
    tableY = NULL,
    tableX = NULL,
    typeY = NULL,
    typeX = NULL,
    force_new = FALSE,
    experiment1 = NULL,
    experiment2 = NULL,
    assay.type1 = NULL,
    assay.type2 = NULL) {
    weaveWeb.MultiAssayExperiment(
        x, link, tableY, tableX, typeY, typeX, force_new,
        experiment1, experiment2, assay.type1, assay.type2
    )
}

weaveWeb.MultiAssayExperiment <- function(x,
    link = NULL,
    tableY = NULL,
    tableX = NULL,
    typeY = NULL,
    typeX = NULL,
    force_new = FALSE,
    experiment1 = NULL,
    experiment2 = NULL,
    assay.type1 = NULL,
    assay.type2 = NULL) {
    y_ids <- .test_coherent(tableY, experiment1, typeY, assay.type1)
    x_ids <- .test_coherent(tableX, experiment2, typeX, assay.type2)
    tableY <- y_ids[[1L]]
    tableX <- x_ids[[1L]]

    # Check experiments
    .test_mae_has_exp(x, tableY)
    .test_mae_has_exp(x, tableX)

    # Extract assays
    tY <- t(assay(experiments(x)[tableY], y_ids[[2L]]))
    tX <- t(assay(experiments(x)[tableX], x_ids[[2L]]))

    # Check if x already contains a dictionary
    if (!force_new) {
        m <- x@metadata

        d <- if (is.null(link)) {
            "dictionary"
        } else {
            ""
        }
        if (.check_valid_selection(link, m)) {
            d <- link
        }
        if (d %in% names(m)) {
            return(AnansiWeb(
                tableX = tX,
                tableY = tY,
                dictionary = m[[d]],
                metadata = as.data.frame(colData(x))
            ))
        }
    }
    # Else, generate web object
    weaveWeb(
        x = tableX,
        y = tableY,
        link = link,
        tableX = tX,
        tableY = tY,
        metadata = as.data.frame(colData(x))
    )
}

#' @export
#' @importClassesFrom SingleCellExperiment SingleCellExperiment
#' @importFrom SummarizedExperiment colData assay assayNames
#' @importFrom SingleCellExperiment SingleCellExperiment altExp altExpNames
#' @importClassesFrom TreeSummarizedExperiment TreeSummarizedExperiment
#' @method weaveWeb SingleCellExperiment
S7::method(
    weaveWeb,
    methods::getClass(
        "SingleCellExperiment",
        where = "SingleCellExperiment"
    ) |
        methods::getClass(
            "TreeSummarizedExperiment",
            where = "TreeSummarizedExperiment"
        )
) <- function(x,
    link = NULL,
    tableY = NULL,
    tableX = NULL,
    typeY = NULL,
    typeX = NULL,
    force_new = FALSE,
    experiment1 = NULL,
    experiment2 = NULL,
    assay.type1 = NULL,
    assay.type2 = NULL) {
    weaveWeb.SingleCellExperiment(
        x, link, tableY, tableX, typeY, typeX, force_new,
        experiment1, experiment2, assay.type1, assay.type2
    )
}

weaveWeb.SingleCellExperiment <- function(x,
    link = NULL,
    tableY = NULL,
    tableX = NULL,
    typeY = NULL,
    typeX = NULL,
    force_new = FALSE,
    experiment1 = NULL,
    experiment2 = NULL,
    assay.type1 = NULL,
    assay.type2 = NULL) {
    y_ids <- .test_coherent(tableY, experiment1, typeY, assay.type1)
    x_ids <- .test_coherent(tableX, experiment2, typeX, assay.type2)
    tableY <- y_ids[[1L]]
    tableX <- x_ids[[1L]]


    tse_list <- .get_table_from_tse(x, tableY, tableX)


    # Extract assays
    tY <- t(assay(tse_list[[1L]], y_ids[[2L]]))
    tX <- t(assay(tse_list[[2L]], x_ids[[2L]]))

    # Check if x already contains a dictionary
    if (!force_new) {
        m <- x@metadata

        d <- if (is.null(link)) {
            "dictionary"
        } else {
            ""
        }
        if (.check_valid_selection(link, m)) {
            d <- link
        }
        if (d %in% names(m)) {
            return(AnansiWeb(
                tableX = tX,
                tableY = tY,
                dictionary = m[[d]],
                metadata = as.data.frame(colData(x))
            ))
        }
    }
    # Generate web object
    weaveWeb(
        x = names(tse_list)[2L],
        y = names(tse_list)[1L],
        link = link,
        tableX = tX,
        tableY = tY,
        metadata = as.data.frame(colData(x))
    )
}

test_that("kegg wrapper is equivalent", {
    data("ec2cpd", package = "anansi", envir = environment())
    data("ec2ko", package = "anansi", envir = environment())
    expect_warning(
        web_g <- weaveWeb(cpd ~ ko, link = list(ec2ko = ec2ko, ec2cpd = ec2cpd)),
        "Argument `metadata` not provided; Please validate sample ID order.")
    expect_warning(
    web_k <- weaveKEGG(cpd ~ ko),
    "Argument `metadata` not provided; Please validate sample ID order.")
    expect_identical(web_g, web_k)
})

test_that("Swapping terms in formula is equivalent to transposition", {
    expect_warning(
        a <- weaveWeb(ko ~ cpd, link = kegg_link())@dictionary,
        "Argument `metadata` not provided; Please validate sample ID order.")
    expect_warning(
    b <- weaveWeb(cpd ~ ko, link = kegg_link())@dictionary,
    "Argument `metadata` not provided; Please validate sample ID order.")

    expect_identical(a, Matrix::t(b))
})

#' Dissociation plot
#'
#' `plotAnansi` generates an association plot from the output of
#' [anansi()] in the table format. It provides a convenient way to
#' visually assess relevant results from the anansi analysis, either in the
#' form of a dotplot or a graph.
#'
#' @param x a `data.frame` object output of [anansi()] in
#'   the table format.
#'
#' @param layout `Character scalar`. Specifies the plot layout to generate. It
#'   must be one of `c("dotplot, graph)`. (Default: `dotplot`)
#'
#' @param association.type `Character scalar`. Specifies the type of
#'   association to show in the plot. One of `"disjointed"`,
#'   `"emergent"` and `"full"`. (Default: `NULL`)
#'
#' @param model.var `Character scalar`. Specifies the name of a variable
#'   in the anansi model. It is relevant only when `association.type` is
#'   `"disjointed"` or `"emergent"`. (Default: `NULL`)
#'
#' @param group `Character scalar`. Selects one of the groups included in the
#'   anansi model. It is relevant only when `layout` is `graph`.
#'   (Default: `All`)
#'
#' @param signif.threshold `Numeric scalar`. Specifies the threshold to
#'   mark the significance of `association.type`. (Default: `NULL`)
#'
#' @param colour_by `Character scalar`. Specifies one of the `groups`
#'    terms used in the original `anansi` call, `x` by which points
#'    should be coloured. (Default: `NULL`)
#'
#' @param color_by `Character scalar`. Alias to `colour_by`.
#'
#' @param fill_by `Character scalar`. Specifies one of the `groups`
#'    terms used in the original `anansi` call, `x` by which points
#'    should be filled (Default: `"group"`)
#'
#' @param size_by `Character scalar`. Specifies one of the `groups`
#'    terms used in the original `anansi` call, `x` by which points
#'    should be sized. (Default: `NULL`)
#'
#' @param shape_by `Character scalar`. Specifies one of the `groups`
#'    terms used in the original `anansi` call, `x` by which points
#'    should be shaped. (Default: `NULL`)
#'
#' @param x_lab `Character scalar`. Specifies the label of the x axis.
#'   (Default: `"cor"`)
#'
#' @param y_lab `Character scalar`. Specifies the label of the y axis.
#'   (Default: `""`)
#'
#' @param y_position `Character scalar`. Specifies the position of the y
#'   labels. It should be either `"left"` or `"right"`.
#'   (Default: `"right"`)
#'
#' @param show.cor `Logical scalar`. Whether correlation edges should be
#'   labelled with correlation coefficients when `layout` is `graph`.
#'   (Default: `FALSE`)
#' @param ... additional parameters
#'
#' @details
#' `plotAnansi` provides a standardised method to visualise the results
#' of anansi by means of a differential association plot. The input for this
#' function should be generated from [anansi()] or
#' [anansi()], with `return.format = "table"`
#'
#' @return
#' A ggplot2 object.
#'
#' @examples
#' # Import libraries
#' library(mia)
#' library(TreeSummarizedExperiment)
#' library(MultiAssayExperiment)
#' library(ggraph)
#'
#' web <- randomWeb(n_samples = 100)
#' mae <- asMAE(web)
#'
#' # Perform anansi analysis
#' out <- weaveWeb(mae,
#'     tableY = "y", tableX = "x"
#' ) |> anansi(formula = ~group_ab)
#'
#' # Select significant interactions
#' out <- out[out$full_p.values < 0.05, ]
#'
#' # Visualise disjointed associations filled by group
#' plotAnansi(out,
#'     association.type = "disjointed",
#'     model.var = "group_ab",
#'     signif.threshold = 0.05,
#'     fill_by = "group"
#' )
#'
#' # Visualise full associations filled by group
#' plotAnansi(out,
#'     association.type = "full",
#'     signif.threshold = 0.05,
#'     fill_by = "group"
#' )
#'
#' # Visualise full associations as graph
#' plotAnansi(out,
#'     layout = "graph",
#'     association.type = "full",
#'     signif.threshold = 0.05,
#'     show.cor = TRUE
#' )
#'
#' # Visualise disjointed associations as graph
#' plotAnansi(out,
#'     layout = "graph",
#'     association.type = "disjointed",
#'     model.var = "group_ab",
#'     signif.threshold = 0.05
#' )
#'
#' @seealso [anansi()]
#' @name plotAnansi
#' @aliases plotAnansi-methods
#'
NULL

#' @name plotAnansi
#' @rdname plotAnansi
#' @export
#' @importFrom rlang .data
#' @importFrom ggplot2 ggplot aes theme guides labs geom_vline geom_point
#'     scale_x_continuous scale_y_discrete scale_alpha_manual theme_bw
#' @importFrom ggforce facet_col
#' @importFrom stats setNames
#' @importFrom S4Vectors isEmpty
S7::method(plotAnansi, S7::class_data.frame) <- function(x,
    layout = "dotplot",
    association.type = NULL,
    model.var = NULL,
    group = "All",
    signif.threshold = NULL,
    colour_by = NULL,
    color_by = colour_by,
    fill_by = "group",
    size_by = NULL,
    shape_by = NULL,
    y_position = "right",
    x_lab = "cor",
    y_lab = "",
    show.cor = FALSE) {
    # Create list of Booleans whether args are defined
    defined_args <- lapply(
        list(
            association = association.type,
            model.var = model.var,
            signif = signif.threshold
        ),
        function(x) !is.null(x)
    )
    # Check association.type
    if (defined_args[["association"]]) {
        match.arg(
            association.type,
            choices = c("disjointed", "emergent", "full")
        )
    }
    # Check model.var
    if (defined_args[["model.var"]]) {
        match.arg(model.var, names(attr(x, "model_terms")))
    }
    # Check that group is valid for graph layout
    if (layout == "graph") {
        match.arg(group, attr(x, "group_terms")$All)
    }
    # Check association.type and model.var
    if (
        defined_args[["association"]] &&
            !defined_args[["model.var"]] &&
            association.type %in% c("disjointed", "emergent")
    ) {
        stop(
            "'model.var' must specify a variable of the anansi model ",
            "when 'association type' is set to ",
            association.type,
            call. = FALSE
        )
    }
    if (
        defined_args[["association"]] &&
            defined_args[["model.var"]] &&
            association.type == "full"
    ) {
        model.var <- NULL
        warning(
            "'model.var' is ignored when 'association type' ",
            "is set to ",
            association.type,
            call. = FALSE
        )
        model.var <- NULL
    }
    # Derive p-value column from association.type and model.var
    pval <-
        paste0(c(association.type, model.var, "p.values"), collapse = "_")
    # Check x
    if (isEmpty(x)) {
        stop("'x' is an empty data.frame", call. = FALSE)
    }
    if (!all(c("feature_X", "feature_Y") %in% colnames(x))) {
        stop(
            "'x' must be the output of 'anansi' in the table format ",
            "and must contain columns 'feature_X' ,'feature_Y'",
            call. = FALSE
        )
    }
    if (!any(grepl(pval, names(x)))) {
        stop("Could not find p-values in 'x'.", call. = FALSE)
    }
    # Convert anansi wide to long format
    x <- .wide2long(x)
    # Select group for graph layout
    if (layout == "graph") {
        x <- x[x$group == group, ]
    }
    # Update colour_by if color_by is defined
    if (!is.null(color_by) && is.null(colour_by)) {
        colour_by <- color_by
    }
    # Check aesthetics
    defined_args <- c(
        defined_args,
        mapply(
            .check_aes,
            aes_name = c("colour_by", "fill_by", "size_by", "shape_by"),
            aes_var = list(colour_by, fill_by, size_by, shape_by),
            MoreArgs = list(x = x),
            SIMPLIFY = FALSE
        )
    )
    # Check signif.threshold
    if (
        defined_args[["signif"]] &&
            (!is.numeric(signif.threshold) ||
                signif.threshold < 0 ||
                signif.threshold > 1)
    ) {
        stop(
            "'signif.threshold' must be a number between 0 and 1",
            call. = FALSE
        )
    }
    if (!defined_args[["association"]] && defined_args[["signif"]]) {
        warning(
            "'signif.threshold' is ignored when ",
            "'association type' is not defined",
            call. = FALSE
        )
    }
    # Check y_position
    match.arg(y_position, choices = c("left", "right"))
    # Check show.cor
    if (!is.logical(show.cor)) {
        stop("'show.cor' must be either TRUE or FALSE.", call. = FALSE)
    }
    # Assemble plot data
    pData <- data.frame(
        x = x[["r.values"]],
        y = x[["feature_X"]],
        facet = x[["feature_Y"]],
        colour = if (defined_args[["colour_by"]]) x[[colour_by]] else NA,
        fill = if (defined_args[["fill_by"]]) x[[fill_by]] else NA,
        size = if (defined_args[["size_by"]]) x[[size_by]] else NA,
        shape = if (defined_args[["shape_by"]]) x[[shape_by]] else NA,
        alpha = if (defined_args[["signif"]]) {
            factor(x[[pval]] < signif.threshold, levels = c(TRUE, FALSE))
        } else {
            NA
        }
    )
    if (layout == "dotplot") {
        # Generate dotplot
        p <- .create_dotplot(
            pData,
            defined_args,
            association.type,
            signif.threshold,
            colour_by,
            fill_by,
            shape_by,
            size_by,
            y_position,
            x_lab,
            y_lab
        )
    } else if (layout == "graph") {
        # Generate graph plot
        p <- .create_graphplot(
            pData,
            defined_args,
            association.type,
            signif.threshold,
            show.cor
        )
    }
    return(p)
}

################################ HELP FUNCTIONS ################################
# Convert anansi wide to long format
#' @description
#' Base R pivot longer for group terms of anansi output object
#' @param x data.frame, anansi() output.
#' @return a pivoted table
#' @noRd
.wide2long <- function(x) {
    mt <- attr(x, "model_terms")
    gt <- attr(x, "group_terms")
    groups <- gt$All
    # To dodge partial matches, require front.
    gr_regex <- paste0("^", groups, "_")

    gterms <- gsub("All_", "", colnames(x)[grepl("^All_", colnames(x))])
    l <- lapply(
        X = gr_regex,
        FUN = function(y) {
            `colnames<-`(
                x[, grepl(x = colnames(x), y), drop = FALSE],
                gterms
            )
        }
    )
    d <- do.call(rbind.data.frame, l)

    f <- `row.names<-.data.frame`(
        x[,
            -unlist(
                lapply(
                    X = gr_regex,
                    FUN = function(y) grep(x = colnames(x), y)
                ),
                FALSE,
                FALSE
            ),
            drop = FALSE
        ],
        NULL
    )
    # If "All" is in group column, make it the last level.
    groups <- factor(
        rep(groups, each = NROW(x)),
        levels = c(
            groups[groups != "All"],
            groups[groups == "All"]
        )
    )
    x <- cbind(f, group = groups, d)
    # Restore terms
    x <- `attr<-`(x, "model_terms", mt)
    x <- `attr<-`(x, "group_terms", gt)
    x
}

# Check aesthetics
.check_aes <- function(x, aes_name, aes_var) {
    # Check if aesthetic is defined
    aes_defined <- !is.null(aes_var)
    # Rise exception if aesthetic is not character or not in x
    if (aes_defined && !(aes_var %in% colnames(x) && is.character(aes_var))) {
        stop(
            "'",
            aes_name,
            "' must be a character string specifying the",
            " name of a 'groups' term used in the original anansi call",
            call. = FALSE
        )
    }
    return(aes_defined)
}
# Create graph plot
#' @importFrom patchwork wrap_plots plot_layout
.create_graphplot <- function(
        pData,
        defined_args,
        association.type,
        signif.threshold,
        show.cor) {
    requireNamespace("ggraph")
    graph_list <- lapply(
        unique(pData$facet),
        .plot_facet_graph,
        pData = pData,
        defined_args = defined_args,
        association.type = association.type,
        signif.threshold = signif.threshold,
        show.cor = show.cor
    )

    p <- wrap_plots(graph_list) +
        plot_layout(guides = "collect")

    return(p)
}

#' @importFrom tidygraph tbl_graph
#' @importFrom ggraph ggraph geom_edge_link scale_edge_colour_gradient2
#'   geom_node_point geom_node_text theme_graph
.plot_facet_graph <- function(
        facet,
        pData,
        defined_args,
        association.type,
        signif.threshold,
        show.cor) {
    requireNamespace("ggraph")
    # Select group
    pData <- pData[pData$facet == facet, ]
    # Retrieve node data
    node_data <- data.frame(
        node_key = c(facet, pData$y),
        alpha = factor(c(TRUE, as.character(pData$alpha)),
            levels = c(TRUE, FALSE)
        )
    )
    # Retrieve edge data
    edge_data <- data.frame(
        from = facet,
        to = pData$y,
        cor = pData$x,
        label = if (show.cor) round(pData$x, 2) else NA
    )
    # Combine node and edge data into graph
    graph <- tbl_graph(nodes = node_data, edges = edge_data)
    # Visualise graph
    p <- ggraph(graph, layout = "kk") +
        geom_edge_link(aes(colour = .data$cor, label = .data$label),
            label_size = 3
        ) +
        scale_edge_colour_gradient2(
            low = "blue", high = "red", limits = c(-1, 1)
            )
    # Add nodes
    if (defined_args[["signif"]]) {
        p <- p + geom_node_point(aes(alpha = .data$alpha))
    } else {
        p <- p + geom_node_point()
    }
    # Add node text
    p <- p + geom_node_text(aes(label = .data$node_key), repel = TRUE)
    # Add significance legend
    if (defined_args[["association"]] && defined_args[["signif"]]) {
        p <- p +
            scale_alpha_manual(
                values = c("TRUE" = 1, "FALSE" = 1 / 3),
                paste(association.type, "association\np <", signif.threshold)
            )
    }
    # Apply graph theme
    p <- p + theme_graph()
    # Remove legend when all associations are (non-)significant
    if (length(unique(node_data$alpha)) != 2L) {
        p <- p +
            guides(alpha = "none")
    }
    return(p)
}
# Create dotplot
.create_dotplot <- function(
        pData,
        defined_args,
        association.type,
        signif.threshold,
        colour_by,
        fill_by,
        shape_by,
        size_by,
        y_position,
        x_lab,
        y_lab) {
    # Create base plot
    p <- ggplot(data = pData) +
        aes(
            x = .data$x,
            y = .data$y,
            colour = .data$colour,
            fill = .data$fill,
            shape = .data$shape,
            size = .data$size,
            alpha = .data$alpha
        ) +
        geom_vline(xintercept = 0, linetype = "dashed", colour = "red")
    # Set point size, shape and border colour if not defined
    point_args <- list()
    if (!defined_args[["size_by"]]) {
        point_args["size"] <- 3
    }
    if (!defined_args[["shape_by"]]) {
        point_args["shape"] <- 21
    }
    if (!defined_args[["colour_by"]]) {
        point_args["colour"] <- "black"
    }
    # Add points and facets
    p <- p +
        do.call(geom_point, point_args) +
        facet_col(~ .data$facet, space = "free", scales = "free_y") +
        scale_x_continuous(
            limits = c(-1, 1),
            n.breaks = 11,
            expand = c(0, 0)
        ) +
        scale_y_discrete(limits = rev, position = y_position)
    # Add significance legend
    if (defined_args[["association"]] && defined_args[["signif"]]) {
        p <- p +
            scale_alpha_manual(
                values = c("TRUE" = 1, "FALSE" = 1 / 3),
                paste(association.type, "association\np <", signif.threshold)
            )
    }
    # Add labels
    p <- p +
        theme_bw() +
        labs(
            x = x_lab,
            y = y_lab,
            fill = fill_by,
            colour = colour_by,
            shape = shape_by,
            size = size_by
        )
    # Remove legend if aesthetics and significance are not defined
    if (
        !any(unlist(
            defined_args[c("colour_by", "fill_by", "size_by", "shape_by")]
        )) &&
            !defined_args[["signif"]]
    ) {
        p <- p + theme(legend.position = "none")
    }
    # Remove legend for undefined aesthetics
    guide_names <- c("colour", "fill", "alpha")[
        !unlist(defined_args[c("colour_by", "fill_by", "signif")])
    ]
    guide_args <- setNames(
        as.list(rep("none", length(guide_names))),
        guide_names
    )
    p <- p + do.call(guides, guide_args)

    return(p)
}

#' Manages group-wise association calls
#' @description If the `groups` argument is suitable, will also run correlation
#'     analysis per group. Typically, the main `anansi()` function will run
#'     this for you.
#' @param web An `AnansiWeb` object, containing two tables with omics data and a
#'      dictionary that links them. See `weaveWebFromTables()` for how to weave
#'      a web.
#' @param groups A categorical or continuous value necessary for differential
#'     correlations. Typically a state or treatment score.
#' @param metadata A vector or data.frame of categorical or continuous value
#'     necessary for differential correlations. Typically a state or treatment
#'     score.
#' @param verbose A boolean. Toggles whether to print diagnostic information
#'     while running. Useful for debugging errors on large datasets.
#' @returns `anansiCorTestByGroup`
#' @noRd
#'
call_groupwise <- function(web, groups, metadata, verbose) {
    if (is.null(groups)) {
        group.vec <- NULL
    } else {
        group.vec <-
            apply(metadata[, groups, drop = FALSE], 1, paste, collapse = "_")
    }
    return(
        anansiCorTestByGroup(web, group.vec, verbose)
    )
}

#' Run correlations for all interacting metabolites and functions.
#' @description If the `groups` argument is suitable, will also run correlation
#'     analysis per group. Typically, the main `anansi()` function will run this
#'     for you.
#' @param web An `AnansiWeb` object, containing two tables with omics data and a
#'     dictionary that links them. See `weaveWebFromTables()` for how to weave a
#'     web.
#' @param group.vec A character vector denoting group membership. Typically a
#'     state or treatment score.
#' @param verbose A boolean. Toggles whether to print diagnostic information
#'     while running. Useful for debugging errors on large datasets.
#' @return a list of `AnansiTale` result objects, one for the total dataset and
#'     per group if applicable.
#' @seealso [anansi()]
#' @keywords internal
#'
anansiCorTestByGroup <- function(web, group.vec, verbose = TRUE) {
    # Determine all groups
    all_groups <- unique(group.vec)

    # Generate container list of suitable length for all results
    out_list <- vector(mode = "list", length = 1 + length(all_groups))
    names(out_list) <- c("All", all_groups)

    # first run for all groups together
    out_list$All <- anansiCorPvalue(
        web,
        group.bool = rep(TRUE, NROW(web@tableY)),
        verbose
    )

    if (!is.null(all_groups)) {
        # If verbose, verbalize.
        if (verbose) {
            message(paste(
                "Running correlations for the following groups:\n",
                paste(all_groups, collapse = ", ")
            ))
        }
        for (i in seq_along(all_groups)) {
            out_by_group <- anansiCorPvalue(
                web,
                group.bool = group.vec == all_groups[i],
                verbose
            )
            out_by_group@subject <- all_groups[i]
            out_list[[i + 1]] <- out_by_group
        }
    }

    # Return results
    return(out_list)
}

#' Compute r-statistics for each featureY-featureX pair in the dictionary.
#' Typically, the main `anansi()` function will run this for you.
#' @param web An `AnansiWeb` object, containing two tables with omics data and
#'     a dictionary that links them. See `weaveWebFromTables()` for how to
#'     weave a web.
#' @param group.bool A categorical or continuous value necessary for
#'     differential correlations. Typically a state or treatment score. If no
#'     argument provided, anansi will let you know and still to regular
#'     correlations according to your dictionary.
#' @param verbose A boolean. Toggles whether to print diagnostic information
#'     while running. Useful for debugging errors on large datasets.
#' @return An `AnansiTale` result object.
#' @seealso [anansi()] \cr [anansiCorTestByGroup()]
#' @importFrom stats pt
#' @keywords internal
#'
anansiCorPvalue <- function(web, group.bool, verbose) {
    # Compute correlation coefficients
    r <- anansiCor(web = web, group.bool = group.bool)

    # Compute p-value through t-statistic based on n and correlation coefficient
    n <- sum(group.bool)
    t <- abs((r * sqrt(n - 2)) / sqrt(1 - r^2))
    p <- 2 * (1 - pt(t, (n - 2)))

    # Collate correlation coefficients, p-values and q-values into an AnansiTale
    out <- AnansiTale(
        subject = "All",
        type = "r.values",
        estimates = r,
        df = n - 2,
        t.values = t,
        p.values = p
    )
    return(out)
}

#' Compute r-statistics for each featureY-featureX pair in the dictionary.
#' Typically, the main `anansi()` function will run this for you.
#' @param web An `AnansiWeb` object, containing two tables with omics data and a
#'      dictionary that links them. See `weaveWebFromTables()` for how to weave
#'      a web.
#' @param group.bool A boolean vector used to select which samples should be
#'      included in the correlations.
#' @seealso [anansi()] \cr [anansiCorTestByGroup()]
#' @return A matrix of r-statistics.
#' @importFrom stats cor
#' @keywords internal
#'
anansiCor <- function(web, group.bool) {
    # Run correlations on subsections of your data
    cors <- cor(
        x = web@tableY[group.bool, ],
        y = web@tableX[group.bool, ],
        method = "pearson",
        use = "pairwise.complete.obs"
    )
    cors[!Matrix::as.matrix(web@dictionary)] <- 0
    # set non-canonical correlations to zero using the binary adjacency matrix.
    return(cors)
}

#' make a link data.frame for biobakery mapping files input
#' @param map `Character`, result from `readLines()` on uncompressed humann
#'     mapping files.
#' @returns a two-column `data.frame` that can be converted into an adjacency
#'     matrix used as input for `weaveWeb()`.
#' @seealso [weaveWeb()]
#' @examples
#' # some dummy input, as a character vector of IDs separated by tabs.
#' x <- c("x_1\ty_1\ty_2\ty_4", "x_2\ty_1\ty_3", "x_3\ty_2\ty_y")
#' linkBiobakeryMap(x)
#' @export
#'
linkBiobakeryMap <- function(map) {
    linkmap <- strsplit(x = map, split = "\t")
    linkmap_names <- vapply(linkmap, FUN = function(x) x[1], FUN.VALUE = "")
    linkmap <- lapply(linkmap, FUN = function(x) x[-1])

    linkmap <- data.frame(
        id.x = rep(linkmap_names, vapply(linkmap, length, 1)),
        id.y = unlist(linkmap, recursive = TRUE, use.names = FALSE)
    )
    return(linkmap)
}

#' @title Extract a graph object from a MultiFactor.
#' @name getGraph.anansi::MultiFactor
#' @rdname MultiFactor-methods
#' @param x `MultiFactor`
#' @param format
#' `Character scalar`, controls output format by package name.
#' `"igraph"` and `"graph"` are supported.
#' @importFrom igraph graph_from_data_frame as_graphnel
#' @returns a specified graph object.
#' @seealso [igraph::graph_from_data_frame()] and
#'     [igraph::as_graphnel()], which are used under the hood, from
#'     [igraph::igraph()] package.
#' @examples
#' # Generate an igraph object
#' g <- getGraph(x = kegg_link(), format = "igraph")
#' plot(g)
#'
NULL

#' @export
S7::method(getGraph, MultiFactor) <- function(
        x, format = "igraph"
) `getGraph.anansi::MultiFactor`(x, format)

`getGraph.anansi::MultiFactor` <- function(x, format = "igraph") {
    g <- graph_from_data_frame(getEdgeList(x), directed = FALSE)

    switch(
        format,
        "igraph" = {},
        "graph" = g <- as_graphnel(g)
    )
    return(g)
}

#' @method getGraph list
#' @name getGraph.list
#' @rdname getGraph
#'
S7::method(getGraph, S7::class_list | S7::class_data.frame) <- function(
        x, format = "igraph"
) {
    getGraph(asMultiFactor(x), format)
}

#' Run differential correlation analysis for all interacting metabolites and
#' functions.
#' @description
#' Can either take continuous or categorical data. Typically, the main
#' `anansi()` function will run this for you.
#' @param web
#' An `AnansiWeb` object, containing two tables with omics data and a
#' dictionary that links them. See `weaveWebFromTables()` for how to weave
#' a web.
#' @param sat_model A `formula` object, containing the full model
#' @param errorterm
#' A `character vector`, containing the metadata col.name denoting repeated
#' measures.
#' @param int.terms
#' A `character vector`, containing the metadata col.names denoting
#' covariates interacting with X to be tested for differential associations.
#' @param metadata
#' A vector or data.frame of categorical or continuous value necessary for
#' differential correlations. Often a state or treatment score. If no argument
#' provided, anansi will let you know and still to regular correlations
#' according to your dictionary.
#' @param verbose A boolean. Toggles whether to print diagnostic information
#' while running. Useful for debugging errors on large datasets.
#' @return a list of `AnansiTale` result objects, one for the total model,
#' one for emergent correlations and one for disjointed correlations.
#' @importFrom stats anova lm pf residuals model.matrix.default terms.formula
#' @importFrom methods is
#' @keywords internal
#'
anansiDiffCor <- function(
        web,
        sat_model,
        errorterm,
        int.terms,
        metadata,
        verbose) {
    tY <- web@tableY
    tX <- web@tableX
    dic <- Matrix::as.matrix(web@dictionary)

    # Compute shape of model.matrix and initialize qr.mm
    base.mm <- build.mm(sat_model, metadata)

    # identify the columns of the variables that change based on X-variable
    x.fct <- get_x.fct(sat_model, errorterm)

    x.assign <- as.integer(which(x.fct[1, ] == 1))
    all.assign <- attr(base.mm, "assign")
    x.vars <- all.assign %in% x.assign
    x.int <- x.assign[-1]

    # set up F-tests
    # number of samples in full model
    n <- nrow(tY)

    # strip mm of attributes
    mm <- matrix(base.mm, ncol = NCOL(base.mm))

    # Null models for general association of y and x:
    # Remove all components with x.
    Y.TSS <- SS(fast.qr.resid(
        y = tY,
        x = mm[, !x.vars]
    ))

    # prepare output
    df_mat <- dfmat(x.assign, x.int, all.assign, x.fct, n)

    d.dim <- matrix(1, ncol = NCOL(dic), nrow = NROW(dic))
    full_model <- AnansiTale(
        subject = "full",
        type = "r.squared",
        df = df_mat[, 1],
        estimates = d.dim * Y.TSS, # start with RSS0
        f.values = d.dim,
        p.values = d.dim
    )

    disjointed <- lapply(
        seq_len(length(int.terms)),
        function(x) {
            AnansiTale(
                subject = paste("disjointed", int.terms[x], sep = "_"),
                type = "r.squared",
                df = df_mat[, x + 1],
                estimates = d.dim,
                f.values = d.dim,
                p.values = d.dim
            )
        }
    )

    emergent <- lapply(
        seq_len(length(int.terms)),
        function(x) {
            AnansiTale(
                subject = paste("emergent", int.terms[x], sep = "_"),
                type = "r.squared",
                df = df_mat[, x + 1] + c(0, -1, -1 / df_mat[1, x + 1]),
                estimates = d.dim,
                f.values = d.dim,
                p.values = d.dim
            )
        }
    )

    # Compute R^2 for full model
    full_model@estimates[dic] <- 1 -
        (unlist(lapply(
            seq_len(NCOL(tX)),
            function(x) R_full(y = x, mm, tY, tX, dic, x.fct, x.vars)
        )) /
            full_model@estimates[dic])

    # compute all disjointed R^2 values, return to matrix with
    # rows as values and columns as terms
    for (t in seq_along(x.int)) {
        i.disj <- index.disj(x = x.int[t], all.assign, x.fct)

        for (y in seq_len(NCOL(tX))) {
            # adjust the input model.matrix: multiplying relevant columns by x.
            qr.mm <- mm
            qr.mm[, x.vars] <- mm[, x.vars] * tX[, y]
            y.ind <- dic[, y]
            y.vals <- `dimnames<-`(tY[, y.ind], NULL)

            # straight to web!!
            disjointed[[t]]@estimates[dic[, y], y] <-
                R_disj(y.vals, qr.mm, i.disj)
            emergent[[t]]@estimates[dic[, y], y] <-
                R_emerg(y.vals, qr.mm, i.disj)
        }
    }
    model.list <- c(full_model, disjointed, emergent)
    # Add F and P statistics
    out.list <- lapply(model.list, get_PF, dic)

    return(out.list)
}

#' Get total sum of squares for residual
#' @noRd
#'
colwiseTSS <- function(x) apply(x, 2, function(x) sum((x - mean(x))^2))

#' fast resids
#' @noRd
#'
fast.qr.resid <- function(x, y) {
    y - crossprod(t(x), qr.coef(qr(x), y))
}

#' Get df1 & 2 for f-ratio
#' @noRd
#'
dfmat <- function(x.assign, x.int, all.assign, x.fct, n) {
    df0 <- colSums(
        !vapply(
            x.assign,
            FUN.VALUE = vector("logical", length = length(all.assign)),
            function(x) index.self.high(x, all.assign, x.fct)
        )
    )
    df1 <- c(
        sum(index.self.high(x.assign[1], all.assign, x.fct)),
        vapply(x.int, FUN.VALUE = 2, function(x) sum(all.assign %in% x))
    )
    df2 <- n - (df0 + df1)
    dfr <- df2 / df1
    return(rbind(df1, df2, dfr))
}

#' @noRd
#'
make_contrasts <- function(metadata) {
    contr.in <- NULL
    f.names <- colnames(metadata)[unlist(lapply(
        metadata,
        function(x) (is.character(x) || is.factor(x) || is.logical(x))
    ))]
    if (length(f.names) > 0) {
        contr.in <- `names<-`(
            rep("contr.sum", times = length(f.names)),
            f.names
        )
        contr.in[names(which(vapply(
            metadata,
            FUN.VALUE = FALSE,
            is.ordered
        )))] <- "contr.poly"
    }
    return(as.list(contr.in))
}

#' @noRd
#'
build.mm <- function(sat_model, metadata) {
    contr <- make_contrasts(metadata)

    return(model.matrix.default(
        sat_model,
        metadata,
        contrasts.arg = contr
    ))
}


#' Calculate sums of squares by column
#' @description Sums of Squared residuals, short for `colSums(x^2)`.
#' @noRd
#'
SS <- function(x) {
    dn <- dim(x)
    .colSums(x^2, dn[1L], dn[2L], FALSE)
}

#' Transform to odds: `x / (1-x)`.
#' @noRd
#'
oddify <- function(x) x / (1 - x)

#' Compute F and P statistic for `AnansiTale` object.
#' @description Populate `AnansiTale` object with F statistics.
#' @param object An `AnansiTale` object.
#' @param d A binary adjacency matrix, corresponding to the relevant dictionary
#' @noRd
#'
get_PF <- function(object, d) {
    object@f.values[d] <- oddify(object@estimates[d]) * object@df[3]
    object@p.values[d] <- pf(
        object@f.values[d],
        df1 = object@df[1],
        df2 = object@df[2],
        lower.tail = FALSE
    )

    return(object)
}


#' @noRd
#'
R_disj <- function(y.vals, qr.mm, i.disj) {
    # Disjointed
    mm.0 <- qr.mm[, i.disj[, 1]]
    mm.1 <- qr.mm[, i.disj[, 2]]

    # Cycle through dropping interactions with x, incl higher order.
    RSS_i0 <- SS(fast.qr.resid(y = y.vals, x = mm.0))

    # Cycle through returning interactions with x, no higher order.
    RSS_i1 <- SS(fast.qr.resid(y = y.vals, x = mm.1))

    return(1 - (RSS_i1 / RSS_i0))
}

#' @noRd
#'
R_emerg <- function(y.vals, qr.mm, i.disj) {
    # Emergent
    e.ord <- order(rowSums(qr.mm[, i.disj[, 3], drop = FALSE]))
    e.mm <- cbind(1, diff(qr.mm[e.ord, -1]))

    e.y <- diff(y.vals[e.ord, drop = FALSE])

    # use diff to look at var
    e.mm.0 <- e.mm[, i.disj[, 1]]
    e.mm.1 <- e.mm[, i.disj[, 2]]

    # Cycle through dropping interactions with x, incl higher order
    RSS_i0 <- SS(fast.qr.resid(y = e.y, x = e.mm.0))

    # Cycle through returning interactions with x, no higher order.
    RSS_i1 <- SS(fast.qr.resid(y = e.y, x = e.mm.1))

    return(1 - (RSS_i1 / RSS_i0))
}


#' @noRd
#'
R_full <- function(y, mm, tY, tX, dic, x.fct, x.vars) {
    # adjust the input model.matrix by multiplying the relevant columns by x
    qr.mm <- mm
    qr.mm[, x.vars] <- mm[, x.vars] * tX[, y]
    y.ind <- dic[, y]
    y.vals <- `dimnames<-`(tY[, y.ind], NULL)

    # Return H1 SSR
    SS(fast.qr.resid(y = y.vals, x = qr.mm))
}

#' @noRd
#'
index.self.high <- function(x, all.assign, x.fct) {
    all.assign %in%
        which(apply(
            x.fct * x.fct[, x] == x.fct[, x],
            MARGIN = 2,
            all
        ))
}

#' @noRd
#'
index.high <- function(x, all.assign, x.fct) {
    all.assign %in%
        which(
            `[[<-`(
                apply(
                    x.fct * x.fct[, x] == x.fct[, x],
                    MARGIN = 2,
                    all
                ),
                subscript = x,
                FALSE
            )
        )
}

#' @noRd
#'
index.disj <- function(x, all.assign, x.fct) {
    # first is full null,
    # second is parameter to investigate returned
    i0 <- !all.assign %in%
        which(apply(
            x.fct * x.fct[, x] == x.fct[, x],
            MARGIN = 2,
            all
        ))
    ix <- all.assign %in% x
    cbind(i0, i1 = i0 | ix, ix)
}

#' Generate x.fct table, deal with `Error` terms.
#' @noRd
#'
get_x.fct <- function(sat_model, errorterm) {
    x.fct <- attr(terms.formula(sat_model), "factors")

    if (!is.null(errorterm)) {
        x.fct <- x.fct[rownames(x.fct) != errorterm, ]
    }

    return(`dimnames<-`(x.fct, NULL))
}

#' Generate table, to compute contrasts, basis for base.mm
#' @noRd
#'
subset_metadata <- function(metadata, keep, raw_terms, indErr) {
    if (!is.null(indErr)) {
        all_terms <-
            c(
                all_terms,
                deparse1(
                    attr(raw_terms, "variables")[[1L + indErr]][[2L]],
                    backtick = TRUE
                )
            )
    }
    return(cbind(
        x = 1,
        metadata[, colnames(metadata) %in% all_terms, drop = FALSE]
    ))
}

#' MultiFactor S7 container class
#' @name MultiFactor
#' @rdname MultiFactor-class
#' @description
#' `MultiFactor` is an S7 class to organize and manage multiple sets of factors,
#' for instance when tracing or converting feature IDs across databases. Methods
#' for `MultiFactor` aim to follow `factor` behaviour.
#'
#' @details
#' The most straightforward way to construct a `MultiFactor` object is as a
#' named list of named data.frames. The columns of the data.frames indicate the
#' category of factor in that column.
#'
#' A `MultiFactor` object presents itself similar to a `data.frame`, in the
#' sense that level types can be called as columns and individual data.frame
#' components can be called as rows.
#' @slot index Named `list` of named integer data frames of at least two columns
#'     each. The column names correspond to names in the `levels` slot. Similar
#'     to `factor`s, the integers in those columns correspond to the characters
#'     in that level. Accessed through regular list methods (e.g., `[`, `[[`).
#' @slot levels `Named list of character vectors`. Accessed through `levels(x)`
#' @slot map `(sparse) Matrix` specifying which elements contain which levels.
#'     Accesses through `x@dictionary`.
#' @returns a MultiFactor object.
#' @importFrom methods getClass
#' @importFrom S7 new_class
#' @inheritParams MultiFactor-methods
#' @seealso [MultiFactor-methods()]
#' @examples
#' # Generate some random linkage input
#' x <- data.frame(
#'     a = sample(letters[seq(3)], 10, replace = TRUE),
#'     A = sample(LETTERS[seq(3)], 10, replace = TRUE)
#' )
#' MultiFactor(x)
#'
#' @export
#'
MultiFactor <- S7::new_class(
    name = "MultiFactor",
    package = "anansi",
    properties = list(
        index = S7::class_list,
        levels = S7::class_list,
        map = methods::getClass("Matrix", where = "Matrix")
    ),
    constructor = function(x, levels = NULL, drop.unmatched = FALSE) {
        if (!is(x, "anansi::MultiFactor")) {
            if (validLinkDF(x)) {
                x <- list(x = x)
            }
            stopifnot(
                "Input not correctly formatted." = all(
                    vapply(
                        as.list(x, use.names = FALSE),
                        validLinkDF,
                        NA,
                        USE.NAMES = FALSE
                    )
                )
            )
        }
        if (is(x, "anansi::MultiFactor")) {
            if (is.null(levels)) {
                levels <- levels(x)
            }
            x <- x@index
        }
        x <- checkMergers(x)
        if (drop.unmatched) {
            x <- trimMultiFactor(x)
        }
        m <- mapMultiFactor(x)

        # Integer DF Input
        if (all(vapply(x, validIntLinkDF, NA, USE.NAMES = FALSE))) {
            stopifnot(
                "Input is integers, levels must be provided. " =
                    !is.null(levels)
            )
            # Factor DF input
        } else if (all(vapply(x, validFactLinkDF, NA, USE.NAMES = FALSE))) {
            if (is.null(levels)) {
                levels <- factorInputMultiFactorLevels(x, m)
            }
            x <- listFactRefactor(x, m, levels)
            x <- lapply(x, factToIntDF)
            # Character DF input
        } else if (all(vapply(x, validCharLinkDF, NA, USE.NAMES = FALSE))) {
            if (is.null(levels)) {
                levels <- generateMultiFactorLevels(x, m)
            }
            x <- listCharToIntegers(x, m, levels)
        }
        # Get rid of row.names.
        x <- lapply(x, `row.names<-.data.frame`, value = NULL)
        S7::new_object(
            S7::S7_object(),
            index = x,
            levels = levels,
            map = m
        )
    }
)
S7::S4_register(MultiFactor)


#' @rdname MultiFactor-class
#' @name asMultiFactor
#' @param levels an optional named list of vectors of the unique values (as
#'     character strings) that x might have taken. The default is the unique set
#'     of values taken by lapply(x, as.character), sorted into increasing order
#'     of x.
#' @param drop.unmatched `Logical scalar` If `TRUE` (Default), for feature types
#'     that are seen at least twice, exclude features that only present in one
#'     of their respective link data frames.
#' @usage
#' ## Constructor for `MultiFactor` objects
#' MultiFactor(x, levels = NULL, drop.unmatched = FALSE)
#'
#' @export
#' @seealso \itemize{
#' \item [kegg_link()]: for an example of valid input.
#' }
#'
asMultiFactor <- function(x, levels = NULL, drop.unmatched = TRUE) {
    MultiFactor(x, levels, drop.unmatched)
}

#' @title AnansiWeb S7 container class
#' @rdname AnansiWeb-class
#' @name AnansiWeb
#' @description
#' `AnansiWeb` is an S7 class containing two feature tables as well as a
#' dictionary to link them. `AnansiWeb` is the main container that will
#' hold your input data throughout the `anansi` pipeline.
#'
#' Typical use of the `anansi` package will involve generating an `AnansiWeb`
#' object using the `weaveWeb()` function.
#'
#' The function `AnansiWeb()` constructs an `AnansiWeb` object from two
#' feature tables and an adjacency matrix.
#'
#' @slot tableY,tableX Two `matrix` objects of measurements, data. Rows are
#'     samples and columns are features. Access with `@tableY` and `@tableX`.
#' @slot dictionary `Matrix`, binary adjacency matrix. Optionally sparse.
#'     Typically generated using the `weaveWeb()` function. Access with
#'     `@dictionary`.
#' @slot metadata Optional `data.frame` of sample metadata. Access with
#'     `@metadata`.
#' @param tableY,tableX A table containing features of interest. Rows should be
#'     samples and columns should be features. Y and X refer to the position of
#'     the features in a formula: Y ~ X.
#' @param dictionary A binary adjacency matrix of class `Matrix`, or coercible
#'     to `Matrix`
#' @param metadata `list` of metadata. Optional.
#' @importClassesFrom Matrix Matrix
#' @importFrom Matrix Matrix drop0
#' @importFrom S4Vectors DataFrame
#' @importFrom S7 new_class
#' @export
#' @seealso \itemize{
#'  \item [AnansiWeb-methods]
#'  \item [randomAnansi]: For generation of random AnansiWeb objects.
#'  \item [kegg_link()]: For examples of input for link argument.
#' }
#' @returns an `AnansiWeb` object, with sparse binary biadjacency matrix
#' with features from `y` as rows and features from `x` as columns in
#' `dictionary` slot.
#' @usage
#' ## Constructor for `AnansiWeb` objects
#' AnansiWeb(tableX, tableY, dictionary, metadata = data.frame())
#' @examples
#'
#' # Use AnansiWeb() to construct an AnansiWeb object from components:
#' tX <- `dimnames<-`(replicate(5, (rnorm(36))),
#'     value = list(
#'         as.character(seq_len(36)),
#'         letters[1:5]
#'     )
#' )
#' tY <- `dimnames<-`(replicate(3, (rnorm(36))),
#'     value = list(
#'         as.character(seq_len(36)),
#'         LETTERS[1:3]
#'     )
#' )
#'
#' d <- matrix(TRUE,
#'     nrow = NCOL(tY), ncol = NCOL(tX),
#'
#'     # Note: Dictionary should have named dimensions
#'     dimnames = list(
#'         y_names = colnames(tY),
#'         x_names = colnames(tX)
#'     )
#' )
#' web <- AnansiWeb(tableX = tX, tableY = tY, dictionary = d)
#'
AnansiWeb <- S7::new_class(
    name = "AnansiWeb",
    package = "anansi",
    properties = list(
        tableY = S7::class_numeric | S7::class_logical,
        tableX = S7::class_numeric | S7::class_logical,
        dictionary = S7::class_any,
        metadata = S7::class_data.frame
    ),
    constructor = function(
        tableX, tableY, dictionary, metadata = data.frame()
        ) {
        # coerce
        if (!is(dictionary, "Matrix")) {
            dictionary <- drop0(Matrix(dictionary, sparse = TRUE))
        }
        if (!is(tableX, "matrix")) tableX <- as.matrix(tableX)
        if (!is(tableY, "matrix")) tableY <- as.matrix(tableY)

        # check validity
        stopifnot(
            "'tableX' and 'tableY' need same number of rows (observations)" =
                NROW(
                    tableX
                ) ==
                NROW(tableY)
        )
        stopifnot(
            "cols in 'tableY' need same amount as rows in dictionary" = NCOL(
                tableY
            ) ==
                NROW(dictionary)
        )
        stopifnot(
            "cols in 'tableX' need same amount as rows in dictionary" = NCOL(
                tableX
            ) ==
                NCOL(dictionary)
        )
        if (
            is.null(names(dimnames(dictionary))) ||
                any(names(dimnames(dictionary)) %in% "")
        ) {
            warning(
                "Dimnames of 'dictionary' were missing; Assigned 'y' and 'x'."
            )
            names(dimnames(dictionary)) <- c("y", "x")
        }
        metadata <- .check_metadata_labels( metadata, tableY, tableX )

        # return AnansiWeb
        S7::new_object(
            S7::S7_object(),
            tableY = `rownames<-`(tableY, row.names(metadata)),
            tableX = `rownames<-`(tableX, row.names(metadata)),
            dictionary = dictionary,
            metadata = as.data.frame(metadata)
        )
    }
)
S7::S4_register(AnansiWeb)

#' AnansiWeb constructor helper function
#' @noRd
.check_metadata_labels <- function(metadata, tableY, tableX) {
    ry <- if(is.null(row.names(tableY))) {
        as.character(seq_len(NROW(tableY))) } else row.names(tableY)
    rx <- if(is.null(row.names(tableX))) {
        as.character(seq_len(NROW(tableX))) } else row.names(tableX)

    id <- if(identical(ry, rx)) ry else as.character(seq_along(ry))

    rn <- id
    prepend_these <- !grepl("^anansi_ID_", rn)
    rn[prepend_these] <- paste0("anansi_ID_", rn[prepend_these])

    if(rlang::is_empty(metadata)) {
        warning(
            "Argument `metadata` not provided; Please validate sample ID order."
        )
        metadata <- data.frame(row.names = rn)
        return(metadata)
    }

    if(NROW(metadata) != length(rn)) {
        stop("Metadata must have exactly one row per sample in tableY,tableX")
        }

    row.names(metadata) <- rn

    return(metadata)
}

#' @title AnansiTale S7 container class. Not intended for general use.
#' @rdname AnansiTale
#' @name AnansiTale
#' @aliases AnansiTale-class
#' @importFrom S7 new_class
#' @slot subject A character that describes the data that was queried.
#' @slot type A character that describes type of parameter contained in the
#'     `estimates` slot. For example r.values for correlations or r.squared
#'     for models.
#' @slot df a vector of length 2, containing df1 and df2 corresponding to the
#'     F-ratio considered.
#' @slot estimates A matrix containing the estimates for the parameters named in
#'     the `type` slot.
#' @slot f.values A matrix containing the f-values, for least-squares.
#' @slot t.values A matrix containing the t-values, for correlations.
#' @slot p.values A matrix containing the p.values for the parameters named in
#'     the `type` slot.
#' @description `AnansiTale` is the main container that will hold your
#'     stats output data coming out of the `anansi` pipeline.
#' @param subject A character that describes the data that was queried.
#' @param type A character that describes type of parameter contained in the
#'     `estimates` slot. For example r.values for correlations or r.squared
#'     for models.
#' @param df a vector of length 2, containing df1 and df2 corresponding to the
#'     F-ratio considered.
#' @param estimates A matrix containing the estimates for the parameters named
#'     in the `type` slot.
#' @param f.values A matrix containing the f-values, for least-squares.
#' @param t.values A matrix containing the t-values, for correlations.
#' @param p.values A matrix containing the p.values for the parameters named in
#'     the `type` slot.
#' @returns an `AnansiTale`
#' @examples
#' AnansiTale
#'
#' @export
#'
AnansiTale <- S7::new_class(
    name = "AnansiTale",
    package = "anansi",
    properties = list(
        subject = S7::class_character,
        type = S7::class_character,
        df = S7::class_numeric,
        estimates = S7::class_numeric,
        f.values = S7::class_numeric,
        t.values = S7::class_numeric,
        p.values = S7::class_numeric
    )
)

# Find a path through different feature types.
# returns a Character vector of the ids to walk in order.
#' @importFrom igraph shortest_paths
#'
termSeq <- function(x, y, link) {
    stopifnot(
        "both 'x' and 'y' terms must be found as colnames in 'link'" = all(
            c(x, y) %in% colnames(link)
        )
    )
    g <- getGraph(link)
    sp <- igraph::shortest_paths(g, from = y, to = x, output = "vpath")
    names(unlist(sp, FALSE, FALSE)[[1]])
}

#' Find the order in which link data frames should be listed
#' @param term_list list of `Character vectors`, each with length of two.
#' @param d link@map.
#' @returns a numeric vector with order in which row data frames should be
#'     traversed.
#' @noRd
#'
stepSeq <- function(term_list, d) {
    vapply(
        term_list,
        rowsWithCol,
        d = d,
        name = FALSE,
        FUN.VALUE = 0L,
        USE.NAMES = FALSE
    )
}

subsetByPath <- function(link, all_terms) {
    term_list <- lapply(
        seq_len(length(all_terms) - 1L),
        FUN = function(x) all_terms[c(x, x + 1L)]
    )
    steps <- stepSeq(term_list, link@map)
    link@index <- link@index[steps]
    link@levels <- link@levels[all_terms]
    link@map <- mapMultiFactor(link[steps], mode = "counts")

    return(link)
}

#' Generate dictionary Matrix from link input
#' @inheritParams weaveWeb
#' @param all_terms `Character vector` of all path terms in sequence.
#'     `termSeq(x, y, link)`
#' @importMethodsFrom Matrix %&%
#' @noRd
#'
dictionaryMatrix <- function(link, all_terms) {
    term_list <- lapply(
        seq_len(length(all_terms) - 1L),
        FUN = function(x) all_terms[c(x, x + 1L)]
    )
    steps <- stepSeq(term_list, link@map)
    lv_len <- vapply(X = levels(link), FUN = length, 0L, USE.NAMES = TRUE)

    # Handle simple case of one link df first, return sparse matrix.
    if (length(steps) == 1L) {
        return(mapFromLink(
            all_terms,
            df = link@index[[steps]],
            dims = lv_len[all_terms]
        ))
    }
    lv_list <- lapply(term_list, function(x) lv_len[x])
    # Otherwise, make a list of matrices to Reduce to final dictionary
    mat_list <- mapply(
        FUN = mapFromLink,
        terms = term_list,
        df = link@index[steps],
        dims = lv_list
    )
    Reduce(Matrix::`%&%`, mat_list)
}

#' @param terms id of cols. `c(y, x)`.
#' @param df element of a `MultiFactor` object
#' @param dims length-2 integer vector of matrix dimensions.
#' @importFrom Matrix sparseMatrix
#' @returns a sparse biadjacency Matrix
#' @noRd
#'
mapFromLink <- function(terms, df, dims) {
    Matrix::sparseMatrix(i = df[[terms[1]]], j = df[[terms[2]]], dims = dims)
}

#' @description Called by weaveWeb to subset link to inly include the features
#'     found in the input table.
#' @returns a MultiFactor subsetted by relevant features
#' @param link a `MultiFactor` .
#' @param id `Character scalar`, naming the x term to be trimmed
#' @param tableID A table containing features of interest, `tableX` or `tableY`.
#' @noRd
#'
trimByInput <- function(link, tableID, id) {
    lv <- levels(link)[[id]]
    d <- link@map
    r <- rowsWithCol(d, id)
    stopifnot(
        "Feature names appeared in several index elements. " = length(r) == 1L
    )
    # Subset index by table columns
    xr <- link@index[[r]]
    x.names <- match(colnames(tableID), lv)
    xr <- xr[xr[, id] %in% x.names, ]
    xr.id <- xr[, id]

    # Subset levels
    link@levels[[id]] <- lv[sort(unique(xr.id))]
    # Reorder and replace indices
    xr[, id] <- match(xr.id, sort(unique(xr.id)))
    link@index[[r]] <- xr

    link@map <- mapMultiFactor(link@index, mode = "counts")

    return(link)
}


# Make a full web; for all vs all association testing
# Make a fully TRUE biadjacency matrix with dimensions of the two input tables.
# tableX, tableY: matrix of features of tableX.
# terms: character vector names of x & y terms
# Returns an `AnansiWeb` object with both tables and a fully `TRUE`
# (non-sparse) matrix from the `Matrix` package.
#' @importFrom Matrix Matrix
#'
web_missing_link <- function(tableX, tableY, terms, metadata = NULL) {
    d <- Matrix(
        data = TRUE,
        nrow = NCOL(tableY),
        ncol = NCOL(tableX),
        dimnames = list(sort(colnames(tableY)), sort(colnames(tableX)))
    )
    names(dimnames(d)) <- rev(terms)

    AnansiWeb(
        tableY = as.matrix(tableY)[, rownames(d)],
        tableX = as.matrix(tableX)[, colnames(d)],
        dictionary = d,
        metadata = metadata
    )
}

# TRUE if i can select in x
# i: Character or numeric scalar. Index to check.
# x: object to check i in
# returns TRUE if i selects in x, FALSE otherwise
#
.check_valid_selection <- function(i, x) {
    # Need to be length 1.
    if (length(i) != 1L) FALSE

    # If numeric, needs to be within element length of x
    if (is.numeric(i)) i <= length(x)
    # If character, x needs to be named and i needs to be within those names
    if (is.character(i)) {
        if (is.null(names(x))) FALSE

        !is.na(match(i, names(x)))
    }
    # If that didn't work, invalid selection. return FALSE.
    FALSE
}

.check_fixed_args <- function(kwargs) {
    # Check fixed arguments
    fixed_args <- c("web", "metadata")
    remove <- names(kwargs) %in% fixed_args
    # If fixed arguments in kwargs, remove them
    if (any(remove)) {
        removed <- paste0(
            names(kwargs[remove]),
            sep = "'",
            collapse = ", '"
        )
        warning(
            "The arguments '",
            removed,
            " should not be used, ",
            "as they are extracted from 'x'.",
            call. = FALSE
        )
    }
    kwargs <- kwargs[!remove]
    return(kwargs)
}

#' @noRd
.test_coherent <- function(tab, exp, tab.type, ass.type) {
    e_out <- unique(c(tab, exp))
    if (length(e_out) == 0L) {
        e_out <- 1L
    }
    stopifnot(
        "args 'tableY,X' cannot contradict args 'experiment1,2'." =
            length(e_out) == 1L
    )

    t_out <- unique(c(tab.type, ass.type))
    if (length(t_out) == 0L) {
        t_out <- 1L
    }
    stopifnot(
        "args 'typeY,X.' cannot contradict args 'assay.type1,2'." =
            length(t_out) == 1L
    )

    return(list(e_out, t_out))
}

#' @importFrom SummarizedExperiment assay assayNames
#' @importFrom SingleCellExperiment altExp altExpNames
.get_table_from_tse <- function(tse, y_id, x_id) {
    all_assays <- c(assayNames(tse), altExpNames(tse))
    if (is.numeric(y_id)) {
        y_id <- all_assays[y_id]
    }
    if (is.numeric(x_id)) {
        x_id <- all_assays[x_id]
    }


    tab_list <- lapply(
        c(y_id, x_id),
        FUN = function(id) {
            if (!id %in% all_assays) {
                stop(
                    "no assay with name ", id,
                    " found.", call. = FALSE
                )
            }
            if (id %in% assayNames(tse)) {
                return(tse)
            } else {
                return(altExp(tse, id))
            }
        }
    )
    names(tab_list) <- c(y_id, x_id)

    return(tab_list)
}

#
.test_mae_has_exp <- function(x, experiment) {
    if (is.numeric(experiment) && experiment > length(experiments(x))) {
        stop(
            "'", deparse(substitute(experiment)), "' is greater than the ",
            "number of experiments in MAE object.",
            call. = FALSE
        )
    }
    if (
        !(
            is.character(experiment) && experiment %in% names(experiments(x)) ||
            is.numeric(experiment) && experiment <= length(experiments(x))
        )
    ) {
        stop(
            "'",
            deparse(substitute(experiment)),
            "' ",
            "must be numeric or character value specifying ",
            "experiment in experiment(x).",
            call. = FALSE
        )
    }
    obj <- x[[experiment]]
    if (!(is(obj, "SummarizedExperiment"))) {
        stop(
            "The class of experiment specified by ",
            deparse(substitute(experiment)),
            " must be 'SummarizedExperiment'.",
            call. = FALSE
        )
    }
    return(NULL)
}

#' @noRd
#' @importFrom Matrix sparseMatrix
#' @description
#' Helper function that takes an MultiFactor and returns a sparse biadjacency
#' Matrix with link df names as rownames and id names as colnames. Called
#' internally.
#' @param x a named list of data frames with named character columns.
#' @param mode `Character scalar` One of `counts`, `binary` or `pattern`.
#'     Determines cell content of Matrix:  unique counts, 0/1 or or otherwise a
#'     sparse pattern Matrix.
#' @returns a sparse biadjacency Matrix with link df names as rownames and id
#'     names as colnames. Values count unique features in that position.
#'
mapMultiFactor <- function(x, mode = "counts") {
    # Some flexibility in input
    if (is(x, "anansi::MultiFactor")) {
        x <- x@index
    }
    mode <- match.arg(mode, choices = c("counts", "binary", "pattern"))
    all_names <- lapply(x, base::names)
    i <- factor(
        rep(
            names(all_names),
            vapply(all_names, length, 1, USE.NAMES = FALSE)
        ),
        levels = names(all_names)
    )
    j <- factor(
        unlist(all_names, use.names = FALSE),
        levels = unique(unlist(all_names, use.names = FALSE))
    )

    # mx is a vector of length i that determines the values of sparse Matrix.
    mx <- switch(mode,
        "counts" = unlist(
            lapply(x, function(y) {
                lapply(y, function(z) length(unique(z)))
            }),
            use.names = FALSE
        ),
        "binary" = 1L,
        "pattern" = TRUE
    )

    return(
        Matrix::sparseMatrix(
            i = i,
            j = j,
            x = mx,
            dimnames = list(
                levels(i),
                levels(j)
            )
        )
    )
}

#' @importFrom Matrix which tcrossprod
#' @noRd
#'
checkMergers <- function(link, verbose = TRUE) {
    d <- mapMultiFactor(link, mode = "binary")
    dm <- rep(NROW(d), 2)
    m <- Matrix::which(.row(dm) < .col(dm) & Matrix::tcrossprod(d) >= 2L, TRUE)
    if (NROW(m) == 0L) {
        # No duplicates, all good.
        return(link)
    }
    if (verbose) {
        message(
            "Duplicate id pairs detected in elements: ",
            apply(m, 1L, FUN = function(x) rownames(d)[x], simplify = FALSE),
            "\nAttempting to solve with rbind()...\n"
        )
    }
    # Otherwise, attempt to fix
    mergeElements(link, d, m)
}

#' @noRd
#'
mergeElements <- function(link, d, m) {
    dupeList <- apply(m, 1L, FUN = function(x) rownames(d)[x], simplify = FALSE)

    full_match <- vapply(
        dupeList,
        function(x) {
            Reduce(
                identical,
                lapply(link[x], function(y) {
                    sort(colnames(y))
                })
            )
        },
        FUN.VALUE = FALSE
    )
    if (!all(full_match)) {
        stop(
            "Cannot safely merge elements, names do not fully match.\n",
            "Issue found in the following pairs of elements:\n",
            apply(
                m[!full_match, , drop = FALSE],
                1L,
                FUN = function(x) rownames(d)[x],
                simplify = FALSE
            )
        )
    }
    # Collect all duplicates, they could be different sets of duplications.
    all_dupes <- unique(c(m))
    dupe_set <- vector("list", 1L)
    # as long as we have unaccounted duplicates, add to the dupe set.
    i <- 1L
    while (length(all_dupes) > 0) {
        xx <- all_dupes[1]
        mx <- m[m[, 1] == xx | m[, 2] == xx, ]
        x_dupes <- unique(c(mx))
        dupe_set[[i]] <- x_dupes
        i <- 1 + 1L
        # Remove
        all_dupes <- all_dupes[!all_dupes %in% x_dupes]
    }
    # merge and replace
    index_merge <- vector("list", length(dupe_set))
    for (i in seq_along(dupe_set)) {
        index_merge[[i]] <- unique(do.call(
            rbind.data.frame,
            c(
                link[dupe_set[[i]]],
                make.row.names = FALSE
            )
        ))
    }
    names(index_merge) <- paste0("merged_", seq_along(index_merge))
    link <- c(link[-unique(c(m))], index_merge)
    return(link)
}

#' @noRd
#' @param x a list in `MultiFactor` formatting.
#' @description Called by `MultiFactor()` if `drop.unmatched` is `TRUE`. Runs
#'     part of `subset` method.
#' @returns a subsetted list with `MultiFactor`.formatting.
#' @importFrom Matrix colSums
#'
trimMultiFactor <- function(x) {
    # Determine positions of feature names that occur in several edge link dfs
    m <- mapMultiFactor(x, mode = "pattern")
    jj <- colnames(m)[Matrix::colSums(m) > 1]

    # Sequentially subset over feature names
    for (j in jj) {
        # Select all those data frames where that term is mentioned
        ii <- rowsWithCol(m, j, FALSE)
        keep <- Reduce(intersect, lapply(x[ii], base::`[[`, j))

        # Filter feature ids in each df to only universally shared ones.
        x[ii] <- lapply(x[ii], function(df) {
            return(df[df[[j]] %in% keep, ])
        })
    }
    return(x)
}


#' @noRd
#' @description
#' helper function to make levels named list for MultiFactor. Not intended to be
#' called directly.
#' @details Especially for large input with many repeated features, it can be
#'     much more efficient to provide levels if known.
#' @param x a named list of data frames with named character columns.
#' @param m Matrix resulting from `mapMultiFactor(x)`
#' @returns a named list of levels.
#'
generateMultiFactorLevels <- function(x, m) {
    lv_names <- colnames(m)
    lev_out <- lapply(
        seq_along(lv_names),
        function(y) {
            unique(
                unlist(
                    lapply(
                        x[rowsWithCol(m, y, FALSE)],
                        function(z) unique(z[[lv_names[y]]])
                    ),
                    FALSE,
                    FALSE
                )
            )
        }
    )
    names(lev_out) <- lv_names
    lev_out
}

#' @noRd
#' @description
#' helper function to make levels named list for MultiFactor. Not intended to be
#' called directly.
#' @details Especially for large input with many repeated features, it can be
#'     much more efficient to provide levels if known.
#' @param x a named list of data frames with named character columns.
#' @param m Matrix resulting from `mapMultiFactor(x)`
#' @returns a named list of levels.
#'
factorInputMultiFactorLevels <- function(x, m) {
    lv_names <- colnames(m)
    lev_out <- lapply(
        seq_along(lv_names),
        function(y) {
            unique(
                unlist(
                    lapply(
                        x[rowsWithCol(m, y, FALSE)],
                        function(z) levels(z[[lv_names[y]]])
                    ),
                    FALSE,
                    FALSE
                )
            )
        }
    )
    names(lev_out) <- lv_names
    lev_out
}


#' @noRd
#' @description
#' Converts input relational data for MultiList into respective from character
#' to integer, according to ranking in overarching levels.
#' @param x list of character data frames
#' @param m map matrix of x
#' @param l levels(x)
#' @returns list of integer data frames accordng to levels.
#'
listCharToIntegers <- function(x, m, l) {
    lv_names <- colnames(m)
    for (id in seq_along(lv_names)) {
        idx <- rownames(m)[m[, id] != 0]

        x[idx] <- lapply(x[idx], charToIntDF, id = lv_names[id], r = l[[id]])
    }
    x
}

#' @noRd
#' @description
#' Converts input relational data for MultiList into respective from character
#' to integer, according to ranking in overarching levels.
#' @param x list of character data frames
#' @param m map matrix of x
#' @param l levels(x)
#' @returns list of integer data frames accordng to levels.
#'
listFactRefactor <- function(x, m, l) {
    lv_names <- colnames(m)
    for (i in seq_along(lv_names)) {
        idx <- rownames(m)[m[, i] != 0]
        x[idx] <- lapply(x[idx], factorToMF, id = lv_names[i], r = l[[i]])
    }
    x
}


#' @param x `data frame`, input
#' @param id `Character scalar`, name of column in x.
#' @param r reference levels
#' @noRd
#'
charToIntDF <- function(x, id, r) {
    x[[id]] <- match(x[[id]], r)
    return(x)
}

#' @param x `data frame`, input
#' @param id `Character scalar`, name of column in x.
#' @param r reference levels
#' @importFrom forcats lvls_expand
#' @noRd
#'
factorToMF <- function(x, id, r) {
    x[[id]] <- forcats::lvls_expand(x[[id]], r)
    return(x)
}

#' @param x `data frame`, input
#' @noRd
#'
factToIntDF <- function(x) {
    x[] <- lapply(x, as.integer)
    return(x)
}

#' @noRd
#' @description not intended for direct use.
#' @param id feature name, one of `colnames(x)`.
#' @param x named list of data frames with `id %in% colnames()` of those data
#'     frames.
#'
lv_list_char <- function(id, x) {
    sort(
        unique(unlist(
            lapply(x, function(y) {
                unique(y[[id]])
            }),
            recursive = FALSE,
            use.names = FALSE
        ))
    )
}


#' @noRd
#' @description
#' Based on base::factor object validation.
#'
validLevels <- function(levs) {
    if (
        any(vapply(
            levs,
            function(x) any(!is.character(x)),
            NA,
            USE.NAMES = FALSE
        ))
    ) {
        return("factor levels must be \"character\"")
    }
    if (
        any(
            d <- as.logical(vapply(levs, anyDuplicated, 1, USE.NAMES = FALSE))
        )
    ) {
        return(paste("duplicated factor levels in level number(s)", which(d)))
    }
    ## 'else' ok :
    TRUE
}

#' Is this a data.frame with at least two columns, that all are named?
#' @noRd
validLinkDF <- function(x) {
    is.data.frame(x) &&
        NCOL(x) %in% c(2L, 3L) &&
        length(colnames(x)) == NCOL(x)
}

#' @noRd
#' @description Based on `base::factor` object validation.
#' @returns `Logical scalar`, TRUE if valid.
#'
validIntLinkDF <- function(x) {
    validLinkDF(x) &&
        all(vapply(x, is.numeric, NA, USE.NAMES = FALSE))
}

#' @noRd
#' @description Based on `base::factor` object validation.
#' @returns `Logical scalar`, TRUE if valid.
#'
validCharLinkDF <- function(x) {
    validLinkDF(x) &&
        all(vapply(x, is.character, NA, USE.NAMES = FALSE))
}

#' @noRd
#' @description Based on `base::factor` object validation.
#' @returns `Logical scalar`, TRUE if valid.
#'
validFactLinkDF <- function(x) {
    validLinkDF(x) &&
        all(vapply(x, is.factor, NA, USE.NAMES = FALSE))
}

#' Simplified snippet of the Krebs cycle
#' @name krebs
#' @description
#' Enzymes and metabolites that make up the Krebs cycle. Features share a row
#' when they play a role in the same reaction.
#' Used in vignettes, exported to facilitate user exploration.
#' @format A `data.frame` of two factors, named `"Enzyme"` and `"Metabolite"`,
#'     which contain these respective components of the krebs cycle.
#' @source Generated by hand based on the krebs cycle.
#' @seealso [krebsDemoWeb()]
#' @importFrom utils data
#' @usage data("krebs", package = "anansi")
#'
"krebs"

#' @rdname kegg_link
#' @format `ec2ko`: a `data.frame` of two columns, named `"ec"`
#'     and `"ko"`. The IDs refer to KEGG orthologues. Enzyme commission numbers,
#'     ecs, typically describe reactions captured by them.
#'
#' @source `ec2ko`: Adapted from <https://www.genome.jp/kegg/>, using
#'     `KEGGREST`. Script to generate available in example.
#' @usage data("ec2ko", package = "anansi")
"ec2ko"

#' @rdname kegg_link
#' @format `ec2cpd`: a `data.frame` of two columns, named `"ec"` and `"cpd"`.
#'     The IDs refer to compounds in the KEGG database. Enzyme commission
#'     numbers, ecs, typically describe reactions either producing or requiring
#'     them.
#' @source `ec2cpd`: Adapted from <https://www.genome.jp/kegg/> using
#'     `KEGGREST`. Script to generate available in example.
#' @usage data("ec2cpd", package = "anansi")
#'
"ec2cpd"

#' Use linking data from the KEGG database.
#' @description
#' `kegg_link()` is a convenience function to return a list containing two
#' `data.frame`s; `ec2cpd` and `ec2ko`. This will be their most likely use.
#' `ec2cpd` and `ec2ko` are two `data.frame`s, used to link ko, ecs and cpd
#' identifiers in the KEGG database.
#'
#' @returns `kegg_link()` returns a list containing the two aforementioned
#'     data.frames, `ec2cpd` and `ec2ko`.
#' @examples
#' kegg_link()
#'
#' # Generate ec2ko and ec2cpd:
#' # Don't download during tests. set to `TRUE` to download.
#' dry_run <- TRUE
#'
#' if (!dry_run) {
#'     ec2ko <- KEGGREST::keggLink("ec", "ko")
#'     ec2ko <- data.frame(
#'         ec = gsub("ec:", "", x = ec2ko, fixed = TRUE),
#'         ko = gsub("ko:", "", x = names(ec2ko), fixed = TRUE),
#'         row.names = NULL
#'     )
#'
#'     ec2cpd <- KEGGREST::keggLink("ec", "cpd")
#'     ec2cpd <- data.frame(
#'         ec = gsub("ec:", "", x = ec2cpd, fixed = TRUE),
#'         cpd = gsub("cpd:", "", x = names(ec2cpd), fixed = TRUE),
#'         row.names = NULL
#'     )
#' }
#'
#' @export
#'
kegg_link <- function() {
    local({
        data("ec2ko", package = "anansi", envir = environment())
        data("ec2cpd", package = "anansi", envir = environment())
        return(list(ec2ko = get("ec2ko"), ec2cpd = get("ec2cpd")))
    })
}

#' Snippet of the CLR-transformed hippocampal metabolomics data from the FMT
#'     Aging study.
#'
#' @format A matrix object with three rows, compounds, and 36 columns, samples.
#' @source \doi{10.1038/s43587-021-00093-9}
#' @usage data("FMT_data", package = "anansi")
#'
"FMT_metab"

#' Snippet of the CLR-transformed inferred functional data from the FMT Aging
#'     study.
#'
#' @description Piphillin was used to infer functions from the 16S sequencing
#'     data in terms of KOs. Unfortunately, the Piphillin algorithm is
#'     proprietary and has since been taken down.
#'
#' @format A marix object with 6468 rows, KOs, and 36 columns, samples.
#' @source \doi{10.1038/s43587-021-00093-9}
#' @usage data("FMT_data", package = "anansi")
#'
"FMT_KOs"

#' Snippet of the metadata from the FMT Aging study.
#'
#' @description There were three treatment groups in the study that all received
#'     faecal microbiota transplantation (FMT). Young mice that received FMT
#'     from young mice (Young yFMT), aged mice that received FMT from aged mice
#'     (Aged oFMT) and aged mice that received FMT from young mice (Aged yFMT).
#' @format A data.frame object with 36 rows, samples, and two columns, denoting
#'     sample ID and treatment group, respectively.
#' @source \doi{10.1038/s43587-021-00093-9}
#' @usage data("FMT_data", package = "anansi")
#'
"FMT_metadata"

#' Methods for MultiFactor S7 container class
#' @name MultiFactor-methods
#' @rdname MultiFactor-methods
#' @aliases levels.anansi::MultiFactor
#' @aliases unfactor unfactor,anansi::MultiFactor-method
#' @description \describe{
#'     \item{`droplevels()` }{`droplevels(MultiFactor)` returns a `MultiFactor`
#'     with unused levels removed, Analogous to the `factor` method.}
#'     }
#' @examples
#' # Setup
#' x <- MultiFactor(kegg_link())
#' x
#'
#' # Basic properties
#' dim(x)
#' dimnames(x)
#'
#' # Factor-like properties
#' head(levels(x)$ko)
#' droplevels(x)
#' head(unfactor(x)$ec2ko)
#'
#' # Extract common output formats
#' getEdgeList(x)
#'
#' @param x `MultiFactor` on which the method should be applied, or, in
#'     case of the constructor `MultiFactor()`, a named `list` of data.frames
#'     with two named columns each, where elements that share a row indicates
#'     thet are adjacent.
#' @param ... `i,j` indices specifying elements to extract or replace. Indices
#'     are numeric or character vectors or empty (missing) or NULL. Numeric
#'     values are coerced to integer or whole numbers as by as.integer or for
#'     large values by trunc (and hence truncated towards zero). Character
#'     vectors will be matched to the names of the object.
#' @param value Replacement value, typically of same type as that which is to be
#'     replaced.
#' @param exclude `NULL` or `Named character list` of similar structure as
#'     `levels(MultiFactor)`. Which levels to drop from output.
#' @param select `NULL` or `Named character list` of similar structure as
#'     `levels(MultiFactor)`. Which levels to keep in output.
#' @param use.names,ignore.mcols For compatibility, not used.
#' @details Only one of `select` and `exclude` should be provided, as they are
#'     each others complement.
#' @examples
#' droplevels(x, exclude = list(ko = "K00001"))
#' droplevels(x, select = list(ko = "K00001"))
#' @returns A MultiFactor
NULL

#' @export
#'
S7::method(getEdgeList, MultiFactor) <- function(x) {
    as.data.frame(do.call(rbind, base::names(x)))
}

#' @export
#'
S7::method(dim, MultiFactor) <- function(x) dim(x@map)


#' @export
#'
S7::method(names, MultiFactor) <- function(x) {
    lapply(x@index, base::names)
}

#' @export
#'
S7::method(dimnames, MultiFactor) <- function(x) {
    dimnames(x@map)
}

#' @importMethodsFrom methods show
#' @export
#'
S7::method(show, MultiFactor) <- function(object) {
    cat(
        "An ", paste(class(object), collapse = " "),
        ",\n    ", NCOL(object),
        " feature types across ",
        NROW(object),
        " edge lists.\n\n",
        sep = ""
    )
    Matrix::printSpMatrix(object@map)

    cat(
        "\nValues represent unique feature names in that edge list.\n\n",
        "Levels:\n\n",
        sep = ""
    )
    id_w <- max(nchar(colnames(object)))
    nm_w <- max(nchar(nlevels(object)))
    for (id in colnames(object)) {
        num_lvs <- length(levels(object)[[id]])
        cat(
            format(id, width = id_w),
            " : ",
            format(num_lvs, width = nm_w),
            " Levels: ",
            sep = ""
        )

        if (num_lvs > 4L) {
            cat(
                levels(object)[[id]][1],
                levels(object)[[id]][2],
                "...",
                levels(object)[[id]][num_lvs],
                "\n",
                sep = " "
            )
        } else {
            cat(levels(object)[[id]], "\n", sep = " ")
        }
    }
    invisible(NULL)
}

#' @export
#'
S7::method(print, MultiFactor) <- function(x, ...) show(x)


#' @importMethodsFrom S4Vectors unfactor
#' @importFrom S4Vectors unfactor
#' @export
#'
S7::method(unfactor, MultiFactor) <-
    function(x, use.names = TRUE, ignore.mcols = TRUE) {
        lv <- levels(x)
        ns <- rownames(x)
        x <- x@index

        x[] <- lapply(x, function(df) {
            for (id in names(df)) {
                df[, id] <- lv[[id]][df[, id]]
            }
            return(df)
        })

        return(x)
    }

#' @export
#' @importMethodsFrom S4Vectors droplevels
#'
S7::method(droplevels, MultiFactor) <- function(
        x, ..., exclude = NULL, select = NULL
) {
    stopifnot(
        "Only one of 'exclude' and 'select' may be provided" = sum(
            is.null(exclude),
            is.null(select)
        ) >
            0L
    )
    stopifnot("'x' is not a MultiFactor." = is(x, "anansi::MultiFactor"))
    # Section 1. Trimming the indices by user input
    lvs <- levels(x)
    d <- x@map
    if (!is.null(exclude)) {
        stopifnot(
            "`'exclude' must be a named list of character vectors ." = is.list(
                exclude
            ) &&
                any(names(exclude) %in% names(lvs))
        )
        # Names not mentioned will be left alone.
        jj <- intersect(names(exclude), names(lvs))
        for (j in jj) {
            ex.ind <- match(exclude[[j]], lvs[[j]], nomatch = 0L)
            ii <- rowsWithCol(d, j)
            for (i in ii) {
                x@index[[i]] <- x@index[[i]][!x@index[[i]][, j] %in% ex.ind, ]
            }
        }
    } else if (!is.null(select)) {
        stopifnot(
            "'select' arg must be a named list of character vectors." = is.list(
                select
            ) &&
                any(names(select) %in% names(lvs))
        )
        jj <- intersect(names(select), names(lvs))
        for (j in jj) {
            ex.ind <- match(select[[j]], lvs[[j]], nomatch = 0L)
            ii <- rowsWithCol(d, j)
            for (i in ii) {
                x@index[[i]] <- x@index[[i]][x@index[[i]][, j] %in% ex.ind, ]
            }
        }
    }
    # Section 2. Trimming the levels by indices. .
    for (lv in names(lvs)) {
        # Loop over cols. First determine which rows are relevant per col/type
        rs <- rowsWithCol(d, lv, names = TRUE)
        x_index <- lapply(x@index[rs], `[[`, lv)
        # Get unique feature names in that type and are within levels.
        x_tot <- unique(unlist(x_index, use.names = FALSE))
        keep_ix <- which(seq_along(lvs[[lv]]) %in% x_tot)
        # Keep levels that show up in data
        x@levels[[lv]] <- lvs[[lv]][keep_ix]
        # Update indices to reflect fewer level names.
        for (r in rs) {
            x@index[[r]][, lv] <- match(x_index[[r]], table = keep_ix)
        }
    }
    x@map <- mapMultiFactor(x@index, mode = "counts")
    return(x)
}


#' @export
S7::method(levels, MultiFactor) <- function(x) `levels.anansi::MultiFactor`(x)

#' @export
`levels.anansi::MultiFactor` <- function(x) {
    x@levels
}

local({
    S7::method(`[`, MultiFactor) <- function(x, i, j, ..., drop = TRUE) {
        if (!all(names(sys.call()) %in% c("", "drop"))) {
            warning("named arguments other than 'drop' are discouraged")
        }
        raw_call <- rlang::call_match(
            dots_expand = FALSE, defaults = TRUE
        )


        do.call(
            ".sub_MultiFactor", rlang::call_args(raw_call),
            quote = FALSE, envir = rlang::caller_env()
        )
    }


    S7::method(`[[`, MultiFactor) <- function(x, i, ...) {
        raw_call <- rlang::call_match(
            dots_expand = FALSE, defaults = TRUE, fn = .subsub_MultiFactor
        )

        do.call(".subsub_MultiFactor", rlang::call_args(raw_call),
                envir = rlang::caller_env()
        )
    }
})

#' @export
`[<-.anansi::MultiFactor` <- function(x, i, j, ..., value) {
    if (!all(names(sys.call()) %in% c("", "value"))) {
        warning("named arguments are discouraged")
    }

    raw_call <- rlang::call_match(
        dots_expand = FALSE, defaults = TRUE, fn = .sub_rep_MultiFactor
    )
    do.call(".sub_rep_MultiFactor", rlang::call_args(raw_call),
            envir = rlang::caller_env()
    )
}

#' @export
`[[<-.anansi::MultiFactor` <- function(x, i, ..., value) {
    if (!all(names(sys.call()) %in% c("", "value"))) {
        warning("named arguments are discouraged")
    }
    raw_call <- rlang::call_match(
        dots_expand = FALSE, defaults = TRUE, fn = .sub_sub_rep_MultiFactor
    )
    do.call(".sub_sub_rep_MultiFactor", rlang::call_args(raw_call),
            envir = rlang::caller_env()
    )
}



.sub_MultiFactor <- function(x, i, j, drop = TRUE) {
    missing_i <- rlang::is_missing(i)
    missing_j <- rlang::is_missing(j)
    missing_x <- rlang::is_missing(x)
    dot_len <- sum(!missing_i, !missing_j)

    stopifnot("Too many arguments provided" = dot_len %in% seq(0L, 2L, 1L))
    # missing_i <- rlang::is_missing(dot_args[[1L]])

    if (dot_len == 0L) {
        return(x)
    }

    d <- x@map
    l <- levels(x)
    x <- x@index

    if (!missing_i) {
        ii <- rownames(d[i, , drop = FALSE])
        x <- x[ii]
    }
    if (dot_len == 2L) {
        if (!missing_i) ii <- rownames(d[i, , drop = FALSE])
        if (!missing_j) jj <- colnames(d[, j, drop = FALSE])

        if (missing_i) {
            ii <- rowsWithCol(d, jj, FALSE)
            x <- lapply(x[ii], `[`, i = jj)
        } else if (missing_j) {
            x <- x[ii]
        } else {
            x <- lapply(x[ii], `[`, i = jj)
        }
    }

    if (drop) {
        return(x)
    }

    MultiFactor(x, levels = l)
}


#' @importFrom rlang is_missing
.subsub_MultiFactor <- function(x, i) {
    # Empty returns self
    if (rlang::is_missing(i)) {
        return(x)
    }
    i <- i[[1L]]
    d <- x@map

    # If i can't index d, return NULL

    if (!all(i %in% colnames(d))) {
        if (anyNA(colnames(d)[i]) || length(colnames(d)[i]) != length(i)) {
            return(NULL)
        }
    }
    # Otherwise, return selected elements.
    ii <- rowsWithCol(d, i, FALSE)
    .sub_MultiFactor(x, ii)
}


.sub_rep_MultiFactor <- function(x, i, j, value) {
    missing_i <- rlang::is_missing(i)
    missing_j <- rlang::is_missing(j)
    missing_x <- rlang::is_missing(x)
    dot_len <- sum(!missing_i, !missing_j)

    stopifnot("Too many arguments provided" = dot_len %in% seq(0L, 2L, 1L))

    if (dot_len == 0L) {
        return(x)
    }

    d <- x@map

    if (!missing_i) ii <- rownames(d[i, , drop = FALSE])
    if (!missing_j) jj <- colnames(d[, j, drop = FALSE])

    if (missing_j) {
        x@index[ii] <- value
        return(x)
    }

    if (missing_i) {
        ii <- rowsWithCol(d, jj, names = TRUE)
    }

    for (i in ii) {
        for (j in jj) {
            x@index[[i]][, j] <- value[[i]][, j]
        }
    }
    return(x)
}

.sub_sub_rep_MultiFactor <- function(x, i, value) {
    d <- x@map
    # If i can't index d, stop. Appending not supported through `[[<-`.
    if (!all(i %in% colnames(d))) {
        if (anyNA(colnames(d)[i]) || length(colnames(d)[i]) != length(i)) {
            stop("No levels corresponding to `i` found in MultiFactor. ")
        }
    }
    ii <- rowsWithCol(d, i, FALSE)
    x@index[ii] <- value
    x
}


##############################################################################
##############################################################################
##############################################################################

#' For S4 compatibility
#' @noRd
unfactor <- function(x) S4Vectors::unfactor(x)

#' @noRd
#' @param `MultiFactor@index` from first `MultiFactor` in `c()` Method.
#' @param y a second `MultiFactor`
#' @returns Merged index.
mergeMultiFactorInds <- function(x, y) c(x, y@index)

#' @noRd
#' @param x `levels(MultiFactor)` from first `MultiFactor` in `c()` Method.
#' @param y a second `MultiFactor`.
#' @returns Merged levels
mergeMultiFactorLvs <- function(x, y) {
    y <- levels(y)
    i <- intersect(names(x), names(y))
    x[i] <- union(x[i], y[i])
    return(c(x, y[!names(y) %in% i]))
}


#' @param d `MultiFactor@map`
#' @param id `Character or Integer scalar`. Selects column(s) of `d`.
#' @param names Whether to return characters (Default) or integer indices.
#' @returns A vector indicating which elements of `MultiFactor` contain `id`.
#' @importFrom Matrix rowSums
#' @noRd
#' @description Helper function for `MultiFactor` to get names or indices of
#' data frames that contain an id column
#'
rowsWithCol <- function(d, id, names = TRUE) {
    rowInds <- which(Matrix::rowSums(d[, id, drop = FALSE] > 0L) == length(id))
    if (length(rowInds) == 0L) {
        return(NULL)
    }
    if (names) {
        rowInds <- rownames(d)[rowInds]
    }
    return(rowInds)
}

#' @noRd
#' @description `rowsWithCol` but returns union rather than intersect.
rowsInCol <- function(d, id, names = TRUE) {
    rowInds <- which(Matrix::rowSums(d[, id, drop = FALSE] > 0L) > 0L)
    if (length(rowInds) == 0L) {
        return(NULL)
    }
    if (names) {
        rowInds <- rownames(d)[rowInds]
    }
    return(rowInds)
}

#' @param d `MultiFactor@map`
#' @param id `Character or Integer vector`. Selects row(s) of `d`.
#' @param names Whether to return characters (Default) or integer indices.
#' @returns A vector indicating which feature types are in element `id`.
#' @importFrom Matrix colSums
#' @noRd
#' @description Helper function for `MultiFactor` to get names or indices of
#'     features contained in a given data frame element of `MultiFactor`.
#'
colsWithRow <- function(d, id, names = TRUE) {
    colInds <- which(Matrix::colSums(d[id, , drop = FALSE] > 0L) == length(id))
    if (length(colInds) == 0L) {
        return(NULL)
    }
    if (names) {
        colInds <- colnames(d)[colInds]
    }
    return(colInds)
}

#' @noRd
#' @description `colsWithRow` but returns union rather than intersect.
colsInRow <- function(d, id, names = TRUE) {
    colInds <- which(Matrix::colSums(d[id, , drop = FALSE] > 0L) > 0L)
    if (length(colInds) == 0L) {
        return(NULL)
    }
    if (names) {
        colInds <- colnames(d)[colInds]
    }
    return(colInds)
}

#' Methods for AnansiWeb S7 container class
#' @name AnansiWeb-methods
#' @param x input, AnansiWeb object
#' @aliases show.anansi::AnansiWeb names.anansi::AnansiWeb
#' @aliases dimnames.anansi::AnansiWeb dim.anansi::AnansiWeb
#' @seealso \itemize{
#' \item [weaveWeb()]: for general use.
#' \item [AnansiWeb-pairwise]: for methods for pairwise operations
#' }
#' @returns The desired information from an AnansiWeb object
#' @examples
#' # Setup
#' web <- randomWeb(n_samp = 36)
#'
#' # Accessors
#' dimnames(web)
#' dim(web)
#' names(web)
#'
#' # Getters and setters:
#'
#' tableX(web)[1:5, 1:5]
#' tableY(web)[1:5, 1:5]
#' dictionary(web)
#' head(metadata(web))
#'
#' # Assign some random metadata
#' metadata(web) <- data.frame(
#'     id = row.names(tableY(web)),
#'     a = rnorm(36),
#'     b = sample(c("a", "b"), 36, TRUE),
#'     row.names = "id"
#' )
#'
#' # Coerce to list
#' weblist <- as.list(web)
#'
#' # Coerce to Data.frame
#' webdf <- as.data.frame(web)
#'
#' # Coerce to MultiAssayExperiment
#' mae <- asMAE(web)
#'
#' # Coerce to TreeSummarizedExperiment
#' tse <- asTSE(web)
NULL

#' @importFrom methods show
#' @importMethodsFrom methods show
#' @export
#'
S7::method(show, AnansiWeb) <- function(object) {
    cat(
        paste(class(object), collapse = " "),
        " with ",
        NROW(object@tableX),
        " observations:\n    tableY: ",
        names(object)[1],
        " (",
        NROW(object),
        " features)\n    tableX: ",
        names(object)[2],
        " (",
        NCOL(object),
        " features)\n",
        sep = ""
    )
    cat("Use @ to access: tableX, tableY, dictionary, metadata.")
    invisible(NULL)
}

#' @export
#'
S7::method(print, AnansiWeb) <- function(x, ...) show(x)


#' @export
#'
S7::method(dimnames, AnansiWeb) <- function(x) dimnames(x@dictionary)

#' @export
#'
S7::method(dim, AnansiWeb) <- function(x) dim(x@dictionary)

S7::method(names, AnansiWeb) <- function(x) names(dimnames(x@dictionary))

#' @export
#'
S7::method(tableX, AnansiWeb) <- function(x) S7::prop(x, "tableX")

#' @export
#'
S7::method(tableY, AnansiWeb) <- function(x) S7::prop(x, "tableY")

#' @export
#'
S7::method(dictionary, AnansiWeb) <- function(x) S7::prop(x, "dictionary")

#' Get metadata.
#' @name metadata
#' @details
#' Compatible with S4Vectors generic.
#' @rdname metadata
#' @importMethodsFrom S4Vectors metadata
#' @aliases metadata,anansi::AnansiWeb-method
#' @param x input object
#' @param ... additional arguments
#' @returns `metadata` slot of x
#' @examples
#' x <- randomWeb(10)
#' metadata(x)
#' @export
#'
S7::method(metadata, AnansiWeb) <- function(x, ...) S7::prop(x, "metadata")

#' Set metadata.
#' @name metadata<-
#' @aliases metadata.set
#' @details
#' Compatible with S4Vectors generic.
#' @importMethodsFrom S4Vectors metadata<-
#' @aliases metadata<-,anansi::AnansiWeb-method
#' @param x input object
#' @param ... additional arguments
#' @param value replacement value, coerced to data.frame
#' @returns `x` with modified `metadata` slot.
#' @examples
#' x <- randomWeb(10)
#' metadata(x) <- cbind(
#'     metadata(x),
#'     new_groups = c("A", "B")
#' )
#' @export
#'
S7::method(`metadata<-`, AnansiWeb) <- function(x, ..., value) {
    x@metadata <- as.data.frame(value)
    x
}

################################################################################
################################################################################

#' For S4Vectors::metadata compatibility
#' @noRd
metadata <- function(x) S7::prop(x, "metadata")

#' For S4Vectors::metadata<- compatibility
#' @noRd
`metadata<-` <- function(x, value) {
    xx@metadata <- as.data.frame(value)
    x
}

#' Is this a data.frame with exactly two columns that are named?
#' @noRd
validWeb <- function(x) {
    y_names <- identical(rownames(x), colnames(x@tableY))
    x_names <- identical(colnames(x), colnames(x@tableX))
    s_names <- identical(rownames(x@tableY), rownames(x@tableX))
    meta_dim <- any(
        NROW(x@metadata) == NROW(x@tableY),
        prod(dim(x@metadata)) <= 1
    )
    if (!y_names) {
        message("colnames(tableY), rownames(dictionary) not identical.")
    }
    if (!x_names) {
        message("colnames(tableX), colnames(dictionary) not identical.")
    }
    if (!s_names) {
        message("rownames(tableX), rownames(tableX) not identical.")
    }
    if (!meta_dim) {
        message("NROW(metadata) does not equal rows of tableY, tableX.")
    }

    return(all(y_names, y_names, s_names, meta_dim))
}

#' Get a listof edges
#' @name getEdgeList
#' @rdname getEdgeList
#' @param x input object
#' @param ... additional arguments
#' @examples
#' x <- randomMultiFactor(n_features = 10)
#' getEdgeList(x)
#' @returns a two-column data.frame that lists the content of each entry in the
#'     input MultiFactor
#' @export
getEdgeList <- S7::new_generic("getEdgeList", "x")

#' Get a graph object.
#' @name getGraph
#' @seealso [`MultiFactor-methods`]
#' @param x input
#' @param ... additional arguments (currently not used).
#' @examples
#' # Show methods
#' getGraph
#' @returns a specified graph object.
#' @export
getGraph <- S7::new_generic("getGraph", "x")

#' Get a list of all pairs of features
#' @name getFeaturePairs
#' @rdname getFeaturePairs
#' @param x input object
#' @param ... additional arguments for specific methods
#' @returns an list of two-column data.frames that represent all feature pairs.
#' @examples
#' x <- randomWeb(10)
#' head(getFeaturePairs(x), 3)
#'
#' @export
getFeaturePairs <- S7::new_generic("getFeaturePairs", "x")

#' Weave an AnansiWeb object
#' @name weaveWeb
#' @rdname weaveWeb-generic
#' @param x input object
#' @param ... additional arguments
#' @seealso [weaveWeb-methods()]
#' @returns an `AnansiWeb` object, with sparse binary biadjacency matrix
#'     with features from `y` as rows and features from `x` as columns in
#'     `dictionary` slot.
#' @examples
#' # Setup demo tables
#' ec2ko <- kegg_link()[["ec2ko"]]
#' ec2cpd <- kegg_link()[["ec2cpd"]]
#'
#' # Basic usage
#' weaveWeb(cpd ~ ko, link = kegg_link())
#' weaveWeb(x = "ko", y = "ec", link = ec2ko)
#' weaveWeb(ec ~ cpd, link = ec2cpd)
#'
#' # A wrapper is available for kegg ko, ec and cpd data
#' generic <- weaveWeb(cpd ~ ko, link = kegg_link())
#' kegg_wrapper <- weaveKEGG(cpd ~ ko)
#'
#' identical(generic, kegg_wrapper)
#'
#' # The following are equivalent to transposition:
#' a <- weaveWeb(ko ~ cpd, link = kegg_link()) |> dictionary()
#' b <- weaveWeb(cpd ~ ko, link = kegg_link()) |> dictionary()
#'
#' identical(a, Matrix::t(b))
#'
#' @export
weaveWeb <- S7::new_generic("weaveWeb", "x")

#' @name plotAnansi
#' @rdname plotAnansi
#' @aliases plotAnansi-generic
#' @export
#' @returns a figure that can be further modified using the `ggplot2` suite
#' @usage NULL
plotAnansi <- S7::new_generic("plotAnansi", "x")

#' Apply a function on each pair of features
#' @name pairwiseApply
#' @rdname pairwiseApply
#' @aliases pairwiseApply-generic
#' @param X input object
#' @param ... additional arguments
#' @returns
#' a list containing the output of applying the function to each feature pair.
#'     See `?base::mapply()`
#' @examples
#' web <- randomWeb(10)
#'
#' # For each feature pair, was the value for x higher than the value for y?
#' pairwise_gt <- pairwiseApply(
#'     X = web,
#'     FUN = function(x, y) x > y,
#'     MoreArgs = NULL, SIMPLIFY = FALSE, USE.NAMES = TRUE
#' )
#' 
#' head(pairwise_gt)
#'
#' # Run cor.test() on each pair of features
#' pairwise_cor <- pairwiseApply(
#'     X = web,
#'     FUN = function(x, y) cor.test(x, y),
#'     MoreArgs = NULL, SIMPLIFY = FALSE, USE.NAMES = TRUE
#' )
#' 
#' pairwise_cor[1]
#'
#' @export
pairwiseApply <- S7::new_generic("pairwiseApply", "X")

#' @export
unfactor <- S7::new_external_generic(
    package = "S4Vectors", name = "unfactor", "x"
    )

#' @export
metadata <- S7::new_external_generic(
    package = "S4Vectors", name = "metadata", "x"
    )

#' @export
`metadata<-` <- S7::new_external_generic(
    package = "S4Vectors", name = "metadata<-", "x"
    )


#' Get tableX
#' @name tableX
#' @rdname tableX
#' @aliases tableX-generic
#' @param x input object
#' @param ... additional arguments
#' @returns `tableX` slot of x
#' @examples
#' x <- randomWeb(10)
#' tableX(x)
#' @export
#'
tableX <- S7::new_generic("tableX", "x")

#' Get tableY
#' @name tableY
#' @aliases tableY-generic
#' @rdname tableY
#' @param x input object
#' @param ... additional arguments
#' @returns `tableY` slot of x
#' @examples
#' x <- randomWeb(10)
#' tableY(x)
#' @export
#'
tableY <- S7::new_generic("tableY", "x")

#' Get dictionary
#' @name dictionary
#' @rdname dictionary
#' @aliases dictionary-generic
#' @param x input object
#' @param ... additional arguments
#' @returns `dictionary` slot of x
#' @examples
#' x <- randomWeb(10)
#' dictionary(x)
#'
#' @export
#'
dictionary <- S7::new_generic("dictionary", "x")

#' @noRd
#'
tell_F <- function(tale) {
    if (is(tale, "anansi::AnansiTale")) {
        return(tale@f.values)
    }
    if (is.list(tale)) {
        return(lapply(tale, tell_F))
    }
}

#' @noRd
#'
tell_T <- function(tale) {
    if (is(tale, "anansi::AnansiTale")) {
        return(tale@t.values)
    }
    if (is.list(tale)) {
        return(lapply(tale, tell_T))
    }
}

#' @noRd
#'
tell_P <- function(tale) {
    if (is(tale, "anansi::AnansiTale")) {
        return(tale@p.values)
    }
    if (is.list(tale)) {
        return(lapply(tale, tell_P))
    }
}


#' @noRd
#'
tell_e <- function(tale) {
    if (is(tale, "anansi::AnansiTale")) {
        return(tale@estimates)
    }
    if (is.list(tale)) {
        return(lapply(tale, tell_e))
    }
}


#' @noRd
#'
tell_df1 <- function(tale) {
    if (is(tale, "anansi::AnansiTale")) {
        return(tale@df[1])
    }
    if (is.list(tale)) {
        return(lapply(tale, tell_df1))
    }
}

#' @noRd
#'
tell_df2 <- function(tale) {
    if (is(tale, "anansi::AnansiTale")) {
        return(tale@df[2])
    }
    if (is.list(tale)) {
        return(lapply(tale, tell_df2))
    }
}

#' @noRd
#'
tell_dfr <- function(tale) {
    if (is(tale, "anansi::AnansiTale")) {
        return(tale@df[3])
    }
    if (is.list(tale)) {
        return(lapply(tale, tell_dfr))
    }
}

#' Extract information from an AnansiTale object and parse it into a neat table
#' @param tale An `AnansiTale` object
#' @param dic A dictionary.
#' @noRd
#' @return A wide format data.frame with summary statistics by feature pair.
#'
frame.tale <- function(tale, dic) {
    switch(tale@type,
        "r.values" = frame.tale.cor(tale, dic),
        "r.squared" = frame.tale.ols(tale, dic)
    )
}

#' @noRd
#'
frame.tale.cor <- function(tale, dic) {
    out.df <- data.frame(
        r.values = tale@estimates[dic],
        t.values = tale@t.values[dic],
        p.values = tale@p.values[dic]
        # q.values =  tale@q.values[dic]
    )
    colnames(out.df) <- paste(tale@subject, colnames(out.df), sep = "_")
    return(out.df)
}

#' @noRd
#'
frame.tale.ols <- function(tale, dic) {
    out.df <- data.frame(
        r.squared = tale@estimates[dic],
        f.values = tale@f.values[dic],
        p.values = tale@p.values[dic]
        # q.values =  tale@q.values[dic]
    )
    colnames(out.df) <- paste(tale@subject, colnames(out.df), sep = "_")
    return(out.df)
}

#' Shape list of `AnansiTale` results into a data.frame.
#' @noRd
#'
result.df <- function(out.list, dic) {
    feature_labs <- expand.grid(
        feature_Y = row.names(dic),
        feature_X = colnames(dic),
        stringsAsFactors = FALSE
    )[dic, ]

    df.list <- c(feature_labs, lapply(out.list, frame.tale, dic))
    do.call(what = "cbind.data.frame", args = df.list, quote = TRUE)
}

#' Handle FDR methods for anansi.
#' @param results The results table containing p-values to be adjusted.
#' @param method The p-value adjustment method. See ?p.adjust.
#' @return The `results` table, extended with adjusted p.values.
#' @importFrom stats p.adjust
#' @noRd
#'
anansi.p.adjust <- function(results, method) {
    p.cols <- grep("p.values", colnames(results))
    q.cols <- apply(results[, p.cols, drop = FALSE], 2, p.adjust, method)
    colnames(q.cols) <- gsub(
        "_p.values",
        "_q.values",
        x = colnames(q.cols),
        fixed = TRUE
    )
    cbind.data.frame(results, q.cols)
}

test_that("full and disjointed parameters correspond to stats::lm()", {
    tX <- `colnames<-`(replicate(5, c(scale(rnorm(36)))), letters[1:5])
    tY <- `colnames<-`(replicate(3, c(scale(rnorm(36)))), LETTERS[1:3])

    d <- matrix(
        TRUE,
        nrow = NCOL(tY),
        ncol = NCOL(tX),
        dimnames = list(y = colnames(tY), x = colnames(tX))
    )

    m <- data.frame(
        continuous = scale(rnorm(36)),
        categorical = sample(c("X", "Y", "Z"), size = 36, replace = TRUE)
    )

    web <- AnansiWeb(
        tableY = tY,
        tableX = tX,
        metadata = m,
        dictionary = d
    )

    anansi.res <- anansi(
        web = web,
        metadata = m,
        formula = ~categorical,
        verbose = FALSE,
        return.format = "raw"
    )
    a.full <- anansi.res[[5]]
    a.disj <- anansi.res[[6]]

    a.full.F <- tell_F(a.full)
    a.disj.F <- tell_F(a.disj)

    a.full.Rsq <- tell_e(a.full)
    a.disj.Rsq <- tell_e(a.disj)

    a.full.P <- tell_P(a.full)
    a.disj.P <- tell_P(a.disj)

    t.full <- matrix(
        nrow = NCOL(tY),
        ncol = NCOL(tX)
    )

    t.disj.F <- t.full.F <- t.full.P <- t.disj.P <- t.disj <- t.full

    for (y in seq_len(NCOL(tY))) {
        m$y.val <- y.val <- tY[, y]

        for (x in seq_len(NCOL(tX))) {
            m$x.val <- x.val <- tX[, x]

            f.1 <- lm(y.val ~ categorical, data = m)
            f.2 <- lm(y.val ~ x.val * categorical, data = m)
            f.x <- lm(y.val ~ x.val + categorical, data = m)

            df1.f <- f.2$rank - f.1$rank
            df1.x <- f.2$rank - f.x$rank
            df2 <- NROW(f.2$model) - f.2$rank

            ss.1 <- sum(residuals(f.1)^2)
            ss.2 <- sum(residuals(f.2)^2)
            ss.x <- sum(residuals(f.x)^2)

            t.full[y, x] <- 1 - (ss.2 / ss.1)
            t.full.F[y, x] <- oddify(t.full[y, x]) * (df2 / df1.f)
            t.full.P[y, x] <- pf(t.full.F[y, x], df1.f, df2, lower.tail = FALSE)

            t.disj[y, x] <- 1 - (ss.2 / ss.x)
            t.disj.F[y, x] <- oddify(t.disj[y, x]) * (df2 / df1.x)
            t.disj.P[y, x] <- pf(t.disj.F[y, x], df1.x, df2, lower.tail = FALSE)
        }
    }

    # following should be the same, by group
    expect_equal(t.full, a.full.Rsq)
    expect_equal(t.disj, a.disj.Rsq)

    expect_equal(t.full.P, a.full.P)
    expect_equal(t.disj.P, a.disj.P)

    expect_equal(t.full.F, a.full.F)
    expect_equal(t.disj.F, a.disj.F)

    # And test repeated measurements through random intercepts
    # with Error() notation
    anansi.res <- anansi(
        web = web,
        metadata = m,
        formula = ~ continuous + Error(categorical),
        verbose = FALSE,
        return.format = "raw"
    )
    a.full <- anansi.res[[2]]
    a.disj <- anansi.res[[3]]

    a.full.F <- tell_F(a.full)
    a.disj.F <- tell_F(a.disj)

    a.full.Rsq <- tell_e(a.full)
    a.disj.Rsq <- tell_e(a.disj)

    a.full.P <- tell_P(a.full)
    a.disj.P <- tell_P(a.disj)

    t.full <- matrix(
        nrow = NCOL(tY),
        ncol = NCOL(tX)
    )

    t.disj.F <- t.full.F <- t.full.P <- t.disj.P <- t.disj <- t.full

    for (y in 1:NCOL(tY)) {
        m$y.val <- y.val <- tY[, y]

        for (x in 1:NCOL(tX)) {
            m$x.val <- x.val <- tX[, x]

            f.1 <- lm(y.val ~ categorical + continuous, data = m)
            f.2 <- lm(y.val ~ categorical + x.val * continuous, data = m)
            f.x <- lm(y.val ~ categorical + x.val + continuous, data = m)

            df1.f <- f.2$rank - f.1$rank
            df1.x <- f.2$rank - f.x$rank
            df2 <- NROW(f.2$model) - f.2$rank

            ss.1 <- sum(residuals(f.1)^2)
            ss.2 <- sum(residuals(f.2)^2)
            ss.x <- sum(residuals(f.x)^2)

            t.full[y, x] <- 1 - (ss.2 / ss.1)
            t.full.F[y, x] <- oddify(t.full[y, x]) * (df2 / df1.f)
            t.full.P[y, x] <- pf(t.full.F[y, x], df1.f, df2, lower.tail = FALSE)

            t.disj[y, x] <- 1 - (ss.2 / ss.x)
            t.disj.F[y, x] <- oddify(t.disj[y, x]) * (df2 / df1.x)
            t.disj.P[y, x] <- pf(t.disj.F[y, x], df1.x, df2, lower.tail = FALSE)
        }
    }

    # following should be the same, by group
    expect_equal(t.full, a.full.Rsq)
    expect_equal(t.disj, a.disj.Rsq)

    expect_equal(t.full.P, a.full.P)
    expect_equal(t.disj.P, a.disj.P)

    expect_equal(t.full.F, a.full.F)
    expect_equal(t.disj.F, a.disj.F)
})

---
title: "Getting started with anansi"
output:
    BiocStyle::html_document:
        toc: true
        toc_float: true
        toc_depth: 2
        number_sections: true
vignette: >
  %\VignetteIndexEntry{1. Getting started with anansi}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style>
body {
text-align: justify}
</style>

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    echo = TRUE,
    tidy.opts = list(width.cutoff = 80),
    tidy = TRUE
)
# long links
adj_link <- paste0(
    "https://thomazbastiaanssen.github.io/anansi/",
    "articles/adjacency_matrices.html"
)
dif_link <- paste0(
    "https://thomazbastiaanssen.github.io/anansi/",
    "articles/differential_associations.html"
)

library(anansi)
```
# Overview {-}

This vignette can be divided into three sections:

* [Section 1](#sec-anansi-1) introduces the `anansi` framework.
* [Section 2](#sec-anansi-2) goes through a simple workflow and required input.
* [Section 3](#sec-anansi-3) discusses more advanced applications of anansi. 

# Introduction {#sec-anansi-1}
The `anansi` package computes and compares the association between the features 
of two 'omics datasets that are known to interact based on a database such as 
KEGG. Studies including both functional microbiome and metabolomics data are
becoming more common. Often, it would be helpful to integrate both datasets in
order to see if they corroborate each others patterns. However, all-vs-all
association analyses are imprecise and likely to yield spurious associations.
This package takes a knowledge-based approach to constrain association search
space, only considering  metabolite-function interactions that have been
recorded in a pathway database. In addition, it provides a framework to assess
differential associations. 

While `anansi` is geared towards metabolite-function interactions in the context
of host-microbe interactions, it is perfectly capable of handling any other pair
of data sets where some features interact canonically. 

## A note on functional microbiome data

Two common questions in the host-microbiome field are "Who's there?" and "What 
are they doing?". Techniques like 16S rRNA sequencing and shotgun metagenomics 
sequencing are most commonly used to answer the first question. The second
question can be a bit more tricky, as it often requires functional inference 
software to address them. For 16S sequencing, programs like 
[PICRUSt2](https://github.com/picrust/picrust2/) can be used to infer function. 
For shotgun metagenomics, this can be done with programs such as 
[HUMANn3 in the bioBakery suite](https://github.com/picrust/picrust2) and
[woltka](https://github.com/qiyunzhu/woltka). All of these algorithms can 
produce functional count data in terms of KEGG Orthologues (KOs) organised as 
tables, which can be directly passed to `anansi`. 

# Getting started with anansi {#sec-anansi-2}

## Installation instructions

Get the latest stable `R` release from [CRAN](http://cran.r-project.org/). Then
install `anansi` from [Bioconductor](http://bioconductor.org/) using the
following code:

```{r install, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("anansi")
```

And the development version from
[GitHub](https://github.com/thomazbastiaanssen/anansi) with `remotes`: 

```{r remotes-install, eval = FALSE}
install.packages("remotes")
remotes::install_github("thomazbastiaanssen/anansi")
```

## Setup 
```{r load-libraries, eval=TRUE}
# load anansi
library(anansi)

# load ggplot2 and ggforce to plot results
library(ggplot2)
library(ggforce)

# load example data + metadata from FMT Aging study
data(FMT_data)
```
    
## Data preparation

The anansi framework requires input data to be prepared in a specific format, 
namely the `AnansiWeb` S7 class. This is to ensure that features are correctly 
linked across input datasets. We will first discuss the required input data and 
then demonstrate the `weaveWeb` method, which is the recommended way to prepare 
input for the anansi framework. 

## Input formatting for anansi

The main `anansi` function expects data in the `AnansiWeb` class format, which 
contains four tables: 

### feature tables: `tableY` and `tableX`. 

The first two tables, `tableY` and `tableX` are the feature tables of your two
data modalities (e.g., metabolites, genes, microbes, ...). Both tables should 
have columns as features and rows as samples. Below is an example of an 
appropriately formatted feature table, we will use it in the sections below. 

```{r 'ko-table-format', echo = FALSE}
cat("anansi expects samples to be rows and features to be columns:\n")
t(FMT_KOs[seq(4L),seq(10L)])
```

```{r 't-tables',  eval=TRUE}
# Transpose demo tables to input format
tableY <- t(FMT_metab)
tableX <- t(FMT_KOs)
```

### linking information: `link`.

The third table, `link`, provides the information to link the features across 
the first two tables, `tableY` and `tableX`. `anansi` provides a pre-built map 
between ko, cpd and ec feature annotations from the KEGG database, but users can
also provide their custom mapping information. We will not expand on here, but
[see vignette on adjacency matrices.]`r sprintf("(%s)", adj_link)`
Also see `?anansi::MultiFactor()` 

### Additional sample data for analysis: Metadata

The final table is `metadata`: a `data.frame` with additional information about 
the data set. Its rows correspond to samples and its columns to variables that 
can be used as covariates in downstream `anansi()` analysis.

```{r metadata-format, echo = FALSE}
cat("anansi expects samples to be rows and features to be columns:\n")
`rownames<-`(FMT_metadata, NULL)[seq(10L),]
```

## Weave a web🕸️

The `weaveWeb()` function accepts the four input tables discussed above and 
collates them into an `AnansiWeb` object. The `AnansiWeb` format is a necessary 
input file for the main `anansi` workflow. It allows `anansi` to keep track of 
which features from the two input data sets should be considered as pairs. 

### Specify which features to `link`.

Additionally, `weaveWeb()` requires the user to specify the names of the feature
types that should be linked. These names need to be present as the columns of 
the `link` table. In this case, we'll link KEGG compound (cpd) to KEGG 
Orthologues (ko). 

```{r weaveweb, eval=TRUE}
# Create AnansiWeb object from tables
web <- weaveWeb(
    x = "ko", y = "cpd", 
    link = kegg_link(),
    tableY = tableY,
    tableX = tableX,
    metadata = FMT_metadata
)
```

### Specify `y`, and `x`

Though most of the examples use metabolites and functions, `anansi` is able to 
handle any type of big/'omics data, as long as there is a dictionary available. 
Because of this, anansi uses the type-naive nomenclature `tableY` and `tableX`. 
The Y and X refer to the position these measurements will have in the linear 
modeling framework: 

$$
Y \sim X \times{covariates}
$$

### `weaveWeb` with Bioconductor ecosystem

As an alternative to providing input as raw tables, `weaveWeb()` also accepts 
the Bioconductor classes `MultiAssayExperiment` (MAE) and 
`TreeSummarizedExperiment` (TSE) as input. Conversely, an `AnansiWeb` object can
be converted to a MAE or TSE with the functions `asMAE()` and `asTSE()`.

```{r bioc1}
#| eval=TRUE, message=FALSE
# Convert web to mae
mae <- asMAE(web)
```

The MAE object can be passed to the `weaveWeb()` function, which returns the 
same output, ready for the `anansi` function.

```{r bioc2}
#| eval=TRUE

web.mae <- weaveWeb(
    mae,
    tableY = "cpd", tableX = "ko",
    link = kegg_link()
)

out.mae <- anansi(
    web = web.mae,
    formula = ~Legend,
    adjust.method = "BH",
    verbose = TRUE
)

identical(web, web.mae)
```


## Run anansi🕷️

The main workspider is called `anansi` just like the package. Generally, two
main arguments should be specified:

1. `web`: an `AnansiWeb` object, such as the one we generated in the above step.
2. `formula`: a formula specifying which associations to test. For instance, to 
assess differential associations between treatments, we use the formula 
`~ Legend` (corresponding to the `Legend` column in the `metadata` table we 
provided to `weaveWeb()` above).

```{r run-anansi, eval=TRUE}
# Run anansi on AnansiWeb object
anansi_out <- anansi(
    web = web,
    formula = ~Legend,
    adjust.method = "BH",
    verbose = TRUE
)
```

## Reporting output📝

By default, `anansi` outputs a table in the `wide` format. For general 
reporting, we  recommend sticking to the more legible `table` format by 
specifying the `return.format` argument. Let's take a look at the structure of 
the output table:

```{r show-output, eval=TRUE}
# General structure of anansi() output
head(anansi_out, c(5, 5))

# Let's take a look at the column names
colnames(anansi_out)
```

### Output statistics

We can see that the output of `anansi()` is structured so that there is one
feature pair per row, marked by the first two columns. For each of those feature
pairs, the table contains outcome parameters from several models. In order, we
have pooled correlations, group-wise correlations, full model, disjointed and
emergent associations. For each of these models we report four parameters. 
For correlations we provide the Pearson's coefficient ($\rho$) and the
$t$-statistic, whereas we provide their squared counterparts for the linear
regression models: $R^2$ and the $F$ statistic. For all models, we additionally
provide p-values and adjusted p-values. For more on this, 
[see the vignette on differential associations]`r sprintf("(%s)", dif_link)`.

| Association | Method            | Statistics  | Significance  |
|-------------|-------------------|-------------|---------------|
| Pooled      | Correlation       | $\rho$, $t$ | p, adjusted p |
| Group-wise  | Correlation       | $\rho$, $t$ | p, adjusted p |
| Full        | Linear regression | $R^2$, $F$  | p, adjusted p |
| Disjointed  | Linear regression | $R^2$, $F$  | p, adjusted p |
| Emergent    | Linear regression | $R^2$, $F$  | p, adjusted p |

## Plot the results

Finally, the output of `anansi` can be visualized with `plotAnansi` by
specifying the type of association to use.

```{r bioc3}
#| eval=TRUE, fig.width = 10, fig.height = 8

# Filter associations by q value
anansi_out <- anansi_out[anansi_out$full_q.values < 0.2, ]

# Visualize disjointed associations
plotAnansi(
    anansi_out,
    association.type = "disjointed",
    model.var = "Legend",
    signif.threshold = 0.05,
    fill_by = "group"
)
```

# Advanced applications and customization {#sec-anansi-3}

## Generate `AnansiWeb` with random data

The `randomWeb()` function allows the user to generate random data, which can be
useful for statistical modeling or to generate placeholder data, for instance.

```{r r-web}
rweb <- randomWeb(
    n_samples = 10,
    n_reps = 1,
    n_features_x = 8,
    n_features_y = 12,
    sparseness = 0.5
)

# The random object comes with some dummy metadata:
metadata(rweb)

# also see `?randomAnansi`
```

## Apply user-defined function on each pair of features

A typical `anansi` workflow, as the one above, estimates associations between 
each pair of features. More generally, we can use `pairwiseApply()` on the same 
`AnansiWeb` input object to apply a user-defined function on each pair of 
features. The function passes as the `FUN` argument is applied on each pair of 
features, which are pre-mapped to `y` and `x`:  `function(x, y)`. 
[Also see vignette on adjacency matrices.]`r sprintf("(%s)", adj_link)`

```{r pairwiseApply1}
# For each feature pair, was the value for x higher than the value for y?
pairwise_gt <- pairwiseApply(
    X = web,
    FUN = function(x, y) x > y, 
    USE.NAMES = TRUE
)

pairwise_gt[1:5, 1:5]
```


```{r pairwiseApply2}
# Run cor() on each pair of features
pairwise_cor <- pairwiseApply(
    X = web,
    FUN = function(x, y) cor(x, y), 
    USE.NAMES = FALSE
)

pairwise_cor
```

## Plotting examples

Automatic plotting functions such as `plotAnansi()` provide a convenient 
and code-light solution to visualization. However, some may prefer interfacing 
directly with the `ggplot()` code. We provide code to recreate the above figure:

```{r plot-extra1}
#| plot_FMT, eval=TRUE, fig.width=10, fig.height=8, tidy=FALSE,
#| fig.alt="Differential association plot, with facets separating metabolites
#| and metabolites on the y-axis. x-axis depicts pearson's correlation
#| coefficient."
# Use tidyr to wrangle the correlation r-values to a single column
library(tidyr)

anansiLong <- anansi_out |>
    pivot_longer(starts_with("All") | contains("FMT")) |>
    separate_wider_delim(
        name,
        delim = "_", names = c("cor_group", "param")
    ) |>
    pivot_wider(names_from = param, values_from = value)

# Now it's ready to be plugged into ggplot2, though let's clean up a bit more.
# Only consider interactions where the entire model fits well enough.
anansiLong <- anansiLong[anansiLong$full_q.values < 0.2, ]

ggplot(anansiLong) +

    # Define aesthetics
    aes(
        x = r.values,
        y = feature_X,
        fill = cor_group,
        alpha = disjointed_Legend_p.values < 0.05
    ) +

    # Make a vertical dashed red line at x = 0
    geom_vline(xintercept = 0, linetype = "dashed", colour = "red") +

    # Points show  raw correlation coefficients
    geom_point(shape = 21, size = 3) +

    # facet per compound
    ggforce::facet_col(~feature_Y, space = "free", scales = "free_y") +

    # fix the scales, labels, theme and other layout
    scale_y_discrete(limits = rev, position = "right") +
    scale_alpha_manual(
        values = c("TRUE" = 1, "FALSE" = 1 / 3),
        "Disjointed association\np < 0.05"
    ) +
    scale_fill_manual(
        values = c(
            "Young yFMT" = "#2166ac",
            "Aged oFMT" = "#b2182b",
            "Aged yFMT" = "#ef8a62",
            "All" = "gray"
        ),
        breaks = c("Young yFMT", "Aged oFMT", "Aged yFMT", "All"),
        name = "Treatment"
    ) +
    theme_bw() +
    ylab("") +
    xlab("Pearson's \u03c1")
```

## Plotting multiple feature pairs at once with patchwork

We can also also use the `AnansiWeb` object to pull up specific pairs of
features. The [`patchwork`](https://patchwork.data-imaginist.com/) provides a
wonderful interface to combine plots. We can use `getFeaturePairs()` to
extract the relevant data from our `AnansiWeb` object in a convenient list
format:   
```{r plot-extra2}
#| multiplot, eval=TRUE, message=FALSE, tidy = FALSE
library(patchwork)

# Get a list containing data.frames for each feature pair.
feature_pairs <- getFeaturePairs(web)

# For each, prepare a ggplot object with those features on x & y axes.
plot_list <- lapply(
    feature_pairs,
    FUN = function(pair_i) {
        ggplot(pair_i) +
            aes(
                y = .data[[colnames(pair_i)[1L]]],
                x = .data[[colnames(pair_i)[2L]]]
            ) +
            # We'll use facet_grid for labels instead of labs().
            facet_grid(colnames(pair_i)[1L] ~ colnames(pair_i)[2L])
    }
)

# Feed the first six of our ggplots to patchwork, using wrap_plots()
wrap_plots(plot_list[seq_len(6L)], guides = "collect") &
    # Continue as with an ordinary ggplot, but use & instead of +
    geom_point(
        shape = 21, show.legend = FALSE,
        aes(fill = FMT_metadata$Legend)
    ) &
    geom_smooth(
        method = "lm", se = FALSE,
        aes(colour = FMT_metadata$Legend),
        show.legend = FALSE
    ) &

    # Appearance
    scale_fill_manual(
        aesthetics = c("colour", "fill"),
        values = c(
            "Young yFMT" = "#2166ac",
            "Aged oFMT" = "#b2182b",
            "Aged yFMT" = "#ef8a62",
            "All" = "gray"
        )
    ) &
    theme_bw() &
    labs(x = NULL, y = NULL)
```

These types of visualizations of feature pairs work best when the amount of 
feature pairs to be plotted is modest.

### Note on using `patchwork` with `anansi`: 

Especially for large numbers of feature pairs, it's much more efficient to add 
shared elements to the patchwork using the `&` operator, rather than adding them
to the initial `ggplot()` calls in the `lapply()` loop. 


# Session info

```{r session-info}
sessionInfo()
```

---
title: "Association testing"
output:
    BiocStyle::html_document:
        toc: true
        toc_float: true
        toc_depth: 2
        number_sections: true
vignette: >
  %\VignetteIndexEntry{3. Differential associations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style>
body {
text-align: justify}
</style>

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```
    
```{r setup, include = FALSE}
knitr::opts_chunk$set(
    echo = TRUE,
    tidy.opts = list(width.cutoff = 80),
    tidy = TRUE
)
# long links
adj_link <- paste0(
    "https://thomazbastiaanssen.github.io/anansi/",
    "articles/adjacency_matrices.html"
)
library(S7)
```

# Overview {-}
The goal of `anansi` is to provide a framework to interrogate interactions 
between different 'omics datasets, or more generally, between data modalities. 
More concretely, this involves estimating the association between pairs of 
features in several contexts. 
Anansi uses a (bi)adjacency matrix to determine which pars of features to 
investigate. For more on the 
[adjacency matrix, see the vignette]`r sprintf("(%s)", adj_link)`. Here, we will
discuss the different statistical models used to estimate the associations 
between feature-pairs. 

Anansi aims to follow the style of `stats::lm()` and this vignette provides 
equivalent models in `stats` as theoretical familiar ground. For these reasons, 
I warmly recommend the documentation of `stats` for further reading. 

This vignette is divided by three sections:

* [Section 1](#sec-da-1) introduces the anansi framework and covers the 
    *full model*.  
* [Section 2](#sec-da-2) Covers differential associations
* [Section 3](#sec-da-3) Discusses incorporating repeated measures through 
`Error()` notation. 

# Getting started: The full model {#sec-da-1}
After determining which pairs of features should be investigated, `anansi` fits
several linear models for which it returns summary statistics. The user has 
control over these models through the `formula` argument in `anansi(). `

## Setup 
We will use the same example as in 
[the vignette on adjacency matrices]`r sprintf("(%s)", adj_link)`. There, we 
also introduce the `AnansiWeb` class and demonstrate using `weaveWeb()`. Please 
note that the data used in this example have been generated with statistical 
associations manually spiked in for this vignette. 

```{r formula-web-full} 
# Setup AnansiWeb with some dummy data
library(anansi)
web <- krebsDemoWeb()
```

## Formula syntax
For each of the feature-pairs that should be assessed, anansi fits similarly 
structured linear models. Anansi supports complex linear models as well as 
longitudinal models using the R formula syntax. The most basic model in anansi 
is:

```{r y-x-formula,echo=FALSE}
y ~ x
```
where `y` refers to is the feature from `tableY` and `x` is the corresponding 
feature in `tableX`. 

Besides estimating the asociation between `y` and `x`, anansi also allows for 
the  inclusion of covariates that could influence this association. With 
**full model**, we mean the total influence of `x`, including all of its 
interaction terms, on `y`. For example, if our input formula was: 

```{r formulate, echo = FALSE}
f <- y ~ x * (group_ab + score_a)
f
```
R rewrites this as follows: 
```{r reformulate, echo=FALSE}
update.formula(f, . ~ .)
```
The variables that constitute the 'full' effect of x would be: 
`x + x:group_ab + x:score_a`. 

### Formula in anansi()
When providing a formula argument in the main `anansi()` function, the `y` and 
`x` variables should not be mentioned, they are already implied. Rather, only 
additional covariates that could interact with `x` should be provided in the 
`formula` argument. 

```{r formula-anansi-full} 
out <- anansi(web,
    formula = ~ group_ab + score_a,
    groups = "group_ab"
)
```
Note that `anansi()` will tell us how the rewritten model looks if we don't set
`verbose` to FALSE. 

The full model simply estimates the total association between `y` and `x`, 
making interpretation straightforward. We can imagine these associations to be
positive or negative. 

```{r plot_full}
#| echo=FALSE, warning=FALSE, message=FALSE, fig.dim=c(6,3),
#| out.width="80%",
#| fig.cap="Figure 1. Both citrate and cis-aconitate ~ aconitase dehydrogenase, sequentially."
#|
library(patchwork)
library(ggplot2)

full_df <- data.frame(
    citrate = tableX(web)[, "citrate"],
    `cis-aconitate` = tableX(web)[, "cis-aconitate"],
    aconitase = tableY(web)[, "aconitase"]
)

ggplot(full_df) +
    aes(x = aconitase, y = citrate) +
    facet_wrap(~"citrate") +
    ggplot(full_df) +
    aes(x = aconitase, y = cis.aconitate) +
    labs(y = "cis-aconitate") +
    facet_wrap(~"cis-aconitate") +
    plot_layout(axes = "collect", axis_titles = "collect") +
    plot_annotation(
        subtitle =
        ) &
    geom_point(shape = 21, fill = "#ffffbf") &

    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5)) &
    theme_bw()
```

# Differential association {#sec-da-2}

In order to assess differences in associations based on one or more variables 
(such as phenotype or treatment), we make use of the emergent and disjointed 
association paradigm introduced in the context of proportionality 
\citep{diff_prop,quinn2017propr,part_corr} and apply it outside of the simplex.

## Disjointed associations
Disjointed associations describe the situation where there is a detectable 
association in all cases, but the quality of that association, the **slope**, 
changes in a manner explained by a term. We estimate this as interaction terms 
with `x`. In our example we have two such terms, namely `x:group_ab` and 
`x:score_a`. 

### Disjointed associations: Categorical variables
The figure below shows how such a disjointed association might look. We can
imagine some sort of a pharmacological treatment that alters enzymatic 
conversion rates to the point where association between the enzyme and 
the metabolite *flips*. 

```{r disj_cat}
#| echo=FALSE, fig.dim=c(6,3), out.width="80%",
#| fig.cap="isocitrate ~ isocitrate dehydrogenase by category (treatment)."
#|
diff_df <- data.frame(
    group_ab = paste0("Treatment ", metadata(web)$group_ab),
    score_a = metadata(web)$score_a,
    score_a_cat = cut(
        metadata(web)$score_a,
        labels = c("0-20%", "20-40%", "40-60%", "60-80%", "80-100%"),
        quantile(metadata(web)$score_a, probs = seq(0, 1, 1 / 5)),
        include.lowest = TRUE
    ),
    repeated = metadata(web)$repeated,
    sample_id = metadata(web)$sample_id,
    isocitrate = tableX(web)[, "isocitrate"],
    ketoglutarate = tableX(web)[, "ketoglutarate"],
    isocitrate.dehydrogenase = tableY(web)[, "isocitrate dehydrogenase"],
    ketoglutarate.dehydrogenase = tableY(web)[, "ketoglutarate dehydrogenase"],
    succinyl.CoA = tableX(web)[, "succinyl-CoA"],
    succinate = tableX(web)[, "succinate"],
    succinyl.CoA.synthetase = tableY(web)[, "succinyl-CoA synthetase"],
    succinate.dehydrogenase = tableY(web)[, "succinate dehydrogenase"],
    fumarate = tableX(web)[, "fumarate"],
    fumarase = tableY(web)[, "fumarase"]
)

ggplot(diff_df) +
    aes(x = isocitrate.dehydrogenase, y = isocitrate, fill = group_ab) +
    facet_wrap(~group_ab, ) +
    labs(x = "isocitrate dehydrogenase") +
    geom_point(shape = 21, show.legend = FALSE) +
    scale_fill_manual(values = c(
        "Treatment a" = "#d73027",
        "Treatment b" = "#2166ac"
    )) +
    theme_bw() +
    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5))
```

### Disjointed associations: Continuous variables
Similarly, we could imagine that this flip is not binary or otherwise step-wise,
but rather the result of a continuous variable. 

```{r disj_cont}
#| echo=FALSE, fig.dim=c(10,3),
#| fig.cap="ketoglutarate ~ ketoglutarate dehydrogenase stratified by a third continuous variable."
#|
ggplot(diff_df) +
    aes(
        x = ketoglutarate.dehydrogenase,
        y = ketoglutarate,
        fill = pnorm(score_a)
    ) +
    facet_grid(~score_a_cat) +
    geom_point(shape = 21, show.legend = FALSE) +
    scale_fill_gradientn(
        name = "Example health index",
        colors = c("#a50026", "#f46d43", "#ffffbf", "#74add1", "#313695")
    ) +
    labs(x = "ketoglutatate dehydrogenase") +
    theme_bw() +
    guides(fill = guide_colourbar(
        position = "bottom",
        legend.direction = "horizontal"
    )) +
    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5))
```
### Equivalent `stats::lm()` model
We can use `pairwiseApply()` on an `AnansiWeb` object to run a function on the
observations for each pair of features. 
We can use this to confirm the statistical output of `anansi()` against the
`stats` package: 
```{r disj-lm}

score_lm <- pairwiseApply(web, FUN = function(x, y) {
    anova(
        lm(y ~ x * (group_ab + score_a), data = metadata(web))
    )$`F value`[5L]
})

group_lm <- pairwiseApply(web, FUN = function(x, y) {
    anova(
        lm(y ~ x * (score_a + group_ab), data = metadata(web))
    )$`F value`[5L]
})

all.equal(score_lm, out$disjointed_score_a_f.values)
all.equal(group_lm, out$disjointed_group_ab_f.values)
```
We run two separate `pairwiseApply()` calls for the two interaction terms. 
Because of the way `anova()` calculates sum-of-squares (type-I), the order of 
variables matters for `anova()`.

## Emergent association
Emergent associations describe the situation where the **strength** of an 
association is explained by a term interacting with x. We again estimate this as
interaction terms with `x`. In our example we have two such terms, namely 
`x:group_ab` and `x:score_a`. 

### Emergent associations: Categorical variables
The figure below shows how such a emergent association might look. We can
imagine some sort of a pharmacological treatment that can completely shut down
a biochemical pathway. As a result, the members of that pathway, which would 
show a consistently strong association under physiological conditions, could 
go *out of sync*. 

```{r emerg_cat}
#| echo=FALSE, fig.dim=c(6,3), out.width="80%",
#| fig.cap="succinyl-CoA ~ succinyl-CoA synthetase by categorical variable (Treatment)."
#|
ggplot(diff_df) +
    aes(x = succinyl.CoA.synthetase, y = succinyl.CoA, fill = group_ab) +
    facet_wrap(~group_ab) +
    geom_point(shape = 21, show.legend = FALSE) +
    scale_fill_manual(values = c(
        "Treatment a" = "#d73027",
        "Treatment b" = "#2166ac"
    )) +
    labs(x = "succinyl-CoA synthetase", y = "succinyl-CoA") +
    theme_bw() +
    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5))
```

### Emergent associations: Continuous variables
Similarly, we could imagine situations where this loss in association strength 
happens gradually, rather than in a binary manner. For example, perhaps there is
a strong association between a metabolite and an enzyme that metabolizes it 
under physiological conditions, but this association weakens along with some 
health index or environmental variable. 

```{r emerg_cont}
#| echo=FALSE, fig.dim=c(10,3),
#| fig.cap="succinate ~ succinate dehydrogenase stratified by a third continuous variable."
#|
ggplot(diff_df) +
    aes(x = succinate.dehydrogenase, y = succinate, fill = pnorm(score_a)) +
    facet_grid(~score_a_cat) +
    geom_point(shape = 21, show.legend = FALSE) +
    scale_fill_gradientn(
        name = "Example health index",
        colors = c("#a50026", "#f46d43", "#ffffbf", "#74add1", "#313695")
    ) +
    labs(x = "succinate dehydrogenase") +
    theme_bw() +
    coord_equal(ratio = 1, xlim = c(-2.5, 2.5), ylim = c(-2.5, 2.5))
```

# Repeated measures {#sec-da-3}

## Random slopes through `Error()`
Similar to `stats::aov()`, you can wrap a variable in `Error()` for `anansi()` 
to treat it as a stratum, allowing for random intercepts. This is useful when 
dealing with multiple measurements per subject, commonly due to a longitudinal 
design. 
```{r intro-error} 
outErr <- anansi(web,
    formula = ~ group_ab + score_a + Error(sample_id),
    groups = "group_ab"
)
```
Note that `anansi()` acknowledges the random intercept. 

### compare to `aov( Error() )`.
We can use `aov()` to see how `Error()` works. `R` does not use orthogonal 
contrasts and uses type-I sum-of-squares calculations by default. In other 
words, the order of covariates matters. A variable wrapped in `Error()` gets 
moved to the front of the line, so the rest of the model could be understood as 
conditioned on the `Error()`-wrapped term. 

Here is the model with `Error()`
```{r aov-error} 
summary(
    aov(fumarate ~ fumarase * repeated + Error(sample_id), data = diff_df)
)
```

And the equivalent model with `sample_id` moved to the front instead:.
```{r aov-regular}
summary(
    aov(fumarate ~ sample_id + fumarase * repeated, data = diff_df)
)

# See ?stats::aov for more.
```
For every term except for `sample_id`, the results are equivalent. 

# Session info

```{r session-info}
sessionInfo()
```

---
title: "Adjacency matrices"
output:
    BiocStyle::html_document:
        toc: true
        toc_float: true
        toc_depth: 2
        number_sections: true
vignette: >
  %\VignetteIndexEntry{2. Working with (bi)adjacency matrices}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style>
body {
text-align: justify}
</style>

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    echo = TRUE,
    tidy.opts = list(width.cutoff = 80),
    tidy = TRUE
)
library(anansi)
library(ggplot2)
```

# Overview {-}
A common challenge in integrative analysis of high-dimensional data sets is 
subsequent biological interpretation. Anansi addresses this challenge by only 
considering pairwise associations that are known to occur *a priori*. In order
to achieve this, we need to provide this relational information to anansi in the
form of a (bi)adjacency matrix. 

This vignette  can be divided into two sections:

* [Section 1](#sec-mat-1) introduces (bi)adjacency matrices in the context of 
    the anansi framework.  
* [Section 2](#sec-mat-2) introduces the `AnansiWeb` object and demonstrates how
    to work with it

# Understanding adjacency matrices {#sec-mat-1}

## Example from Biology: The Krebs cycle
Recall that the [citric acid cycle, or Krebs cycle][1], is a central piece of 
metabolic machinery involved in oxidative phosphorylation. See figure below for 
a simplified representation with 8 unique enzymatic reactions and 9 unique 
metabolites. 

[1]: https://en.wikipedia.org/wiki/Citric_acid_cycle
```{r plot-krebs}
#| echo=FALSE, out.width='100%', out.height='100%', warning=FALSE,
#| message=FALSE,
#| fig.cap="Simplified representation of the Krebs cycle. Orange boxes mark
#| metabolites and blue arrows represent enyzmes. "
library(patchwork)
library(ggraph)
library(igraph)
library(tidygraph)
library(dplyr)

metabs <- c(
    "isocitrate", "cis-aconitate", "citrate", "oxaloacetate",
    "L-malate", "fumarate", "succinate", "succinyl-CoA",
    "ketoglutarate"
)
d <- data.frame(
    from = c(
        "isocitrate", "ketoglutarate", "succinyl-CoA",
        "succinate", "fumarate", "L-malate", "oxaloacetate", "citrate",
        "cis-aconitate"
    ),
    to = c(
        "ketoglutarate", "succinyl-CoA", "succinate",
        "fumarate", "L-malate", "oxaloacetate", "citrate", "cis-aconitate",
        "isocitrate"
    ),
    vertices = c(
        "isocitrate\ndehydrogenase", "ketoglutarate\ndehydrogenase",
        "succinyl-CoA\nsynthetase", "succinate\ndehydrogenase",
        "\nfumarase", "malate\ndehydrogenase", "citrate\nsynthase",
        "aconitase\n", "aconitase\n"
    )
)

krebs_layout <- data.frame(
    x = (cos(seq(2 * pi, 0, length = 10) + pi / 2) * 1.25)[c(3:9, 1:2)],
    y = (sin(seq(2 * pi, 0, length = 10) + pi / 2) * 1.25)[c(3:9, 1:2)],
    circular = FALSE,
    name = c(
        "isocitrate", "ketoglutarate", "succinyl-CoA",
        "succinate", "fumarate", "L-malate",
        "oxaloacetate", "citrate", "cis-aconitate"
    ),
    .ggraph.orig_index = 1:9,
    .ggraph.index = 1:9
)

graph_from_data_frame(d, directed = TRUE) |>
    as_tbl_graph() |>
    mutate(Legend = if_else(name %in% metabs, "Metabolite", "Enzyme")) |>
    ggraph(layout = krebs_layout) +

    geom_edge_link(
        arrow = arrow(length = unit(4, "mm"), type = "closed"),
        edge_width = 2.5, color = "black",
        end_cap = circle(11.5, "mm"),
        start_cap = circle(11.25, "mm")
    ) +

    geom_edge_link(
        aes(label = vertices),
        arrow = arrow(length = unit(4, "mm"), type = "closed"),
        edge_width = 2, color = "dodgerblue",
        end_cap = circle(11.5, "mm"),
        start_cap = circle(11.5, "mm"),
        angle_calc = "along",
        label_dodge = unit(c(rep(13, 5), rep(-13, 4)), "mm"),
        label_size = 3.15
    ) +

    geom_node_label(aes(label = name),
        size = 3.5, fill = "orange",
        show.legend = FALSE, label.padding = unit(0.4, "lines"),
        nudge_x = c(0, 0.1, 0.05, -0.05, -0.1, 0, -0.1, 0, 0.1)
    ) +

    coord_equal(
        xlim = c(-1.25, 1.25), ylim = c(-1, 1),
        ratio = 1, clip = "off"
    ) +
    theme_graph()
```

## Adjacency matrices allow us to be specific in our questions

Let's imagine we had (high-dimensional proxy) measurements 
available for both types. For instance, we could have transcription data for the
enzymes and metabolomics data for the metabolites. We could represent our two 
data sets like two sets of features.

### All vs-all testing 
If we were to perform an all-vs-all integrative analysis between our two data 
sets, comprehensively testing every metabolite-enzyme pair, we'd test 
$8\times9=72$ hypotheses. 

```{r all-v-all}
#| echo=FALSE, fig.wide=TRUE, out.width='100%', out.height='60%',
#| fig.cap="Graph representation of possible association sctructures in the
#| Krebs cycle. **A)** All-vs-all, with a full graph. **B)** Constrained, only
#| linking metabolites an enzymes that are known to directly interact."

colmat <- data.frame(
    name = c(
        "aconitase", "isocitrate\ndehydrogenase",
        "ketoglutarate\ndehydrogenase", "succinyl-CoA\nsynthetase",
        "succinate\ndehydrogenase", "fumarase", "malate\ndehydrogenase",
        "citrate\nsynthase", "citrate", "cis-aconitate", "isocitrate",
        "ketoglutarate", "succinyl-CoA", "succinate", "fumarate", "L-malate",
        "oxaloacetate"
    ),
    y = c(seq(9, 1, length.out = 8), 9:1),
    x = rep(c(0.7, 0.3), times = c(8, 9)),
    fill = rep(c("Enzyme", "Metabolite"), times = c(8, 9))
)

p_0 <- ggplot() +
    annotate("segment",
        x = c(0.28, 0.72), y = 0.875,
        xend = c(0.28, 0.72), yend = 9.125, color = "gray"
    ) +
    geom_text(
        data = colmat, aes(y = y, x = x, label = name), colour = "black",
        size = 3.25, hjust = c(rep(0, 8), rep(1, 9)), lineheight = 7 / 10,
        nudge_x = c(rep(0.03, 8), rep(-0.03, 9))
    ) +
    scale_fill_manual(
        values = c(Metabolite = "orange", Enzyme = "dodgerblue")
    ) +
    coord_cartesian(xlim = c(1 / 20, 19 / 20), clip = "off") +
    annotate("text",
        label = c("Metabolites", "Enzymes"),
        x = c(2 / 8, 6 / 8), y = 9.6, size = 4
    ) +
    ylab(NULL) +
    xlab(NULL) +
    theme_void()

p_a <- p_0 +
    geom_point(
        data = colmat, aes(y = y, x = x, fill = fill),
        color = "black", shape = 21, size = 4, show.legend = FALSE
    )



p_b <- p_0 + geom_segment(
    data = expand.grid(seq(9, 1, length.out = 8), 9:1),
    aes(y = Var2, yend = Var1), x = 3 / 10, xend = 7 / 10,
    color = "#393D47", inherit.aes = FALSE
) +
    geom_point(
        data = colmat, aes(y = y, x = x, fill = fill),
        color = "black",
        shape = 21, size = 4, show.legend = FALSE
    )
p_c <- p_0 + geom_segment(
    data = data.frame(
        Var2 = seq(1, 9, length.out = 8)[
            c(8, 1, 8, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3, 3, 2, 2, 1)
        ],
        Var1 =
            c(9, 9, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3, 3, 2, 2, 1, 1)
    ),
    aes(y = Var1, yend = Var2), x = 3 / 10, xend = 7 / 10,
    color = "#393D47", inherit.aes = FALSE
) +
    geom_point(
        data = colmat, aes(y = y, x = x, fill = fill),
        color = "black", shape = 21, size = 4, show.legend = FALSE
    )

(p_b | p_c) + plot_annotation(tag_levels = "A")
```
### Knowledge-informed analyis of specific interactions
Often however, only a subset of these comparisons is scientifically relevant to
investigate. In this case for instance, only 17 associations between 
enzyme-metabolite pairs are likely to make sense. 
Non-selectively testing all 72 associations actively harms statistical power, 
as 55 of these tests likely cannot be interpreted, but their p-values will still
be tallied for the purpose of FDR and FWER. Moreover, they tend to obscure any 
biologically interpretable findings. 

In order to 'know' which features-pairs in your data set should be considered, 
anansi requires a *biadjacency matrix*. The biadjacency matrix corresponding to
to the sparser graph on the right can be seen in the figure below. 

```{r krebs-biadj}
#| echo=FALSE, fig.small=TRUE,
#| fig.cap="Matrix representation of the constrained analysis of the Krebs
#| cycle, with cells marked with 'X' signifying tested feature-pairs."
#|

krebs_edge_df <- local({
    data("krebs", package = "anansi")
    return(krebs)
})

krebs_edge_df |>
    mutate(Enzyme = `levels<-`(
        Enzyme,
        gsub(x = levels(Enzyme), " ", replacement = "\n")
    )) |>
    ggplot() +
    geom_text(label = "X", x = I(0.5), y = I(0.5)) +
    facet_grid(Enzyme ~ Metabolite, switch = "y", space = "free_x") +
    theme_bw() +
    theme(
        strip.text.y.left = element_text(
            angle = 0, size = 10, lineheight = 7 / 10, hjust = 1
        ),
        strip.text.x.top = element_text(angle = 90, size = 10, hjust = 0),
        strip.background.x = element_rect(fill = "orange", color = "black"),
        strip.background.y = element_rect(fill = "dodgerblue", color = "black"),
        panel.spacing = unit(0.1, "lines")
    ) +
    coord_equal(ratio = 1) +
    labs(x = NULL, y = NULL)
```

# General use in anansi {#sec-mat-2}
In anansi, we work with biadjacency matrices in the `AnansiWeb` S7 class, which 
consists of a biadjacency matrix, in this case we call it a `dictionary`, as 
well as two tables of observations that should be analysed jointly. 

The anansi package offers some functions that should be sufficient for basic 
analysis. To enable more advanced users to pursue non-standard applications, our
methodology is compatible with several popular interfaces. 

## Make a biadjacency matrix with `weaveWeb()`
The recommended default interface to generate an `AnansiWeb` object is 
`weaveWeb()`. Besides a traditional interface, it also accepts R `formula` 
syntax. Once we have an `AnansiWeb` object, we can use the $ operator to access 
the biadjacency matrix in the 'dictionary' slot.

```{r weaveWeb-intro}
# Two interfaces
form.web <- weaveWeb(ec ~ ko, link = kegg_link())

trad.web <- weaveWeb(y = "ec", x = "ko", link = kegg_link())

# Same output
identical(form.web, trad.web)

# Get the biadjacency matrix using the $ operator
head(dictionary(form.web), c(10, 10))
```
Note that the majority of this matrix consists of dots, it's a sparse matrix, 
which is a format that can considerably speed up calculations involving huge, 
mostly empty matrices. For further reading, see the 
[Matrix package website](https://matrix.r-forge.r-project.org/). 

### `weaveWeb()` input: `link`
`weaveWeb()` takes additional arguments, let's focus on `link`, which will take 
the information necessary to link features between input data sets. The anansi 
package includes such data for the [KEGG database](https://www.genome.jp/kegg/),
let's take a look at some of it:

```{r peakec2ko}
# Extract the data.frame from the list that kegg_link() returns
ec2ko <- kegg_link()[["ec2ko"]]

head(ec2ko)
```

### Structure of `link` input
The format of the `data.frame` is important: It should have two named columns. 
Each column contains feature IDs of one type (in this case, `ec` and `ko`, which
is how the KEGG database refers to 
[Enzyme Commission numbers](https://enzyme.expasy.org/) and 
[kegg orthologes](https://www.genome.jp/kegg/ko.html), respectively. If two 
feature  IDs are on the same row of the `data.frame`, it means those features 
are paired (hence adjacency). 

### linking between feature names
The column names correspond to the types of feature IDs that should be linked. 
These names are used throughout the `AnansiWeb` workflow. They can be directly 
called from an `AnansiWeb` object:

```{r names}
# Note that the formula controls the order
names(weaveWeb(ko ~ ec, link = kegg_link()))

# names() is short for the above
names(weaveWeb(ec ~ ko, link = kegg_link()))
```
## The pre-packaged kegg linking map
We need to provide `weaveWeb()` with a map of which features are linked. We can 
use the pre-packaged kegg link map, which consists of two similarly structured 
`data.frame`s in a list, one of which we just inspected. We can call it directly
using `kegg_link()`: 
```{r weaveKEGG}
# Only print the first few entries
lapply(kegg_link(), head)
```

### linking across two `data.frame`
Note the column names in the two `data.frames`: `ec`, `cpd` and `ko`. Two of 
these correspond to the formula we used just used: `weaveWeb(ec ~ ko)`. We 
didn't use the term `cpd`. Further, `ec` is present in both `data.frame`s. 

We can use `weaveWeb()` to make a biadjacency matrix between any combination of 
one or two similarly structured `data.frame`s, presented as a list. For the pre-
packaged data set, this means we can link between any two of `ec`, `cpd` and 
`ko`, in either order. 

```{r web_formula}
# Formula notation
head(
    dictionary( weaveWeb(cpd ~ ko, link = kegg_link()) ), 
    c(10, 10)
)

# Character notation
head(
    dictionary( weaveWeb(y = "ko", x = "cpd", link = kegg_link()) ), 
    c(10, 10)
)
```

## Use custom biadjacency matrices with AnansiWeb()
The most flexible way to make an `AnansiWeb` object is through `AnansiWeb()`: 
```{r AnansiWeb}
# Prep some dummy input tables
tX <- matrix(rnorm(30),
    nrow = 6,
    dimnames = list(NULL, x = paste("x", seq_len(5), sep = "_"))
)
tY <- matrix(rnorm(42),
    nrow = 6,
    dimnames = list(NULL, y = paste("y", seq_len(7), sep = "_"))
)

# Prep biadjacency matrix
# base::matrix is fine too, but will get coerced to Matrix::Matrix.
d <- matrix(
    data = sample(x = c(TRUE, FALSE), size = 35, replace = TRUE),
    nrow = NCOL(tY),
    ncol = NCOL(tX),
    dimnames = list(colnames(tY), colnames(tX))
)

# make the AnansiWeb
w <- AnansiWeb(
    tableY = tY,
    tableX = tX,
    dictionary = d
)

# Confirm the warning:
names(w)
```
## Additional approaches
There are numerous ways in which we can define an adjacency matrix. Here, we 
demonstrate a graph-based and matrix based approach.

### adjacency matrices with `igraph`
Importantly, (bi)adjacency matrices can be understood as graphs. Two common 
packages that deal with graphs are [`igraph`](https://r.igraph.org/) and
[`graph`](https://bioconductor.org/packages/graph/). To start off, we need to 
have a graph object, so let's make one using `igraph`. 
```{r igraph-1, message=FALSE}
library(igraph)
# Prepared data.frame for the occasion
head(krebs_edge_df)

# Convert data.frame to graph
g <- graph_from_data_frame(krebs_edge_df, directed = FALSE)
```

Now that we have constructed a graph, we still need to identify which features, 
vertices, belong to which data modality, in this case either enzymes and 
metabolites. Using the `igraph` package, this should be done through the `type` 
slot. `type` can be assigned a boolean vector, where `TRUE` values 
become columns, corresponding to features in `tableX`, whereas `FALSE` values 
become rows, which correspond to those in `tableY`.

```{r igraph-2, message=FALSE}
V(g)$type <- V(g)$name %in% krebs_edge_df$Metabolite

# Now that we've defined type, we can convert to biadjacency matrix:
bi_mat1 <- as_biadjacency_matrix(g, sparse = TRUE)


head(bi_mat1, n = c(4, 5))
```
Though biadjacency support in `graph` is currently limited, We note that igraph 
and graph objects can be converted using the `graph_from_graphnel()` and 
`as_graphnel()` functions in `igraph`.

### adjacency matrices with `Matrix`
We can also define a matrix directly. Conveniently, sparse matrices can be 
defined easily from our starting data. The `Matrix` library provides fantastic 
support for this.
```{r Matrix, message=FALSE}
library(Matrix)

# For this approach, it's useful to prepare the input as two factors:
Enzyme <- factor(krebs_edge_df$Enzyme)
Metabolite <- factor(krebs_edge_df$Metabolite)

# We can get integers out of factors, corresponding to their level indices
bi_mat2 <- sparseMatrix(
    i = as.integer(Enzyme),
    j = as.integer(Metabolite),
    dimnames = list(
        Enzyme = levels(Enzyme),
        Metabolite = levels(Metabolite)
    )
)

head(bi_mat2, n = c(4, 5))
```

# Session info

```{r session-info}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{FMT_metab}
\alias{FMT_metab}
\title{Snippet of the CLR-transformed hippocampal metabolomics data from the FMT
Aging study.}
\format{
A matrix object with three rows, compounds, and 36 columns, samples.
}
\source{
\doi{10.1038/s43587-021-00093-9}
}
\usage{
data("FMT_data", package = "anansi")
}
\description{
Snippet of the CLR-transformed hippocampal metabolomics data from the FMT
Aging study.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnansiWeb-pairwise-methods.R
\name{AnansiWeb-pairwise}
\alias{AnansiWeb-pairwise}
\alias{pairs.AnansiWeb}
\alias{pairs.anansi::AnansiWeb}
\alias{pairwiseApply.AnansiWeb}
\alias{pairwiseApply.anansi::AnansiWeb}
\title{Methods for pairwise operations on AnansiWeb objects}
\arguments{
\item{X, x}{input, AnansiWeb object}

\item{...}{additional arguments}

\item{FUN}{a function with at least two arguments. The variables \code{x} and \code{y},
in order, refer to the corresponding values of feature pairs in \code{tableX}
and \code{tableY}.}

\item{MoreArgs, SIMPLIFY, USE.NAMES}{see ?base::mapply}

\item{which}{\verb{integer matrix}, indicating pair positions in \code{x@tableY} and
\code{x@tableX}, respectively. If \code{NULL} (default):
\code{Matrix::which(x@dictionary, TRUE)}.}

\item{with.metadata}{\verb{Logical scalar} whether to append metadata to output}
}
\value{
pairs: A two-column array index, corresponding to i,j coordinates in matrix
notation. \cr
getFeaturePairs: A list of data.frames with the paired data \cr
}
\description{
Methods to run pairwise analysis on multi-modal data.
}
\examples{
web <- randomWeb()
# Extract data.frames in pairs (only show first)
getFeaturePairs(web)[1L]

pairs(x = web)

getFeaturePairs(x = web, which = NULL, with.metadata = FALSE)

pairwiseApply(
    X = web,
    FUN = function(x, y) cor(x, y),
    MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{AnansiWeb}
\alias{AnansiWeb}
\title{AnansiWeb S7 container class}
\usage{
## Constructor for `AnansiWeb` objects
AnansiWeb(tableX, tableY, dictionary, metadata = data.frame())
}
\arguments{
\item{tableY, tableX}{A table containing features of interest. Rows should be
samples and columns should be features. Y and X refer to the position of
the features in a formula: Y ~ X.}

\item{dictionary}{A binary adjacency matrix of class \code{Matrix}, or coercible
to \code{Matrix}}

\item{metadata}{\code{list} of metadata. Optional.}
}
\value{
an \code{AnansiWeb} object, with sparse binary biadjacency matrix
with features from \code{y} as rows and features from \code{x} as columns in
\code{dictionary} slot.
}
\description{
\code{AnansiWeb} is an S7 class containing two feature tables as well as a
dictionary to link them. \code{AnansiWeb} is the main container that will
hold your input data throughout the \code{anansi} pipeline.

Typical use of the \code{anansi} package will involve generating an \code{AnansiWeb}
object using the \code{weaveWeb()} function.

The function \code{AnansiWeb()} constructs an \code{AnansiWeb} object from two
feature tables and an adjacency matrix.
}
\section{Slots}{

\describe{
\item{\code{tableY,tableX}}{Two \code{matrix} objects of measurements, data. Rows are
samples and columns are features. Access with \verb{@tableY} and \verb{@tableX}.}

\item{\code{dictionary}}{\code{Matrix}, binary adjacency matrix. Optionally sparse.
Typically generated using the \code{weaveWeb()} function. Access with
\verb{@dictionary}.}

\item{\code{metadata}}{Optional \code{data.frame} of sample metadata. Access with
\verb{@metadata}.}
}}

\examples{

# Use AnansiWeb() to construct an AnansiWeb object from components:
tX <- `dimnames<-`(replicate(5, (rnorm(36))),
    value = list(
        as.character(seq_len(36)),
        letters[1:5]
    )
)
tY <- `dimnames<-`(replicate(3, (rnorm(36))),
    value = list(
        as.character(seq_len(36)),
        LETTERS[1:3]
    )
)

d <- matrix(TRUE,
    nrow = NCOL(tY), ncol = NCOL(tX),

    # Note: Dictionary should have named dimensions
    dimnames = list(
        y_names = colnames(tY),
        x_names = colnames(tX)
    )
)
web <- AnansiWeb(tableX = tX, tableY = tY, dictionary = d)

}
\seealso{
\itemize{
\item \link{AnansiWeb-methods}
\item \link{randomAnansi}: For generation of random AnansiWeb objects.
\item \code{\link[=kegg_link]{kegg_link()}}: For examples of input for link argument.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/plotAnansi-methods.R
\name{plotAnansi}
\alias{plotAnansi}
\alias{plotAnansi-generic}
\alias{plotAnansi-methods}
\title{Dissociation plot}
\arguments{
\item{x}{a \code{data.frame} object output of \code{\link[=anansi]{anansi()}} in
the table format.}

\item{...}{additional parameters}

\item{layout}{\verb{Character scalar}. Specifies the plot layout to generate. It
must be one of \verb{c("dotplot, graph)}. (Default: \code{dotplot})}

\item{association.type}{\verb{Character scalar}. Specifies the type of
association to show in the plot. One of \code{"disjointed"},
\code{"emergent"} and \code{"full"}. (Default: \code{NULL})}

\item{model.var}{\verb{Character scalar}. Specifies the name of a variable
in the anansi model. It is relevant only when \code{association.type} is
\code{"disjointed"} or \code{"emergent"}. (Default: \code{NULL})}

\item{group}{\verb{Character scalar}. Selects one of the groups included in the
anansi model. It is relevant only when \code{layout} is \code{graph}.
(Default: \code{All})}

\item{signif.threshold}{\verb{Numeric scalar}. Specifies the threshold to
mark the significance of \code{association.type}. (Default: \code{NULL})}

\item{colour_by}{\verb{Character scalar}. Specifies one of the \code{groups}
terms used in the original \code{anansi} call, \code{x} by which points
should be coloured. (Default: \code{NULL})}

\item{color_by}{\verb{Character scalar}. Alias to \code{colour_by}.}

\item{fill_by}{\verb{Character scalar}. Specifies one of the \code{groups}
terms used in the original \code{anansi} call, \code{x} by which points
should be filled (Default: \code{"group"})}

\item{size_by}{\verb{Character scalar}. Specifies one of the \code{groups}
terms used in the original \code{anansi} call, \code{x} by which points
should be sized. (Default: \code{NULL})}

\item{shape_by}{\verb{Character scalar}. Specifies one of the \code{groups}
terms used in the original \code{anansi} call, \code{x} by which points
should be shaped. (Default: \code{NULL})}

\item{x_lab}{\verb{Character scalar}. Specifies the label of the x axis.
(Default: \code{"cor"})}

\item{y_lab}{\verb{Character scalar}. Specifies the label of the y axis.
(Default: \code{""})}

\item{y_position}{\verb{Character scalar}. Specifies the position of the y
labels. It should be either \code{"left"} or \code{"right"}.
(Default: \code{"right"})}

\item{show.cor}{\verb{Logical scalar}. Whether correlation edges should be
labelled with correlation coefficients when \code{layout} is \code{graph}.
(Default: \code{FALSE})}
}
\value{
a figure that can be further modified using the \code{ggplot2} suite

A ggplot2 object.
}
\description{
\code{plotAnansi} generates an association plot from the output of
\code{\link[=anansi]{anansi()}} in the table format. It provides a convenient way to
visually assess relevant results from the anansi analysis, either in the
form of a dotplot or a graph.
}
\details{
\code{plotAnansi} provides a standardised method to visualise the results
of anansi by means of a differential association plot. The input for this
function should be generated from \code{\link[=anansi]{anansi()}} or
\code{\link[=anansi]{anansi()}}, with \code{return.format = "table"}
}
\examples{
# Import libraries
library(mia)
library(TreeSummarizedExperiment)
library(MultiAssayExperiment)
library(ggraph)

web <- randomWeb(n_samples = 100)
mae <- asMAE(web)

# Perform anansi analysis
out <- weaveWeb(mae,
    tableY = "y", tableX = "x"
) |> anansi(formula = ~group_ab)

# Select significant interactions
out <- out[out$full_p.values < 0.05, ]

# Visualise disjointed associations filled by group
plotAnansi(out,
    association.type = "disjointed",
    model.var = "group_ab",
    signif.threshold = 0.05,
    fill_by = "group"
)

# Visualise full associations filled by group
plotAnansi(out,
    association.type = "full",
    signif.threshold = 0.05,
    fill_by = "group"
)

# Visualise full associations as graph
plotAnansi(out,
    layout = "graph",
    association.type = "full",
    signif.threshold = 0.05,
    show.cor = TRUE
)

# Visualise disjointed associations as graph
plotAnansi(out,
    layout = "graph",
    association.type = "disjointed",
    model.var = "group_ab",
    signif.threshold = 0.05
)

}
\seealso{
\code{\link[=anansi]{anansi()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{tableY}
\alias{tableY}
\alias{tableY-generic}
\title{Get tableY}
\usage{
tableY(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
\code{tableY} slot of x
}
\description{
Get tableY
}
\examples{
x <- randomWeb(10)
tableY(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnansiWeb-methods.R
\name{metadata<-}
\alias{metadata<-}
\alias{metadata.set}
\alias{metadata<-,anansi::AnansiWeb-method}
\title{Set metadata.}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}

\item{value}{replacement value, coerced to data.frame}
}
\value{
\code{x} with modified \code{metadata} slot.
}
\description{
Set metadata.
}
\details{
Compatible with S4Vectors generic.
}
\examples{
x <- randomWeb(10)
metadata(x) <- cbind(
    metadata(x),
    new_groups = c("A", "B")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/randomAnansi.R
\name{randomAnansi}
\alias{randomAnansi}
\alias{randomWeb}
\alias{randomMultiFactor}
\alias{krebsDemoWeb}
\title{Generate a random AnansiWeb or MultiFactor}
\usage{
randomWeb(
  n_samples = 10,
  n_reps = 1L,
  n_features_x = 8,
  n_features_y = 12,
  sparseness = 0.5,
  tableY = NULL,
  tableX = NULL,
  dictionary = NULL
)

randomMultiFactor(n_types = 6, n_features = 100, sparseness = 0.5)

krebsDemoWeb(n_samples = 100, n_reps = 4L)
}
\arguments{
\item{n_samples, n_reps}{\verb{Numeric scalar} Number of samples and repeated
measures of those samples to be generated. Ignored if \code{tableY} and
\code{tableX} are provided. (defaults: 10 samples without repeats)}

\item{n_features_y, n_features_x}{\verb{Numeric scalar} Number of features to be
generated. Ignored if \code{tableY} and \code{tableX} are provided.}

\item{sparseness}{\verb{Numeric scalar}, proportion: How rare are connections}

\item{tableY, tableX}{A table containing features of interest. Rows should be
samples and columns should be features. Y and X refer to the position of
the features in a formula: Y ~ X.}

\item{dictionary}{A binary adjacency matrix of class \code{Matrix}, or
coercible to \code{Matrix}.}

\item{n_types}{\verb{Numeric scalar}, number of types of features to generate}

\item{n_features}{\verb{Numeric scalar}, number of features per type}
}
\value{
a randomly generated object of the specified class.
}
\description{
Randomly generate a valid \code{AnansiWeb} or \code{MultiFactor} object.
}
\examples{
# Make a random AnansiWeb object
randomWeb()
krebsDemoWeb()
randomMultiFactor()
}
\seealso{
\code{\link[=AnansiWeb]{AnansiWeb()}}, \code{\link[=MultiFactor]{MultiFactor()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/weaveWeb-methods.R
\name{weaveWeb}
\alias{weaveWeb}
\alias{weaveKEGG}
\title{Weave an AnansiWeb object}
\usage{
weaveWeb(x, ...)

weaveKEGG(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
an \code{AnansiWeb} object, with sparse binary biadjacency matrix
with features from \code{y} as rows and features from \code{x} as columns in
\code{dictionary} slot.
}
\description{
Weave an AnansiWeb object
}
\examples{
# Setup demo tables
ec2ko <- kegg_link()[["ec2ko"]]
ec2cpd <- kegg_link()[["ec2cpd"]]

# Basic usage
weaveWeb(cpd ~ ko, link = kegg_link())
weaveWeb(x = "ko", y = "ec", link = ec2ko)
weaveWeb(ec ~ cpd, link = ec2cpd)

# A wrapper is available for kegg ko, ec and cpd data
generic <- weaveWeb(cpd ~ ko, link = kegg_link())
kegg_wrapper <- weaveKEGG(cpd ~ ko)

identical(generic, kegg_wrapper)

# The following are equivalent to transposition:
a <- weaveWeb(ko ~ cpd, link = kegg_link()) |> dictionary()
b <- weaveWeb(cpd ~ ko, link = kegg_link()) |> dictionary()

identical(a, Matrix::t(b))

}
\seealso{
\code{\link[=weaveWeb-methods]{weaveWeb-methods()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{krebs}
\alias{krebs}
\title{Simplified snippet of the Krebs cycle}
\format{
A \code{data.frame} of two factors, named \code{"Enzyme"} and \code{"Metabolite"},
which contain these respective components of the krebs cycle.
}
\source{
Generated by hand based on the krebs cycle.
}
\usage{
data("krebs", package = "anansi")
}
\description{
Enzymes and metabolites that make up the Krebs cycle. Features share a row
when they play a role in the same reaction.
Used in vignettes, exported to facilitate user exploration.
}
\seealso{
\code{\link[=krebsDemoWeb]{krebsDemoWeb()}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{MultiFactor}
\alias{MultiFactor}
\alias{asMultiFactor}
\title{MultiFactor S7 container class}
\usage{
MultiFactor(x, levels = NULL, drop.unmatched = FALSE)

## Constructor for `MultiFactor` objects
MultiFactor(x, levels = NULL, drop.unmatched = FALSE)
}
\arguments{
\item{x}{\code{MultiFactor}}

\item{levels}{an optional named list of vectors of the unique values (as
character strings) that x might have taken. The default is the unique set
of values taken by lapply(x, as.character), sorted into increasing order
of x.}

\item{drop.unmatched}{\verb{Logical scalar} If \code{TRUE} (Default), for feature types
that are seen at least twice, exclude features that only present in one
of their respective link data frames.}
}
\value{
a MultiFactor object.
}
\description{
\code{MultiFactor} is an S7 class to organize and manage multiple sets of factors,
for instance when tracing or converting feature IDs across databases. Methods
for \code{MultiFactor} aim to follow \code{factor} behaviour.
}
\details{
The most straightforward way to construct a \code{MultiFactor} object is as a
named list of named data.frames. The columns of the data.frames indicate the
category of factor in that column.

A \code{MultiFactor} object presents itself similar to a \code{data.frame}, in the
sense that level types can be called as columns and individual data.frame
components can be called as rows.
}
\section{Slots}{

\describe{
\item{\code{index}}{Named \code{list} of named integer data frames of at least two columns
each. The column names correspond to names in the \code{levels} slot. Similar
to \code{factor}s, the integers in those columns correspond to the characters
in that level. Accessed through regular list methods (e.g., \code{[}, \code{[[}).}

\item{\code{levels}}{\verb{Named list of character vectors}. Accessed through \code{levels(x)}}

\item{\code{map}}{\verb{(sparse) Matrix} specifying which elements contain which levels.
Accesses through \code{x@dictionary}.}
}}

\examples{
# Generate some random linkage input
x <- data.frame(
    a = sample(letters[seq(3)], 10, replace = TRUE),
    A = sample(LETTERS[seq(3)], 10, replace = TRUE)
)
MultiFactor(x)

}
\seealso{
\code{\link[=MultiFactor-methods]{MultiFactor-methods()}}

\itemize{
\item \code{\link[=kegg_link]{kegg_link()}}: for an example of valid input.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnansiWeb-methods.R
\name{metadata}
\alias{metadata}
\alias{metadata,anansi::AnansiWeb-method}
\title{Get metadata.}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
\code{metadata} slot of x
}
\description{
Get metadata.
}
\details{
Compatible with S4Vectors generic.
}
\examples{
x <- randomWeb(10)
metadata(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnansiWeb-methods.R
\name{AnansiWeb-methods}
\alias{AnansiWeb-methods}
\alias{show.anansi::AnansiWeb}
\alias{names.anansi::AnansiWeb}
\alias{dimnames.anansi::AnansiWeb}
\alias{dim.anansi::AnansiWeb}
\title{Methods for AnansiWeb S7 container class}
\arguments{
\item{x}{input, AnansiWeb object}
}
\value{
The desired information from an AnansiWeb object
}
\description{
Methods for AnansiWeb S7 container class
}
\examples{
# Setup
web <- randomWeb(n_samp = 36)

# Accessors
dimnames(web)
dim(web)
names(web)

# Getters and setters:

tableX(web)[1:5, 1:5]
tableY(web)[1:5, 1:5]
dictionary(web)
head(metadata(web))

# Assign some random metadata
metadata(web) <- data.frame(
    id = row.names(tableY(web)),
    a = rnorm(36),
    b = sample(c("a", "b"), 36, TRUE),
    row.names = "id"
)

# Coerce to list
weblist <- as.list(web)

# Coerce to Data.frame
webdf <- as.data.frame(web)

# Coerce to MultiAssayExperiment
mae <- asMAE(web)

# Coerce to TreeSummarizedExperiment
tse <- asTSE(web)
}
\seealso{
\itemize{
\item \code{\link[=weaveWeb]{weaveWeb()}}: for general use.
\item \link{AnansiWeb-pairwise}: for methods for pairwise operations
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anansi_diff_cor_qr.R
\name{anansiDiffCor}
\alias{anansiDiffCor}
\title{Run differential correlation analysis for all interacting metabolites and
functions.}
\usage{
anansiDiffCor(web, sat_model, errorterm, int.terms, metadata, verbose)
}
\arguments{
\item{web}{An \code{AnansiWeb} object, containing two tables with omics data and a
dictionary that links them. See \code{weaveWebFromTables()} for how to weave
a web.}

\item{sat_model}{A \code{formula} object, containing the full model}

\item{errorterm}{A \verb{character vector}, containing the metadata col.name denoting repeated
measures.}

\item{int.terms}{A \verb{character vector}, containing the metadata col.names denoting
covariates interacting with X to be tested for differential associations.}

\item{metadata}{A vector or data.frame of categorical or continuous value necessary for
differential correlations. Often a state or treatment score. If no argument
provided, anansi will let you know and still to regular correlations
according to your dictionary.}

\item{verbose}{A boolean. Toggles whether to print diagnostic information
while running. Useful for debugging errors on large datasets.}
}
\value{
a list of \code{AnansiTale} result objects, one for the total model,
one for emergent correlations and one for disjointed correlations.
}
\description{
Can either take continuous or categorical data. Typically, the main
\code{anansi()} function will run this for you.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weaveWeb-methods.R
\name{weaveWeb-methods}
\alias{weaveWeb-methods}
\alias{weaveWeb.character}
\alias{weaveWeb.formula}
\alias{weaveWeb.MultiAssayExperiment}
\alias{weaveWeb.TreeSumarizedExperiment}
\alias{weaveWeb.SingleCellExperiment}
\title{Weave an AnansiWeb}
\arguments{
\item{x, y}{\verb{Character scalar}, names of feature types that should be
linked. Should be found in the column names of \code{link}.}

\item{link}{One of the following:
\itemize{
\item \verb{Character scalar} with value \code{"none"}.
\item \code{data.frame} with two columns
\item \code{list} with two such \code{data.frame}s.
}}

\item{tableY, tableX}{A table containing features of interest. Rows should be
samples and columns should be features. Y and X refer to the position of
the features in a formula: Y ~ X. \cr
< For Bioconductor S4 objects > \cr
\verb{Character scalar} or \verb{numeric scalar}. Selects experiment
corresponding to \code{tableY} and \code{tableX} from \code{experiments(x)} of
\code{MultiAssayExperiment} object by name or index, name is recommended.
(Default slots: \code{Y = 1L}, \code{X = 2L}).}

\item{metadata}{Optional \code{data.frame} of sample metadata, to be included with
output. Can be accessed from \code{AnansiWeb} generated by \code{weaveWeb()} with
\code{output@metadata}.}

\item{verbose}{\verb{Logical scalar}. Whether to print diagnostic information
(Default: \code{TRUE}).#' @param force_new \code{boolean} If x already has a
dictionary \code{Matrix} in metadata, ignore it and generate a new object
anyway? (Default: FALSE).}

\item{typeY, typeX}{\verb{Character scalar} or \verb{numeric scalar}. Selects assay from experiments to
\code{tableY} and \code{tableX} from \code{experiments(x)}. (Default: \code{1L} - the first assay
in that experiment).}

\item{experiment1, experiment2}{synonymous args to \verb{tableY,tableX} for
compatibility with \code{mia} argument style.}

\item{assay.type1, assay.type2}{synonymous args to \verb{typeY,typeX} for
compatibility with \code{mia} argument style.}
}
\value{
an \code{AnansiWeb} object, with sparse binary biadjacency matrix
with features from \code{y} as rows and features from \code{x} as columns in
\code{dictionary} slot.
}
\description{
Generate a biadjacency matrix, linking the features between two tables.
Return an \code{AnansiWeb} object which contains all three.

\code{weaveWeb()} is for general use and has flexible default settings.

\code{weaveKEGG()} is a wrapper that sets \code{link} to \code{kegg_link()}.
All variants are special cases of \code{weaveWeb()}.
}
\details{
If the \code{link} argument is \code{"none"}, all features will be considered
linked. If one or more \code{data.frame}s, colnames should be as specified in
\code{x} and \code{y}.
}
\examples{
# Setup demo tables, see first vignette.
data(FMT_data)

t1 <- t(FMT_metab)
t2 <- t(FMT_KOs)

# Input objects and syntax:
## define `x` and `y` as characters
web <- weaveWeb(
    x = "ko", y = "cpd", link = kegg_link(),
    tableX = t2, tableY = t1,
    metadata = NULL, verbose = TRUE
)

## define `x` and `y` with a formula
web2 <- weaveWeb(
    x = cpd ~ ko, link = kegg_link(),
    tableX = t2, tableY = t1,
    metadata = NULL, verbose = TRUE
)

identical(web, web2)

# Method for MultiAssayExperiment S4 object
mae <- asMAE(web)
weaveWeb(
    x = mae, link = kegg_link(),
    tableY = "cpd", tableX = "ko",
    force_new = FALSE
)

# Method for TreeSummarizedExperiment S4 object
tse <- asTSE(web)
weaveWeb(
    x = tse, link = kegg_link(),
    tableY = "cpd", tableX = "ko",
    force_new = FALSE
)

}
\seealso{
\itemize{
\item \link{AnansiWeb}: For general constructor and methods.
\item \code{\link[=kegg_link]{kegg_link()}}: For examples of input for link argument.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{getEdgeList}
\alias{getEdgeList}
\title{Get a listof edges}
\usage{
getEdgeList(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
a two-column data.frame that lists the content of each entry in the
input MultiFactor
}
\description{
Get a listof edges
}
\examples{
x <- randomMultiFactor(n_features = 10)
getEdgeList(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{dictionary}
\alias{dictionary}
\alias{dictionary-generic}
\title{Get dictionary}
\usage{
dictionary(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
\code{dictionary} slot of x
}
\description{
Get dictionary
}
\examples{
x <- randomWeb(10)
dictionary(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{FMT_KOs}
\alias{FMT_KOs}
\title{Snippet of the CLR-transformed inferred functional data from the FMT Aging
study.}
\format{
A marix object with 6468 rows, KOs, and 36 columns, samples.
}
\source{
\doi{10.1038/s43587-021-00093-9}
}
\usage{
data("FMT_data", package = "anansi")
}
\description{
Piphillin was used to infer functions from the 16S sequencing
data in terms of KOs. Unfortunately, the Piphillin algorithm is
proprietary and has since been taken down.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{AnansiTale}
\alias{AnansiTale}
\alias{AnansiTale-class}
\title{AnansiTale S7 container class. Not intended for general use.}
\usage{
AnansiTale(
  subject = character(0),
  type = character(0),
  df = integer(0),
  estimates = integer(0),
  f.values = integer(0),
  t.values = integer(0),
  p.values = integer(0)
)
}
\arguments{
\item{subject}{A character that describes the data that was queried.}

\item{type}{A character that describes type of parameter contained in the
\code{estimates} slot. For example r.values for correlations or r.squared
for models.}

\item{df}{a vector of length 2, containing df1 and df2 corresponding to the
F-ratio considered.}

\item{estimates}{A matrix containing the estimates for the parameters named
in the \code{type} slot.}

\item{f.values}{A matrix containing the f-values, for least-squares.}

\item{t.values}{A matrix containing the t-values, for correlations.}

\item{p.values}{A matrix containing the p.values for the parameters named in
the \code{type} slot.}
}
\value{
an \code{AnansiTale}
}
\description{
\code{AnansiTale} is the main container that will hold your
stats output data coming out of the \code{anansi} pipeline.
}
\section{Slots}{

\describe{
\item{\code{subject}}{A character that describes the data that was queried.}

\item{\code{type}}{A character that describes type of parameter contained in the
\code{estimates} slot. For example r.values for correlations or r.squared
for models.}

\item{\code{df}}{a vector of length 2, containing df1 and df2 corresponding to the
F-ratio considered.}

\item{\code{estimates}}{A matrix containing the estimates for the parameters named in
the \code{type} slot.}

\item{\code{f.values}}{A matrix containing the f-values, for least-squares.}

\item{\code{t.values}}{A matrix containing the t-values, for correlations.}

\item{\code{p.values}}{A matrix containing the p.values for the parameters named in
the \code{type} slot.}
}}

\examples{
AnansiTale

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import-utils.R
\name{linkBiobakeryMap}
\alias{linkBiobakeryMap}
\title{make a link data.frame for biobakery mapping files input}
\usage{
linkBiobakeryMap(map)
}
\arguments{
\item{map}{\code{Character}, result from \code{readLines()} on uncompressed humann
mapping files.}
}
\value{
a two-column \code{data.frame} that can be converted into an adjacency
matrix used as input for \code{weaveWeb()}.
}
\description{
make a link data.frame for biobakery mapping files input
}
\examples{
# some dummy input, as a character vector of IDs separated by tabs.
x <- c("x_1\ty_1\ty_2\ty_4", "x_2\ty_1\ty_3", "x_3\ty_2\ty_y")
linkBiobakeryMap(x)
}
\seealso{
\code{\link[=weaveWeb]{weaveWeb()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods-getGraph.R
\name{getGraph}
\alias{getGraph}
\alias{getGraph.list}
\title{Get a graph object.}
\usage{
getGraph(x, ...)
}
\arguments{
\item{x}{input}

\item{...}{additional arguments (currently not used).}
}
\value{
a specified graph object.
}
\description{
Get a graph object.
}
\examples{
# Show methods
getGraph
}
\seealso{
\code{\link{MultiFactor-methods}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anansiCorTestByGroup.R
\name{anansiCorPvalue}
\alias{anansiCorPvalue}
\title{Compute r-statistics for each featureY-featureX pair in the dictionary.
Typically, the main \code{anansi()} function will run this for you.}
\usage{
anansiCorPvalue(web, group.bool, verbose)
}
\arguments{
\item{web}{An \code{AnansiWeb} object, containing two tables with omics data and
a dictionary that links them. See \code{weaveWebFromTables()} for how to
weave a web.}

\item{group.bool}{A categorical or continuous value necessary for
differential correlations. Typically a state or treatment score. If no
argument provided, anansi will let you know and still to regular
correlations according to your dictionary.}

\item{verbose}{A boolean. Toggles whether to print diagnostic information
while running. Useful for debugging errors on large datasets.}
}
\value{
An \code{AnansiTale} result object.
}
\description{
Compute r-statistics for each featureY-featureX pair in the dictionary.
Typically, the main \code{anansi()} function will run this for you.
}
\seealso{
\code{\link[=anansi]{anansi()}} \cr \code{\link[=anansiCorTestByGroup]{anansiCorTestByGroup()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ec2ko}
\alias{ec2ko}
\alias{ec2cpd}
\alias{kegg_link}
\title{Use linking data from the KEGG database.}
\format{
\code{ec2ko}: a \code{data.frame} of two columns, named \code{"ec"}
and \code{"ko"}. The IDs refer to KEGG orthologues. Enzyme commission numbers,
ecs, typically describe reactions captured by them.

\code{ec2cpd}: a \code{data.frame} of two columns, named \code{"ec"} and \code{"cpd"}.
The IDs refer to compounds in the KEGG database. Enzyme commission
numbers, ecs, typically describe reactions either producing or requiring
them.
}
\source{
\code{ec2ko}: Adapted from \url{https://www.genome.jp/kegg/}, using
\code{KEGGREST}. Script to generate available in example.

\code{ec2cpd}: Adapted from \url{https://www.genome.jp/kegg/} using
\code{KEGGREST}. Script to generate available in example.
}
\usage{
data("ec2ko", package = "anansi")

data("ec2cpd", package = "anansi")

kegg_link()
}
\value{
\code{kegg_link()} returns a list containing the two aforementioned
data.frames, \code{ec2cpd} and \code{ec2ko}.
}
\description{
\code{kegg_link()} is a convenience function to return a list containing two
\code{data.frame}s; \code{ec2cpd} and \code{ec2ko}. This will be their most likely use.
\code{ec2cpd} and \code{ec2ko} are two \code{data.frame}s, used to link ko, ecs and cpd
identifiers in the KEGG database.
}
\examples{
kegg_link()

# Generate ec2ko and ec2cpd:
# Don't download during tests. set to `TRUE` to download.
dry_run <- TRUE

if (!dry_run) {
    ec2ko <- KEGGREST::keggLink("ec", "ko")
    ec2ko <- data.frame(
        ec = gsub("ec:", "", x = ec2ko, fixed = TRUE),
        ko = gsub("ko:", "", x = names(ec2ko), fixed = TRUE),
        row.names = NULL
    )

    ec2cpd <- KEGGREST::keggLink("ec", "cpd")
    ec2cpd <- data.frame(
        ec = gsub("ec:", "", x = ec2cpd, fixed = TRUE),
        cpd = gsub("cpd:", "", x = names(ec2cpd), fixed = TRUE),
        row.names = NULL
    )
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{getFeaturePairs}
\alias{getFeaturePairs}
\title{Get a list of all pairs of features}
\usage{
getFeaturePairs(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments for specific methods}
}
\value{
an list of two-column data.frames that represent all feature pairs.
}
\description{
Get a list of all pairs of features
}
\examples{
x <- randomWeb(10)
head(getFeaturePairs(x), 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-coercion.R
\name{anansi-coercion}
\alias{anansi-coercion}
\alias{as.list.anansi::AnansiWeb}
\alias{as.list.anansi::MultiFactor}
\alias{as.data.frame.anansi::AnansiWeb}
\alias{asMAE}
\alias{as.MAE}
\alias{as.MultiAssayExperiment}
\alias{asMultiAssayExperiment}
\alias{asTSE}
\alias{as.TSE}
\alias{as.TreeSummarizedExperiment}
\alias{asTreeSummarizedExperiment}
\title{Coercion functions for anansi}
\usage{
\method{as.list}{`anansi::AnansiWeb`}(x, ...)

\method{as.list}{`anansi::MultiFactor`}(x, ..., use.names = TRUE)

\method{as.data.frame}{`anansi::AnansiWeb`}(x, row.names, optional, ...)

asMAE(x)

asTSE(x)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments (currently not used).}

\item{use.names}{\verb{Logical scalar}, whether output list should contain
character (Default) or integer data frame. If \code{FALSE}, returns
\code{unfactor(x)}.}

\item{row.names, optional}{Not used. See ?base::data.frame}
}
\value{
An object of the desired class.
}
\description{
Coercion functions for anansi
}
\examples{
# AnansiWeb
x <- randomWeb(
    n_samples = 5,
    n_features_x = 4,
    n_features_y = 6
)

as.list(x)
as.data.frame(x)

# AnansiWeb to MultiAssayExperiment
asMAE(x)
asTSE(x)

# MultiFactor
x <- randomMultiFactor(n_types = 3, n_features = 3)
as.list(x, use.names = TRUE)

}
\seealso{
\code{\link[=unfactor]{unfactor()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{FMT_metadata}
\alias{FMT_metadata}
\title{Snippet of the metadata from the FMT Aging study.}
\format{
A data.frame object with 36 rows, samples, and two columns, denoting
sample ID and treatment group, respectively.
}
\source{
\doi{10.1038/s43587-021-00093-9}
}
\usage{
data("FMT_data", package = "anansi")
}
\description{
There were three treatment groups in the study that all received
faecal microbiota transplantation (FMT). Young mice that received FMT
from young mice (Young yFMT), aged mice that received FMT from aged mice
(Aged oFMT) and aged mice that received FMT from young mice (Aged yFMT).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anansiCorTestByGroup.R
\name{anansiCorTestByGroup}
\alias{anansiCorTestByGroup}
\title{Run correlations for all interacting metabolites and functions.}
\usage{
anansiCorTestByGroup(web, group.vec, verbose = TRUE)
}
\arguments{
\item{web}{An \code{AnansiWeb} object, containing two tables with omics data and a
dictionary that links them. See \code{weaveWebFromTables()} for how to weave a
web.}

\item{group.vec}{A character vector denoting group membership. Typically a
state or treatment score.}

\item{verbose}{A boolean. Toggles whether to print diagnostic information
while running. Useful for debugging errors on large datasets.}
}
\value{
a list of \code{AnansiTale} result objects, one for the total dataset and
per group if applicable.
}
\description{
If the \code{groups} argument is suitable, will also run correlation
analysis per group. Typically, the main \code{anansi()} function will run this
for you.
}
\seealso{
\code{\link[=anansi]{anansi()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anansiCorTestByGroup.R
\name{anansiCor}
\alias{anansiCor}
\title{Compute r-statistics for each featureY-featureX pair in the dictionary.
Typically, the main \code{anansi()} function will run this for you.}
\usage{
anansiCor(web, group.bool)
}
\arguments{
\item{web}{An \code{AnansiWeb} object, containing two tables with omics data and a
dictionary that links them. See \code{weaveWebFromTables()} for how to weave
a web.}

\item{group.bool}{A boolean vector used to select which samples should be
included in the correlations.}
}
\value{
A matrix of r-statistics.
}
\description{
Compute r-statistics for each featureY-featureX pair in the dictionary.
Typically, the main \code{anansi()} function will run this for you.
}
\seealso{
\code{\link[=anansi]{anansi()}} \cr \code{\link[=anansiCorTestByGroup]{anansiCorTestByGroup()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anansi-package.R, R/anansi.R
\docType{package}
\name{anansi}
\alias{anansi}
\alias{anansi-package}
\title{anansi: Annotation-based Analysis of Specific Interactions.}
\usage{
anansi(
  web,
  formula,
  groups = NULL,
  metadata = NULL,
  adjust.method = "BH",
  verbose = TRUE,
  return.format = "table",
  ...
)
}
\arguments{
\item{web}{An \code{AnansiWeb} object, containing two tables with 'omics data
and a dictionary that links them. See \code{weaveWebFromTables()} for how to
weave a web.}

\item{formula}{A formula object. Used to assess differential associations.}

\item{groups}{A vector of the column names of categorical values in the
metadata object to control which groups should be assessed for simple
correlations.
If no argument provided, anansi will let you know and still to regular
correlations according to your dictionary.}

\item{metadata}{A vector or data.frame of categorical or continuous value
necessary for differential correlations. Typically a state or treatment.
If no argument provided, anansi will let you know and still to regular
correlations according to your dictionary.}

\item{adjust.method}{Method to adjust p-values for multiple comparisons.
\code{adjust.method = "BH"} is the default value. See \code{p.adjust()} in
the base R \code{stats} package.}

\item{verbose}{\verb{Logical scalar}. Whether to print diagnostic information
(Default: \code{TRUE}).
while running. Useful for debugging errors on large datasets.}

\item{return.format}{\verb{Character scalar}. Should be one of \code{"table"},
\code{"list"}, or \code{"raw"}. Should the output of \code{\link[=anansi]{anansi()}}
respectively be a wide \code{data.frame} of results, a list containing the results
and input, or a list of raw output (used for testing purposes).
convenient use. (Default: \code{"table"})}

\item{...}{additional arguments (currently not used).}
}
\value{
A list of lists containing correlation coefficients, p-values and
q-values for all operations.
}
\description{
The anansi package package provides tools to prepare and facilitate
integrative association analysis between the features of two data sets that
are known to interact.
\subsection{1. Input for \code{anansi()} with \code{\link[=AnansiWeb]{AnansiWeb()}} and \code{\link[=MultiFactor]{MultiFactor()}}}{

\itemize{
\item \link{randomAnansi}, \code{\link[=kegg_link]{kegg_link()}}: Generate example input
\item \link{AnansiWeb}, \link{MultiFactor}: Handle and manipulate
input
}
}

\subsection{2. Output and cross-compatibility}{

\itemize{
\item \code{\link[=getGraph]{getGraph()}}: Compatibility with \link[igraph:aaa-igraph-package]{igraph::igraph}
\item \code{\link[=plotAnansi]{plotAnansi()}}: Plot output in the style of \link[miaViz:miaViz-package]{miaViz::miaViz}
}
}

\subsection{3. Vignettes}{

\itemize{
\item
\href{https://thomazbastiaanssen.github.io/anansi/articles/anansi.html}{1. Getting started with anansi}
\item
\href{https://thomazbastiaanssen.github.io/anansi/articles/adjacency_matrices.html}{2. Adjacency matrices}
\item
\href{https://thomazbastiaanssen.github.io/anansi/articles/association_testing.html}{3. Association testing}
}
}

\subsection{Description: The \code{anansi()} function}{

The main workspider function in the anansi package is called
\code{anansi}. It manages the individual functionalities of anansi, including
correlation analysis, correlation by group and differential associations.
}
}
\examples{
# Load example data:

data(FMT_data)

# Make sure that columns are features and rows are samples.

t1 <- t(FMT_metab)
t2 <- t(FMT_KOs)

# Run anansi pipeline.

web <- weaveWeb(
    cpd ~ ko,
    tableY = t1,
    tableX = t2,
    link = kegg_link()
)

anansi_out <- anansi(
    web = web,
    formula = ~Legend,
    groups = "Legend",
    metadata = FMT_metadata,
    adjust.method = "BH",
    verbose = TRUE
)


library(tidyr)

# Use tidyr to wrangle the correlation r-values to a single column
anansiLong <- anansi_out |>
    pivot_longer(starts_with("All") | contains("FMT")) |>
    separate_wider_delim(
        name,
        delim = "_", names = c("cor_group", "param")
    ) |>
    pivot_wider(names_from = param, values_from = value)

# Only consider interactions where the entire model fits well enough.
library(ggplot2)
anansiLong <- anansiLong[anansiLong$full_p.values < 0.05, ]



ggplot(
    data = anansiLong,
    aes(
        x = r.values,
        y = feature_X,
        fill = cor_group,
        alpha = disjointed_Legend_p.values < 0.05
    )
) +

    # Make a vertical dashed red line at x = 0
    geom_vline(xintercept = 0, linetype = "dashed", colour = "red") +

    # Points show  raw correlation coefficients
    geom_point(shape = 21, size = 3) +

    # facet per compound
    ggforce::facet_col(~feature_Y, space = "free", scales = "free_y") +
    scale_fill_manual(values = c(
        "Young yFMT" = "#2166ac",
        "Aged oFMT" = "#b2182b",
        "Aged yFMT" = "#ef8a62",
        "All" = "gray"
    )) +
    theme_bw()

# Using miaViz style function:

p <- plotAnansi(anansi_out,
    association.type = "disjointed",
    model.var = "Legend",
    fill_by = "group",
    signif.threshold = 0.05,
    x_lab = "Pearson's rho"
)
p <- p +
    scale_fill_manual(values = c(
        "Young yFMT" = "#2166ac",
        "Aged oFMT" = "#b2182b",
        "Aged yFMT" = "#ef8a62",
        "All" = "gray"
    )) +
    theme_bw()

p

}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/thomazbastiaanssen/anansi}
  \item \url{https://thomazbastiaanssen.github.io/anansi}
  \item Report bugs at \url{https://github.com/thomazbastiaanssen/anansi/issues}
}

}
\author{
\strong{Maintainer}: Thomaz Bastiaanssen \email{thomazbastiaanssen@gmail.com} (\href{https://orcid.org/0000-0001-6891-734X}{ORCID})

Authors:
\itemize{
  \item Thomas Quinn \email{contacttomquinn@gmail.com} (\href{https://orcid.org/0000-0003-0286-6329}{ORCID})
  \item Giulio Benedetti \email{giulio.benedetti@utu.fi} (\href{https://orcid.org/0000-0002-8732-7692}{ORCID})
  \item Tuomas Borman \email{tuomas.v.borman@utu.fi} (\href{https://orcid.org/0000-0002-8563-8884}{ORCID})
  \item Leo Lahti \email{leo.lahti@utu.fi} (\href{https://orcid.org/0000-0001-5537-637X}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{pairwiseApply}
\alias{pairwiseApply}
\alias{pairwiseApply-generic}
\title{Apply a function on each pair of features}
\usage{
pairwiseApply(X, ...)
}
\arguments{
\item{X}{input object}

\item{...}{additional arguments}
}
\value{
a list containing the output of applying the function to each feature pair.
See \code{?base::mapply()}
}
\description{
Apply a function on each pair of features
}
\examples{
web <- randomWeb(10)

# For each feature pair, was the value for x higher than the value for y?
pairwise_gt <- pairwiseApply(
    X = web,
    FUN = function(x, y) x > y,
    MoreArgs = NULL, SIMPLIFY = FALSE, USE.NAMES = TRUE
)

head(pairwise_gt)

# Run cor.test() on each pair of features
pairwise_cor <- pairwiseApply(
    X = web,
    FUN = function(x, y) cor.test(x, y),
    MoreArgs = NULL, SIMPLIFY = FALSE, USE.NAMES = TRUE
)

pairwise_cor[1]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MultiFactor-methods.R, R/methods-getGraph.R
\name{MultiFactor-methods}
\alias{MultiFactor-methods}
\alias{levels.anansi::MultiFactor}
\alias{unfactor}
\alias{unfactor,anansi::MultiFactor-method}
\alias{getGraph.anansi::MultiFactor}
\title{Methods for MultiFactor S7 container class}
\arguments{
\item{...}{\verb{i,j} indices specifying elements to extract or replace. Indices
are numeric or character vectors or empty (missing) or NULL. Numeric
values are coerced to integer or whole numbers as by as.integer or for
large values by trunc (and hence truncated towards zero). Character
vectors will be matched to the names of the object.}

\item{value}{Replacement value, typically of same type as that which is to be
replaced.}

\item{exclude}{\code{NULL} or \verb{Named character list} of similar structure as
\code{levels(MultiFactor)}. Which levels to drop from output.}

\item{select}{\code{NULL} or \verb{Named character list} of similar structure as
\code{levels(MultiFactor)}. Which levels to keep in output.}

\item{use.names, ignore.mcols}{For compatibility, not used.}

\item{x}{\code{MultiFactor}}

\item{format}{\verb{Character scalar}, controls output format by package name.
\code{"igraph"} and \code{"graph"} are supported.}
}
\value{
A MultiFactor

a specified graph object.
}
\description{
\describe{
\item{\code{droplevels()} }{\code{droplevels(MultiFactor)} returns a \code{MultiFactor}
with unused levels removed, Analogous to the \code{factor} method.}
}
}
\details{
Only one of \code{select} and \code{exclude} should be provided, as they are
each others complement.
}
\examples{
# Setup
x <- MultiFactor(kegg_link())
x

# Basic properties
dim(x)
dimnames(x)

# Factor-like properties
head(levels(x)$ko)
droplevels(x)
head(unfactor(x)$ec2ko)

# Extract common output formats
getEdgeList(x)

droplevels(x, exclude = list(ko = "K00001"))
droplevels(x, select = list(ko = "K00001"))
# Generate an igraph object
g <- getGraph(x = kegg_link(), format = "igraph")
plot(g)

}
\seealso{
\code{\link[igraph:graph_from_data_frame]{igraph::graph_from_data_frame()}} and
\code{\link[igraph:as_graphnel]{igraph::as_graphnel()}}, which are used under the hood, from
\code{\link[igraph:aaa-igraph-package]{igraph::igraph()}} package.
}
